=head1 NAME

Adult

=head1 DESCRIPTION

Description for B<Adult>

The adult polygon objects contain a list of line objects.  The points that
these are made from belong to the adult polygon.  If the adult polygon is
rotated, all the points and lines belonging to it are moved. Adults also
have colour, this is the same concept as in two colourability! Adjacent
polygons should have different colours. This colour is used to specify
crease assignments for this polygon.

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for Adult
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Adult;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Point;
use Polygon;
use PolygonSet;
use Line;


#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter','Polygon');

#######
# new #
#######
=head1 new
Creates empty adult polygon object
centre of scaling and rotation is passed as an argument that
is converted into a Point object.
This centre is also the vertex of the dual tiling!

  $polygon = new Adult($x,$y,$colour);

=cut

sub new {
    my($class,$x,$y,$colour)=@_;

    my $polygon = new Polygon($x,$y);

    bless $polygon, $class;

    if (! defined $colour){
      die "All adults must have a colour!\n";
    }
    $polygon->colour($colour);

    return($polygon);
}

##########
# colour #
##########

=head1 colour

Returns colour of polygon

Get or set method

$colour=$polygon->colour();
$polygon->colour($colour);

colour can be 1,2 or 3.

=cut

sub colour {
    my $n=@_;

    my ($colour,$polygon);
    if ($n == 2){
      ($polygon,$colour)=@_;
      $polygon->{'colour'}=$colour;
    } elsif ($n == 1){
      ($polygon)=@_;
      $colour = $polygon->{'colour'};
    } else {
      die "Wrong number of args in Adult::colour\n";
    }
    return $colour;
}

#####################
# new_from_old_data #
#####################

=head1 new_from_old_data

Creates polygon object from old style array data.  Adds origin for
scaling and rotation as (0,0).

Example of old style data.
@square=('-50 0 0 50 1',
         '0 50 50 0 -1',
         '50 0 0 -50 1',
         '0 -50 -50 0 -1');

$polygon = new_from_old_data($colour,@shape);

=cut
sub new_from_old_data {
    my $colour=shift @_;
    my (@shape)=@_;

    my $polygon = new Adult(0,0,$colour);
    foreach my $row (@shape){
	my ($x1,$y1,$x2,$y2,$crease)=split (/[ \t\n]+/,shift @_);  # Take out first coordinate

	my $p1 = new Point($x1,$y1,$polygon);
	my $p2 = new Point($x2,$y2,$polygon);
	my $line = new Line($p1,$p2,$crease);
	$polygon->add($line);      # Add point object to polygon object
    }
    return($polygon);
}


##########
# render #
##########
=head1 render

Method to return an XML render stack, that will be used
by View objects.  Also this method returns an array
which has object references in.  These correspond to
tag attributes for objects in XML.  THis means external 
objects to model can refer to objects by these tags and
we can work out which object the external objects mean.

($current_tag,$stack,$tag_ref)=$tiling->render($current_tag,$stack,\@tags);

$stack is XML stack for rendering
$tag_ref is a reference to the array of objects

=cut
sub render {
    my ($self,$current_tag,$stack_ref,$tag_ref) = @_;
    my @tags=@{$tag_ref};

    $current_tag++;
    push (@tags,$self);

    my $centre=$self->centre();
    my ($x,$y)=$centre->xy();

    if ($x eq 'NaNQ'){
	$self->write();
	die "$self NaNQ!";
    }


    my $colour=$self->colour();

    my $open_tag  = "    <polygon tag=\"$current_tag\" type=\"adult\" colour=\"$colour\">\n";
    $current_tag++;
    push (@tags,$centre);

    $open_tag    .= "      <centre tag=\"$current_tag\" x=\"$x\" y=\"$y\"/>\n";

    my $close_tag = "    <polygon/>\n";

    $$stack_ref .= $open_tag;
    foreach my $element ($self->eachline()){
	($current_tag,$stack_ref,$tag_ref)=$element->render($current_tag,$stack_ref,\@tags);
	#warn "Got here! $stack_ref";
	@tags=@{$tag_ref};
    }
    $$stack_ref .= $close_tag;

    return ($current_tag,$stack_ref,\@tags);
}

###########
# reverse #
###########

=head1 reverse

reverses crease assignment of polygon

$polygon->reverse();
=cut

sub reverse {
  my $self = shift @_;
  foreach my $line ($self->eachline()){
    $line->reverse();
  }

  # Edit colour attribute of tile
  my $colour=$self->colour();
  my $new_colour;
  if ($colour eq '1'){
      $new_colour='2';
  } elsif($colour eq '2'){
      $new_colour='1';
  } elsif($colour eq '3a'){
      $new_colour='3b';
  } elsif($colour eq '3b'){
      $new_colour='3a';
  }

  #warn "$self changing colour from $colour to $new_colour";
  $self->colour($new_colour);
}

########
# copy #
########

=head1 copy
Method to copy a polygon object

$new_polygon = $polygon->copy();

=cut
sub copy {
    my $polygon = shift @_;

    my $colour=$polygon->colour();
    my $new_polygon = new Adult($polygon->dual_vertex()->xy(),$colour);

    foreach my $line ($polygon->eachline()){
      $new_polygon->add($line->copy());
    }
    return $new_polygon;
}


#############
# translate #
#############
=head1 translate
Method to translate polygon object

$polygon->translate($x,$y);

=cut
sub translate {
    my ($polygon,$x,$y)=@_;

    my $n = @_;
    if ($n ne 3){
      die "In Adult translate incorrect args [@_]\n";
    }

    # Change origin of scaling and rotation for polygon object
    my ($old_x,$old_y)=$polygon->dual_vertex()->xy();

    $polygon->dual_vertex()->x($old_x+$x);
    $polygon->dual_vertex()->y($old_y+$y);

    # Translate each point of polygon
    foreach my $line ($polygon->eachline()){
      $line->translate($x,$y);
    }
    return $polygon;
}


#########
# scale #
#########
=head1 scale
Method to scale polygon object ($polygon) by a factor ($factor)
will use arguments of x and y as origin of scaling. Otherwise will
use origin of polygon as centre of scaling.

$polygon->scale($factor,$x_origin,$y_origin);
=cut
sub scale {
    my ($polygon,$factor,$x_origin,$y_origin)=@_;      # Get object

    if (! $factor){
	carp "In method Adult::scale, No scale factor given!\n";
    }
    my $len = @_;
    if ($len == 2){
      my ($x,$y) = $polygon->dual_vertex()->xy();    

      # Scale each line by $factor
      foreach my $line ($polygon->eachline()){
	  my $xcache=$x*(1-$factor);
	  my $ycache=$y*(1-$factor);
	  $line->scale($factor,$x,$y,$xcache,$ycache);
      }
    } elsif($len == 4){
      my $x=$x_origin;
      my $y=$y_origin;

      # Scale each line by $factor
      foreach my $line ($polygon->eachline()){
	$line->scale($factor,$x,$y);
      }

      # Also scale centre # I think this is now wrong!
      my ($x_centre,$y_centre)=$polygon->dual_vertex()->xy();
      $x_centre = $x_centre*$factor+$x_origin*(1-$factor);
      $y_centre = $y_centre*$factor+$y_origin*(1-$factor);

      $polygon->dual_vertex()->x($x_centre);
      $polygon->dual_vertex()->y($y_centre);
    } else {
      die "In Adult::scale wrong number of args\n";
    }

}




##########
# rotate #
##########
=head1 rotate
Method to rotate polygon object anticlockwise ($polygon) by an angle ($angle)
using origin of polygon as centre of rotation, unless these are specified

$polygon->rotate($angle,$x,$y);
=cut
sub rotate {
    my ($polygon,$angle,$x_rot,$y_rot)=@_;      # Get object
	      
    my $len = @_;
    if ($len == 2){
      ($x_rot,$y_rot) = $polygon->dual_vertex->xy();
    }

    if ($len == 4){
      # Rotate dual vertex
      my $p1=$polygon->dual_vertex();
      $p1->rotate($angle,$x_rot,$y_rot);
      $polygon->dual_vertex($p1);
    }

    # Do rotation of each line of polygon by $angle
    foreach my $line ($polygon->eachline()){
      $line->rotate($angle,$x_rot,$y_rot);
    }
}

############
# new_ngon #
############
=head1 new_ngon
Creates N-gon polygon object
centre of scaling and rotation as (0,0). $n is number of sides
First side is horizontal and length 100 units.

$polygon = new_ngon($n);

=cut
sub new_ngon {
  my ($n,$colour) = @_;
  my $polygon = new Adult(0,0,$colour);

  if (! $n){
      die "In new_ngon incorrect parameters passed, no number of sides given\n";
  }

  # Find first point
  my $y=50*cos(3.1415927/$n)/sin(3.1415927/$n);
  my $point = new Point(-50,$y,$polygon);

  my @points;
  push (@points,$point);

  # Rotate point
  my $i=1;
  while ($i<$n){
    my $new_point=$point->copy();
    $new_point->rotate((360*$i)/$n);
    push (@points,$new_point);
    #    $polygon->add($new_point);
    $i++;
  }

  my $crease;
  if ($colour eq '1'){
    $crease='1';
  } elsif ($colour eq '2'){
    $crease='-1';
  }

  $i=0;
  while($i<$n){
    ###########################
    # Work out colour of line #
    ###########################
    if ($colour eq '3a'){
      $crease=(-1)**$i;
    } elsif ($colour eq '3b'){
      $crease=(-1)**($i+1);
    }

    my $point1 = $points[$i];
    my $point2 = $points[($i+1)%$n]->copy();

    my $line= new Line($point1,$point2,$crease);
    $polygon->add($line);
    $i++;
  }

  return($polygon);
}

###############
# find_centre #
###############
=head1 find_centre
Finds and adds centre of rotation for polygons.  Only implemented
for triangle so far!

  $polygon->find_centre();

Method from Robert:

But that's too easy, so I'm guessing you want the "center" that is the
intersection of the angle bisectors. There is a very elegant formula for this.
Let A, B, and C be the coordinates of the 3 vertices, and a, b, and c are the
lengths of the opposite sides, as shown here:

      A
    c/ \b
    B---C
      a

A, B, and C are 2-vectors, e.g., A=(x_A, y_A), B={x_B, y_B), C={x_C,y_C).
The lengths of the sides are given by a=sqrt((x_B-x_C)^2+(y_B-y_C)^2), and so
forth.

The 2-vector of the intersection of the angle bisectors is given by

D = (a A + b B + c C)/(a+b+c)

=cut
sub find_centre {
  my $polygon = shift @_;

  # Check has three sides
  my @points=$polygon->each();
  my $n=@points;

  if ($n ne 3){
    die "In Polygon::find_centre polygon is not a triangle has [$n] sides\n";
  }

  my $ax =$points[0]->x();
  my $ay =$points[0]->y();
  my $bx =$points[1]->x();
  my $by =$points[1]->y();
  my $cx =$points[2]->x();
  my $cy =$points[2]->y();

  my $a =sqrt(($bx-$cx)**2+($by-$cy)**2);
  my $b =sqrt(($ax-$cx)**2+($ay-$cy)**2);
  my $c =sqrt(($ax-$bx)**2+($ay-$by)**2);


  my $x_centre =($a*$ax+$b*$bx+$c*$cx)/($a+$b+$c);
  my $y_centre =($a*$ay+$b*$by+$c*$cy)/($a+$b+$c);

  $polygon->dual_vertex()->x($x_centre);
  $polygon->dual_vertex()->y($y_centre);
}

################
# find_centre2 #
################
=head1 find_centre2
Finds and adds centre of rotation for polygons.  Only implemented
for triangle so far! This version adds the centre which is the intersection
of the perpendiculars to the midpoints of the edges.

  $polygon->find_centre2();

=cut
sub find_centre2 {
  my $polygon = shift @_;

  # Check has three sides
  my @points=$polygon->each();
  my $n=@points;

  if ($n > 4){
    die "In Adult::find_centre polygon is not a triangle has [$n] sides\n";
  }

  my $ax=$points[0]->x();
  my $ay=$points[0]->y();
  my $bx=$points[1]->x();
  my $by=$points[1]->y();
  my $cx=$points[2]->x();
  my $cy=$points[2]->y();

  # Gradient of perpendiculars
  my $gradient1;
  if ($ay-$by == 0){
      warn "Sorry trying to divide by zero in Adult::find_centre2 assuming large number\n";
      $gradient1=-99999999999999999;
  } else {
      $gradient1 = -($ax-$bx)/($ay-$by);
  }


  if ($cy-$ay == 0){
    die "Sorry trying to divide by zero in Adult::find_centre2\n";
  }

  my ($gradient2) = -($cx-$ax)/($cy-$ay);
  my($c1)=($ay+$by)/2-$gradient1*($ax+$bx)/2;
  my($c2)=($cy+$ay)/2-$gradient2*($cx+$ax)/2;

  if ($gradient1-$gradient2 == 0){
    print "SERIOUS ERROR GRADIENTS ARE [$gradient1] [$gradient2]\n";
    die "Sorry trying to divide by zero in Adult::find_centre2\n";
  }

  my ($x_centre)= ($c2-$c1)/($gradient1-$gradient2);
  my ($y_centre)= $gradient1*$x_centre+$c1;

  #   print STDERR "Gradient1:$gradient1\tGradient2:$gradient2\nC1:$c1\tC2:$c2\nNew Centre $x_centre,$y_centre\n";
  $polygon->dual_vertex()->x($x_centre);
  $polygon->dual_vertex()->y($y_centre);
}

1;  # says use was ok


__END__


