
#
# Perl Module for Canvas
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Canvas;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use View;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ('Exporter','View');

#######
# new #
#######

=head1 new

Creates new Canvas object. $canvas is a reference to the TK canvas object!

$view = new Canvas($model,$canvas);

=cut

sub new {
    my ($class,$model,$canvas) = @_;

    my $self = new View($model,$canvas);

    bless $self, $class;

    # Attach view object to model so it can notify it of updates.
    $model->attach_view($self);

    return $self;
}

#########
# clear #
#########

=head1 clear

This method will try and clear the current canvas

  $view->clear

=cut

sub clear {
    my $self=shift @_;

    my $canvas=$self->handle();
    my @tags=$canvas->find("all");
    $canvas->delete(@tags);
}

#######################
# canvas_id2model_tag #
#######################

=head1 canvas_id2model_tag

returns reference to canvas_id2model_tag array

    %{$view->canvas_id2model_tag()};

=cut

sub canvas_id2model_tag {
    $self=shift @_;

    my $ref=$self->{'canvas_id2model_tag'};

    if (! defined $ref){
	die "In View::canvas_id2model_tag No reference to return";
    }
    return $ref;
}

##########
# header #
##########

=head1 header

Dummy method no header needed for this format

=cut

sub header {}

##########
# footer #
##########

=head1 footer

Dummy method no header needed for this format

=cut

sub footer {}

########
# body #
########

=head1 body

Print body of PerlTK canvas

    $view->body($canvas);

=cut

sub body {
    my $self   = shift @_;
    my $canvas = shift @_;

    my %args = @_;  # This has other parameters that can be passed to Model
                    # Such as whether to render a light pattern

    my $light;
    if ($args{'light'}){
	$light=$args{'light'};
	#warn "Should try and render light pattern with resolution $args{'light'}";
    }

    if (! $canvas){
	die "No canvas object passed to Canvas::body";
    }

    if ($self->{'model_tag2canvas_id'}){
	# Clear canvas if needed
	$self->clear();
    }

    # Here we are asking Model to render itself, this is not very efficient for
    # multiple views as we will render multiple times! Perhaps Model should cache
    # stack!
    my ($stack) = $self->{'model'}->render(%args);
    my $centres = $self->{'centres'};
    my $fill    = $self->{'fill'};

    my @lines=split /^/,$stack;

    # Set up new attributes in View::Canvas object
    $self->{'canvas_id2model_tag'}={};
    $self->{'model_tag2canvas_id'}={};

    ########################################
    # Should use a proper XML parser here! #
    ########################################


    # Used for -fill option
    my @poly;
    my $create_poly;
    my $polygon_colour;

    foreach my $line (@lines){
	my $tag;
	my $tag_id;
	my $rest;
	my $id;


	my $close_tag=0;

	if ($line =~ /\<(\S+)\s+tag=\"(\d+)\"(.*)/){
	    $tag=$1;
	    $tag_id=$2;
	    $rest=$3;
	} elsif ($line =~ /<\S+\/>/){
	    # End tags are OK, but skip them
	    my $close_tag=1;
	    #next;
	    $tag="";
	    $tag_id="";
	    $rest="";
	} else {
	    die "Cannot parse line $line!";
	}


	if ($tag eq 'line'){
	    if ($rest =~ /crease=\"(\S)\" x1=\"(\S+)\" y1=\"(\S+)\" x2=\"(\S+)\" y2=\"(\S+)\"/){
		if ($1 eq 'm'){
		    $id = $canvas->createLine($2,$3,$4,$5, -fill => 'black',);
		} else {
		    $id = $canvas->createLine($2,$3,$4,$5, -fill => 'black', -width => '2');
		}
	    }
	}

	# If fill option is on keep track of polygon
	if ($fill and not $light){
	    # Collect lines and create polygon in canvas
	    if ($line =~ /<polygon\//){
		# Write out polygon now
		chop $create_poly;
		my @poly=split(/,/,$create_poly);

		my $num_coords=@poly;
		if ($num_coords < 6){
		    # warn "Not enough coordinates to make a polygon! $create_poly\n";
		    next;
		}

		my $fill_colour;
		if ($polygon_colour eq "1"){
		    $fill_colour="light grey";
		} elsif ($polygon_colour eq "2"){
		    $fill_colour="dark grey";
		} elsif ($polygon_colour eq "3a"){
		    $fill_colour="green";
		} elsif ($polygon_colour eq "3b"){
		    $fill_colour="red";
		} else {
		    $fill_colour="black";
		}

		$poly_id = $canvas->createPolygon(@poly,-fill => $fill_colour);

		$polygon_colour=""; # Reset value
		# This is important and complicated!
		$self->{'canvas_id2model_tag'}->{$poly_id}=$poly_tag;
		$self->{'model_tag2canvas_id'}->{$poly_tag}=$poly_id;
	    } elsif ($tag eq 'polygon'){
		if ($rest =~ /colour=\"(\S+)\"/){
		    $polygon_colour=$1;
		}
		$create_poly="";
		@poly="";
		$poly_tag=$tag_id;
	    } elsif ($tag eq 'line'){
		if ($rest =~ /x1=\"(\S+)\" y1=\"(\S+)\"/){
		    $create_poly .="$1,$2,";
		}
	    }
	}


	# Do light pattern renderings
	if ($light and not $fill){
	    if ($tag eq 'polygon'){
		if ($rest =~ /layers=\"(\d+)\" x1=\"(\S+)\" y1=\"(\S+)\" x2=\"(\S+)\" y2=\"(\S+)\" x3=\"(\S+)\" y3=\"(\S+)\" x4=\"(\S+)\" y4=\"(\S+)\"/){
		    my $layers=$1;
		    my $fill_number = 100-(10*$layers);
		    my $fill_colour='grey'.$fill_number;

		    $poly_id = $canvas->createPolygon($2,$3,$4,$5,$6,$7,$8,$9,-fill => $fill_colour);
		}
	    }
	}


	# Show centre of rotation if switched on
	if ($tag eq 'centre'){
	    if ($rest =~ /x=\"(\S+)\" y=\"(\S+)\"/){
		if ($centres){
		    my $radius = 2;
		    $id = $canvas->createOval($1-$radius,$2-$radius,$1+$radius,$2+$radius, -fill => 'black', -tags => "centre");
		} else {
		    my $radius = 0;
		    $id = $canvas->createOval($1-$radius,$2-$radius,$1+$radius,$2+$radius, -outline => 'white' , -tags => "centre");
		}
	    }
	}


	# Set up hashes to allow view to access objects internal to Model object
	if ($id){
	    $self->{'canvas_id2model_tag'}->{$id}=$tag_id;
	    $self->{'model_tag2canvas_id'}->{$tag_id}=$id;
	}
    }
}





1;  # says use was ok

__END__

