=head1 NAME

Dual

=head1 DESCRIPTION

B<Dual> contains a dual object.

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for Dual
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Dual;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Polygon;
use Adult;
use Line;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );


#######
# new #
#######

=head2 new

Creates Dual object. This contains list of pairs of lines.
The first line of each pair forms one of the dual edges.
The second line of the pair forms the edge of the non-dual tiling.
For a correct tessellation these two lines must be orthogonal!

$dual = new Dual($vertex);

The $vertex scalar is the point object that is the centre of the
polygon that this is the dual of!

=cut

sub new {
    my ($class,$vertex)=@_;

    #warn "Creating Dual object for $vertex";
    my $self = {
		"list"   => [],
		"vertex" => $vertex,
	       };

    bless $self, $class;
    return $self;
}


########
# each #
########

=head2 each

Loops over dual and returns list of line objects

@list = $dual->each();

=cut

sub each {
  my($self)=@_;
  my @list=@{$self->{'list'}};

  return @list;
}

=head2 vertex

Returns vertex point object

$vertex = $dual->vertex();

=cut

sub vertex {
  my ($self)=@_;
  my $vertex=$self->{'vertex'};

  return $vertex;
}

########
# cost #
########

=head2 cost

Calculates cost function for Dual object.
Cost will be zero when all line pairs are orthogonal.
This will mean that tiling and dual are orthogonal and
hopefully foldable.

$cost = $dual->cost();

=cut

sub cost {
  my($self)=@_;

  my $cost=0;

  my @list=$self->each();
  while(@list){
    my $line1=shift @list; # Dual line
    my $line2=shift @list; # Non-dual line

    my $angle=$line2->angle($line1);
    if ($angle> 180 or $angle < -180){

	$angle=$angle-180;
#      warn "In Dual::cost code does not deal well with angles like $angle.  Please fix this code to deal with this!\n";
#      $line1->write();
#      $line2->write();
    }
    my $diff_from_orthogonal=sqrt(($angle-90)*($angle-90));

    $cost = $cost + $diff_from_orthogonal;
    #print "ANGLE:$angle VARIANCE:$diff_from_orthogonal COST:$cost\n";
  }
  return $cost;
}

#############
# randomise #
#############

=head2 randomise

Randomly translates Dual object by a number of units specified.  Will
randomly move this much in x and y axes.

$dualset->randomise($fluctuation);

=cut

sub randomise {
  my($self,$size)=@_;

  my $vertex=$self->vertex();

  my $x=$size*(rand(1)-0.5);
  my $y=$size*(rand(1)-0.5);
  $vertex->translate($x,$y);
}


############
# minimise #
############

=head2 minimise

Tries to minimise the cost function for Dual object.
Cost will be zero when all line pairs are orthogonal.
This will mean that tiling and dual are orthogonal and
hopefully foldable. Will return the cost of the final
configuration.

$cost = $dual->minimise();

=cut

sub minimise {
  my ($self)=@_;
  my $vertex=$self->vertex();

  my $oldcost=$self->cost();
  my $newcost;
  # Minimise cost!

  my $debug=0;

  # Loop over all pairs of line objects
  my @pairs=$self->each();

  my $num_pairs=@pairs/2;
  my $i;
  my $total_x;
  my $total_y;

  while (@pairs){
    $i++;
    my $line1=shift @pairs; # This should be dual line
    my $line2=shift @pairs; # This should be edge of original polygon

    if ($debug){
      $line1->write();
      $line2->write();
    }

    my $length1=$line1->length();
    my $length2=$line2->length();
    if ($debug){
      print STDERR "LENGTH1 $length1 LENGTH2 $length2\n";
    }

    my $angle=$line2->angle($line1);


    if ($angle> 180 or $angle < -180){
      die "In Dual::minimise code does not deal well with angles like $angle.  Please fix this code to deal with this!\n";
    }
    #if ($angle>180){$angle=$angle-180};


    my $diff_from_orthogonal=$angle-90;

    $angle=(3.1415927*$diff_from_orthogonal)/180;           # Convert degrees to radians

    my $dist_to_translate=sqrt($length1*sin ($angle)*$length1*sin ($angle));

    # Now use this new length to scale translation

    my ($p1,$p2)=$line2->points();

    my $x_trans=$dist_to_translate*($p1->x()-$p2->x())/($length2*$num_pairs);
    my $y_trans=$dist_to_translate*($p1->y()-$p2->y())/($length2*$num_pairs);

    $total_x += $x_trans;
    $total_y += $y_trans;

    if ($debug){
      print STDERR "ANGLE $angle DIST TO TRANSLATE $dist_to_translate XTRANS $x_trans YTRANS $y_trans\n";
    }

    #$vertex->translate($x_trans,$y_trans);

    #my $newcost=$self->cost();


    #print "Minimise $line1 $line2 $i of $num_pairs $diff_from_orthogonal oldcost $oldcost newcost $newcost\n";

   # if ($newcost>$oldcost){
   #   warn "minimise function failed";
   #   # Undo translation!###
#      $vertex->translate(-$x_trans,-$y_trans);
#    }

  }

  my $factor = 0.1;

#  $vertex->translate($total_x,$total_y);
  $vertex->translate($factor*$total_x,$factor*$total_y);
  $newcost=$self->cost();

  # If move was no good untranslate!
  if ($newcost>$oldcost){
      $vertex->translate(-$factor*$total_x,-$factor*$total_y);
      if ($debug){
	  print STDERR "GOT WORSE! Minimise $num_pairs pair x:$total_x y:$total_y  oldcost $oldcost newcost $newcost\n";
      }
  } else {
      if ($debug){
	  print "Minimise $num_pairs pair x:$total_x y:$total_y  oldcost $oldcost newcost $newcost\n";
      }
  }


  return $newcost;
}


#############
# minimise2 #
#############

=head2 minimise2

Tries to minimise the cost function for Dual object. Does so by random translation and only keeps low cost ones!
Cost will be zero when all line pairs are orthogonal.
This will mean that tiling and dual are orthogonal and
hopefully foldable. Will return the cost of the final
configuration.

$cost = $dual->minimise2($step);

=cut

sub minimise2 {
  my ($self,$factor)=@_;

  if (! $factor){
      my $factor=0.1;
  }
  my $debug=0;
  my $oldcost=$self->cost();

  my $vertex=$self->vertex();

  # Random translation
  my $x=$factor*(rand(1)-0.5);
  my $y=$factor*(rand(1)-0.5);

  if ($debug){
      warn "RANDOM TRANSLATION OF $x,$y";
  }
  $vertex->translate($x,$y);
  
  my $newcost=$self->cost();

  # If move was no good untranslate! But have a small probability of keeping bad moves!
  my $probability=0.01; # Chance of keeping the result if worse cost.
  if ($newcost>$oldcost and (rand(1)>$probability)){
      $vertex->translate(-$x,-$y);
      return $oldcost;
      if ($debug){
	  #print STDERR "GOT WORSE! Minimise $num_pairs pair x:$total_x y:$total_y  oldcost $oldcost newcost $newcost\n";
      }
  } else {
      #if ($newcost>$oldcost){
	 # warn "ACCEPTED POOR TRANSLATION OLD:$oldcost NEW:$newcost";
#      }
      if ($debug){
	  #print "Minimise $num_pairs pair x:$total_x y:$total_y  oldcost $oldcost newcost $newcost\n";
      }
  }
  return $newcost;
}

#############
# minimise3 #
#############

=head2 minimise3

Tries to minimise the cost function for Dual object. 
Does this by making two dual ediges orthogonal to tile edges
and looking for their intersect.

Will return the cost of the final configuration.

$cost = $dual->minimise3();

=cut

sub minimise3 {
  my ($self,$factor)=@_;
  my $vertex=$self->vertex();
  my $oldcost=$self->cost();

  # Loop over all pairs of line objects
  my @pairs=$self->each();

  my $num_pairs=@pairs/2;
  my $i=0;


  # Make an array of all the orthoganal lines into a Dual tile.
  my @orthogonal_lines;
  while (@pairs){
      $i++;
      my $line1=shift @pairs; # This should be dual line
      my $line2=shift @pairs; # This should be edge of original polygon   

      #warn "LINE $line1";

      # Find point that is not the dual
      my ($p1,$p2)=$line1->points();
      #warn "$p1 $p2";

      my $point;
      if ($vertex eq $p1){
	  $point=$p2;
      } elsif ($vertex eq $p2) {
	  $point=$p1;
      } else {
	  die "None of the points in this line seem to be the centre of rotation!";
      }
      my $new_line =  Line::orthogonal($line2,$point);
      push (@orthogonal_lines, $new_line);
  }

  # Try all pairs of elements from @orthogonal_lines and keep lowest cost pair
  my $length=@orthogonal_lines;

  my ($m,$n,$best_x,$best_y);
  my $best_cost=1000000;
  for ($m=0; $m<$length; $m++){
      for ($n=$m+1; $n<$length; $n++){
	  my $testline1=$orthogonal_lines[$m];
	  my $testline2=$orthogonal_lines[$n];

	  if ($testline1->close_to_parallel($testline2)){
	      #warn "Lines close to parrallel";
	      next;
	  }

	  my $point=$testline1->intersect($testline2,'please');
	  if ($point == 0){
	      die "No intersection for two lines. Try another pair";
	  }

	  my $x=$point->x()-$vertex->x();
	  my $y=$point->y()-$vertex->y();

	  if ($x>10000 or $y>10000){
	      
	      warn "FOO";
	      $testline1->write();
	      $testline2->write();
	      my $a=$testline1->gradient();
	      my $b=$testline2->gradient();

	      $point->write();
	      die "Sorry your translations are getting stupid! X $x Y $y GRAD $a $b";
	  }


	  if (! defined $x or ! defined $y){
	      die "X $x Y $y not defined!";
	  }


	  $vertex->translate($x,$y);
	  my $newcost=$self->cost();

	  my $probability=0.4; # Chance of ignoring the result even though lower cost.
	  # Would be good to modulate this based on how close the value was!
	  if ($newcost<$best_cost and (rand(1)>$probability)){
	      $best_cost=$newcost;
	      $best_x=$x;
	      $best_y=$y;
	  }
	  $vertex->translate(-$x,-$y);
      }
  }

  if ((defined $best_x and $best_x>10000) or (defined $best_x and $best_y>10000)){
      die "Sorry your translations are getting stupid! X $best_x Y $best_y";
  }

  # Carry out best translation
  if ($best_x or $best_y){
      $vertex->translate($best_x,$best_y);
  }
  my $finalcost=$self->cost();
  #warn "NEW:$finalcost OLD:$oldcost BEST: $best_cost X;$best_x Y:$best_y";
  return $finalcost;
}


#######
# add #
#######

=head2 add

Add pair of lines to Dual object

    $polygonset->add($line1,$line2);

=cut

sub add {
  my $self = shift @_;
  push(@{$self->{'list'}},@_);

  my %hash;
  # Test if any line occurs twice
  my @list = $self->each();
  foreach my $thing (@list){
    if ($hash{$thing}){
      die "$thing occurs twice in your Dual object!\n";
    } else {
      $hash{$thing}=1;
    }
  }

}


#########
# write #
#########

=head2 write

Debugging method: Write dual to STDERR

$dual->write();

=cut

sub write {
    my ($self)= @_;

    print STDERR "Writing Dual $self\n";
    foreach my $object ($self->each()){
	$object->write();
    }
    print STDERR "\n";

    my $cost=$self->cost();
    print STDERR "COST: $cost\n";
}

1;  # says use was ok
__END__

