=head1 NAME

DualSet

=head1 DESCRIPTION

B<DualSet> contains a collection of dual objects.

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for DualSet
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package DualSet;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Polygon;
use Adult;
use Line;
use Dual;
use List::Util qw(shuffle); # For randomising an array!

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );


#######
# new #
#######

=head2 new

Creates DualSet object. This contains list of Dual objects.

$dualset = new DualSet();

=cut

sub new {
    my ($class)=@_;
    my $self = {"dual" => []};

    bless $self, $class;
    return $self;
}


#############
# randomise #
#############

=head2 randomise

Randomly translates DualSet object by a number of units specified.  Will
randomly move this much in x and y axes.

$dualset->randomise($fluctuation);

=cut

sub randomise {
  my($self,$size)=@_;
  warn "Randomising Dual Set";

  foreach my $object ($self->each()){
      $object->randomise($size);
  }
}


############
# populate #
############

=head2 populate

Populates DualSet object from a given tiling object

$dualset->populate($tiling);

=cut

sub populate {
  my($self,$tiling)=@_;

  # Get list of all polygons in tiling
  warn "Populating Dual Set";

  my @polygons;
  foreach my $set ($tiling->each()){
    foreach my $polygon ($set->each()){
      push (@polygons,$polygon);
    }
  }

  foreach my $p1 (@polygons){
    # Make new Dual object for this polygon

    my $dual_vertex=$p1->dual_vertex();
    if (! $dual_vertex){die "In populate no dual vertex for polygon $p1";}
    my $dual = new Dual($dual_vertex);
    $self->add($dual);

    foreach my $p2 (@polygons){      
      if ($p1 eq $p2){
	next;
      }

      foreach my $l1 ($p1->eachline()){
	foreach my $l2 ($p2->eachline()){
	  if ($l1->compare($l2,10)){ # If lines are the same!
	    
	    # Make dual object
	    my $newline= new Line($p1->{'centre'},$p2->{'centre'});
	    $dual->add($newline,$l1);
	  }
	}
      }
    }
    #$dual->write();
  }
}

########
# each #
########

=head2 each

Loops over dualset and returns a list of dual objects

@duals = $dualset->each();

=cut

sub each {
  my($self)=@_;
  my @list=@{$self->{'dual'}};
  return @list;
}


#######
# add #
#######

=head2 add

Add duals to Set object

    $polygonset->add($dual1..$dualN);

=cut

sub add {
  my $self = shift @_;
  push(@{$self->{'dual'}},@_);
}

############
# minimise #
############

=head2 minimise

Minimise the overall cost of all Dual objects in the dualset

$cost=$dualset->minimise();

returns the sum of costs for all dual objects

=cut

sub minimise {
    my ($self,$step)=@_;

    if (! $step){
	$step=0.1;
    }
#    warn "Trying to minimise over whole tiling with step $step";
    my $total_cost=0;
    foreach my $dual ($self->each()){
#    foreach my $dual (shuffle($self->each())){
	#my $cost=$dual->minimise();
	#my $cost=$dual->minimise2($step);
	my $cost=$dual->minimise3();
	$total_cost=$total_cost+$cost;
    }
    return $total_cost;
}


#########
# write #
#########

=head2 write

Debugging method: Writedualset to STDERR

$dualset->write();

=cut

sub write {
    my ($self)= @_;      # Get object

    print "Writing DualSet $self\n";
    foreach my $object ($self->each()){
	$object->write();
    }
}

1;  # says use was ok
__END__
