=head1 NAME

SessionList

=head1 DESCRIPTION

The SessionList object tracks what user sessions are being used in
the gui.  This object is a container os Session objects. External
code should only need to interact with the SessionList object.

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for SessionList
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package SessionList;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Gui::Session;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

#######
# new #
#######

=head2 new

Create new sessionlist object.

    $sessionlist = new SessionList();

=cut

sub new {
    my ($class)=@_;

    my $self = {
	'list' => [],
	'anonymous_session_number' => '1',
    };

    bless $self, $class;
    return $self;
}

#######
# add #
#######

=head2 add

Add a new Session to SessionList

    $session_name=$sessionlist->add($model,$view,'optional name');

=cut

sub add {
    my ($self,$model,$view,$name)=@_;

    # Assign a session number
    if (! $name){
	my $session_number=$self->{'anonymous_session_number'}++;
	$name='session '.$session_number;
    }

    # If name is used check it hasn't been used before
    if ($name and ! $self->get($name)){

    } elsif ($name eq $self->get($name)->name()){
	if ($name =~ /(.*) \#(\d+)/){
	    my $stem=$1;
	    my $i=$2;
	    $name=$stem." \#".$i++;
	} else {
	    $name.=" \#2";
	}
    } 

    # Create new session
    my $session=new Session($model,$view,$name);
    push(@{$self->{'list'}},$session);
    return $name;
}

##########
# delete #
##########

=head2 delete

Delete a Session from SessionList

    $sessionlist->delete('name');

=cut

sub delete {
    my ($self,$name)=@_;

    if (! $name){
	die "In SessionList::delete no name passed in";
    }

    die "Oh no Alex is soo lazy he can't be bothered to implement SessionList::delete";
}

########
# each #
########

=head2 each

Returns a list of session object references

    @sessions=$sessionlist->each();

=cut

sub each {
    my ($self)=@_;
    my @session_list=@{$self->{'list'}};

    return @session_list;
}

############
# eachname #
############

=head2 eachname

Returns a list of session names

    @sessions=$sessionlist->each();

=cut

sub eachname {
    my ($self)=@_;
    my @session_list=@{$self->{'list'}};

    my @sessions;
    foreach my $session (@session_list){
	warn "SESSION $session";
	push (@sessions,$session->name());
    }
    return @sessions;
}

#######
# get #
#######

=head2 get

Returns a Session object given a session name

    my $session=$sessionlist->get($name);

Should not be used externally. Will return 0 if no matching
session name is found.

=cut

sub get {
    my ($self,$name)=@_;

    if (! $name){
	croak "In SessionList::get no name arg passed";
    }

    my @sessions=$self->each();
    foreach my $session (@sessions){
	if ($session->name() eq $name){
	    return $session;
	}
    }
    return 0;
}


1;  # says use was ok
__END__

