=head1 NAME

Line

=head1 DESCRIPTION

The Line object contains a pair of point objects
as well as a crease attribute

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for Line
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Line;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Point;
use Adult;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

#######
# new #
#######

=head2 new

Create new line object. Object contains two points. Has
a crease attribute that can take value of 1, -1 or 0.

    $line = new Line($point1,$point2);

optionally give crease attribute

    $line = new Line($point1,$point2,$crease);

=cut

sub new {
    my ($class,$point1,$point2,$crease)=@_;
    my $line = {
	"crease" => $crease,
	"p1"     => $point1,
	"p2"     => $point2,
    };

    if ($point1 == $point2){
	confess "In Line::new points are identical!\n";
    }

    # Should check that points exist!
    if (! $point1 or ! $point2){
	die "In Line::new Point does not exist!\n";
    }

    bless $line, $class;   # Tag object with package name
    return $line;          # Return object
}


##############
# orthogonal #
##############

=head2 orthogonal

Creates a new line object that is orthogonal to line and also
passes through point.

    $new_line = orthogonal Line($line,$point);

=cut

sub orthogonal {
  my ($line,$point) = @_;

  my $gradient1=$line->gradient();
  my $intercept1=$line->intercept();

  # Find gradient of orthogonal line
  my $gradient2=-1/($gradient1);
  my $intercept2=($point->y())-$gradient2*($point->x());

  #warn "SADFASD $gradient1 $intercept1 $gradient2 $intercept2";


  # Now find point at which orthogonal line would intersect with original line
  my $x=($intercept1-$intercept2)/($gradient2-$gradient1);
  my $y=$gradient2*$x+$intercept2;

  # Make intersection point
  my $intersect_point = new Point($x,$y);

  my $orthogonal_line = new Line($point->copy(),$intersect_point,1);

  # As a final check find angle between two lines!
  my $angle = $line->angle($orthogonal_line);
  #warn "Angle between orthogonal lines is $angle\n";
  #warn "Line 1";
  #$line->write();
  #warn "Orth Line";
  #$orthogonal_line->write();
  #$point->write();
  return $orthogonal_line;
}


##########
# points #
##########

=head2 points

Gets point objects for line

    ($point1,$point2) = $line->points();

=cut

sub points {
    my $line= shift @_;
    my $p1=$line->{'p1'};
    my $p2=$line->{'p2'};
    return ($p1,$p2);
}

##########
# length #
##########

=head2 length

Get length of line

    $length=$line->length();

=cut

sub length {
    my $line=shift @_;
    my ($p1,$p2)=$line->points();
    my $length=$p1->distance($p2);
    return $length;
}

#########
# label #
#########

=head2 label

Label line object for debugging purposes only. Label will be displayed in output.

    $point->label('some text');

=cut

sub label {
    my $len=@_;
    if ($len == 2){
	my $self  = shift @_;
	my $text  = shift @_;

	$self->{'label'}=$text; 
	return $text;
    } else {
	my $self   = shift @_;
	my $text=$self->{'label'};
	return $text;
    }
    die"Should never get here\n!";
}


########
# swap #
########

=head2 swap

Swaps order of point objects in line.

    $line->swap();

=cut

sub swap {
    my $line= shift @_;

    my $p1=$line->{'p1'};
    my $p2=$line->{'p2'};

    $line->{'p1'}=$p2;
    $line->{'p2'}=$p1;
}

##########
# crease #
##########

=head2 crease

Gets or sets crease assignment for line object

    $crease = $line->crease();

or

    $line->crease($crease);

$crease can be one of 1,0,-1. Cannot currently set crease to 0.
    
=cut

sub crease {
  my $len=@_;
  if ($len == 2){
    my $line   = shift @_;
    my $crease = shift @_;

    $line->{'crease'}=$crease; 
    return $crease;
  } else {
    my $line   = shift @_;
    if (! defined $line->{'crease'}){
      die "Undefined value of crease at Line::crease\n";
    } else {
      my $crease=$line->{'crease'};
      return $crease;
    }
  }
  die"Should never get here\n!";
}

###########
# reverse #
###########

=head2 reverse

Change parity of crease, returns final parity of crease.

    $line->reverse();

=cut

sub reverse {
  my $line = shift @_;
  my $crease = $line->crease();
  $crease = $line->crease($crease*(-1));
  return $crease;
}

########
# copy #
########

=head2 copy

copy line object, keeps same attributes.

    $new_line = $line->copy();

=cut

sub copy {
    my ($line)=@_;

    if (! defined $line->{'crease'}){
      die "Serious error crease not defined here!\n";
    }

    my $new_line = new Line($line->{'p1'}->copy(),$line->{'p2'}->copy(),$line->crease());
    return $new_line;
}

##########
# rotate #
##########

=head2 rotate

Takes line and rotates about point x,y by $angle in degrees.
x,y will default to the origin.

    $point->rotate($angle,$x,$y);

=cut

sub rotate {
    my ($line,$angle,$x,$y)=@_;

    $line->{'p1'}->rotate($angle,$x,$y);
    $line->{'p2'}->rotate($angle,$x,$y);
    return $line;
  }

#############
# translate #
#############

=head2 translate

Takes line and translates by $x and $y

    $line->translate($x,$y);

=cut

sub translate {
    my ($line,$x,$y)=@_;

    my $n=@_;
    if ($n ne 3){
	die "In Line::translate incorrect number args [@_]\n";
    }

    $line->{'p1'}->translate($x,$y);
    $line->{'p2'}->translate($x,$y);
    return $line;
}

############
# gradient #
############

=head2 gradient

Takes lines and works out gradient. Returns large
value if line is vertical

    $line1->gradient();

=cut

sub gradient {
  my $self = shift @_;

  # calculate gradient
  my ($p1,$p2)=$self->points();

  my $gradient;
  if ($p1->x() == $p2->x()){
    $gradient='99999999999999';
  } else {
    $gradient=($p2->y()-$p1->y())/($p2->x()-$p1->x());
  }
  return $gradient;
}


#########
# angle #
#########

=head2 angle

Calculates and between two lines in degrees, angle is measured from
line1 to line2 in a clockwise direction.

If no second line is specified it will use a vertical line.
    $line1->angle($line2);

For some reason I only return a positive value in the range of 0 to 360!

=cut

sub angle {
  my ($line1,$line2) = @_;

  my $p1=$line1->{'p1'};
  my $p2=$line1->{'p2'};
  my ($x1,$y1)=$p1->xy();
  my ($x2,$y2)=$p2->xy();

  my ($p3,$p4,$x3,$y3,$x4,$y4);
  if ($line2){
      $p3=$line2->{'p1'};
      $p4=$line2->{'p2'};
      ($x3,$y3)=$p3->xy();
      ($x4,$y4)=$p4->xy();
  } else {
      $x3=0;
      $y3=0;
      $x4=0;
      $y4=100;

  }

  my $pi=3.1415927;
  my $theta1=180/$pi*atan2($y2-$y1,$x2-$x1);

  if ($theta1<0){
      $theta1=$theta1+360;
  }
  my $theta2=180/$pi*atan2($y4-$y3,$x4-$x3);

  if ($theta2<0){
      $theta2=$theta2+360;
  }

  my $diff = ($theta1-$theta2);
  if ($diff<0){
      $diff=$diff+360;
  }
  #print STDERR "1 $theta1 2 $theta2 dif $diff\n";
  return $diff;
}

############
# midpoint #
############

=head2 midpoint

Takes line and returns midpoint as a point object

  my $midpoint=$line->midpoint();

=cut

sub midpoint {
  my $line = shift @_;

  my $p1=$line->{'p1'};
  my $p2=$line->{'p2'};

  my $mid_x=($p1->x()+$p2->x())/2;
  my $mid_y=($p1->y()+$p2->y())/2;

  my $midpoint=new Point($mid_x,$mid_y);
  return $midpoint;
}

#############
# intercept #
#############

=head2 intercept

Takes line and works out intercept.

    $line1->intercept();

=cut

sub intercept {
  my $self = shift @_;

  my $gradient=$self->gradient(); # Could pass in as arg to save compute.

  my $p=$self->{'p1'};

  my $intercept=$p->y()-$gradient*$p->x();
  return $intercept;
}

#############
# intersect #
#############

=head2 intersect

Takes two lines and works out intersection point. 

Returns intersection point if an intersection.
Returns 0 if no intersection. 
Returns -1 if intersection is close to an endpoint and might be incorrect
due to rounding errors and slop,  etc! Or if two lines are very close to
parallel and intercepts are similar

    $point=$line1->intersect($line2);

Can force this method to return a point of intersection even if the lines do
not truly overlap using the following incantation.

    $point=$line1->intersect($line2,'please');


=cut

sub intersect {
  my ($line1,$line2,$please) = @_;

  my $denominator=$line1->gradient()-$line2->gradient();
  my $x;
  if (! $denominator){
      # Lines are parallel therefore assuming no intersection
      warn "In Line::intersect making assumption that parallel lines don't intersect!\n";
      return 0;
  } else {
      $x=($line2->intercept()-$line1->intercept())/($denominator);
    #  warn "X $x $g1 $g2 $i1 $i2";
#      $x=($line2->intercept()-$line1->intercept())/($line1->gradient()-$line2->gradient());
  }

  my $y=$line1->gradient()*$x+$line1->intercept();

  # Make intersection point
  my $dummy;
  my $intersect_point=new Point($x,$y,$dummy);

  if (defined $please and $please eq 'please'){
      return $intersect_point;
  }

#  print STDERR "So I think intersection is at $x $y\n";

#  my $slop=0.00001;  # This is amount out of scope a point can be!  This helps
                     # with vertical lines!
  my $slop=0;


  my $length1=$line1->length();
  my $length2=$line2->length();

  #print STDERR "Lengths $length1 $length2\n";
  #$line1->write();
  #$intersect_point->write();
  #sleep 1;
  #######################################################################
  # Use fact that if distance from either point to intersect is greater #
  # than length of line then it is out of scope!                        #
  #######################################################################

  my $p1=$line1->{'p1'};
  my $p2=$line1->{'p2'};

  my $dist1=$p1->distance($intersect_point);
  my $dist2=$p2->distance($intersect_point);

  my $p3=$line2->{'p1'};
  my $p4=$line2->{'p2'};

  my $dist3=$p3->distance($intersect_point);
  my $dist4=$p4->distance($intersect_point);

  #print STDERR "distances $dist1 $dist2 $dist3 $dist4\n";

  my $in_scope;
  if ($dist1<=$length1 and $dist2<=$length1 and $dist3<=$length2 and $dist4 <= $length2){
      #print STDERR "By new criterion in scope!\n";
      $in_scope=1;
  } else {
      #print STDERR "distances $dist1 $dist2 $dist3 $dist4\n";
      #print STDERR "By new criterion out of scope!\n";
      $in_scope=0;
  }


  if ($in_scope){
      my $polygon = new Adult(0,0,1); # Dummy adult
      my $point=new Point($x,$y,$polygon);

      # Check that intersection point is not too close to any endpoint
      my $distance=0.0000001;
      if (
	  $point->distance($p1)<$distance or 
	  $point->distance($p2)<$distance or 
	  $point->distance($p3)<$distance or 
	  $point->distance($p4)<$distance
	 ){
	  warn "In Line::intersect intersection is marginal\n";
	  return (-1);
      }

      return $point;

      die "I think this code is unreachable!";
      my $diffgrad=($line1->gradient()-$line2->gradient())**2;
      my $c1=$line1->intercept();
      my $c2=$line2->intercept();
      my $diffc=(($c1-$c2)**2)/($c1+$c2);
      if ($diffgrad<0.01 and $diffc < 0.1){
	  warn "Hmm intersecting lines are almost parallel! with C1 $c1, C2 $c2 $diffc\n";
	  return (-1);
      }
      return $point;
  } else {return 0} # No intersection

  die "Unreachable code\n";
}

#########
# scale #
#########

=head2 scale

Takes line and scales by $factor

    $line->scale($factor,$x,$y);

=cut

sub scale {
    my ($line,$factor,$x,$y,$xcache,$ycache)=@_;
    my $p1=$line->{'p1'};
    my $p2=$line->{'p2'};

    $p1->scale($factor,$x,$y,$xcache,$ycache);
    $p2->scale($factor,$x,$y,$xcache,$ycache);
}

##########
# render #
##########
=head1 render

Method to return an XML render stack, that will be used
by View objects.  Also this method returns an array
which has object references in.  These correspond to
tag attributes for objects in XML.  THis means external 
objects to model can refer to objects by these tags and
we can work out which object the external objects mean.

($stack,$tag_ref)=$tiling->render($current_tag,$stack_ref,$tag_ref);

$stack is XML stack for rendering
$tag_ref is a reference to the array of objects

=cut
sub render {
    my ($self,$current_tag,$stack_ref,$tag_ref) = @_;

    $current_tag++;
    push (@$tag_ref,$self);

    my ($p1,$p2)=$self->points();

    my $precision=2;   # Number of digits after decimal point to include

    my ($x1,$y1)=$p1->xy();
    my ($x2,$y2)=$p2->xy();

    $x1=sprintf "%.${precision}f",$x1;
    $y1=sprintf "%.${precision}f",$y1;
    $x2=sprintf "%.${precision}f",$x2;
    $y2=sprintf "%.${precision}f",$y2;

    my $crease=$self->crease();
    my $crease_tag;
    if ($crease == 1){
	$crease_tag="crease=\"m\"";
    } elsif ($crease == -1){
	$crease_tag="crease=\"v\"";
    } else {
	$crease_tag="crease=\"n\"";
    }

    my $open_tag  = "      <line tag=\"$current_tag\" $crease_tag x1=\"$x1\" y1=\"$y1\" x2=\"$x2\" y2=\"$y2\"/>\n";
    $$stack_ref .= $open_tag;

    return ($current_tag,$stack_ref,$tag_ref);
}

#########
# write #
#########

=head2 write

Debugging method: prints line object data to STDERR

    $line-> write();

=cut

sub write {
    my $line= shift @_;        # Get object

    my ($p1,$p2)=$line->points();

    if (defined $line->{'crease'}){
      print STDERR "$line Crease:",$line->{'crease'};
    } else {
      print STDERR "$line";
    }
    my ($x1,$y1)=$p1->xy();
    my ($x2,$y2)=$p2->xy();

    printf STDERR " (%2.2f,%2.2f) (%2.2f,%2.2f)\n",$x1,$y1,$x2,$y2;
}

###########
# compare #
###########

=head2 compare

Method to say if lines are approximately the same. Order of points does
no matter. Returns 1 if lines are the same and in same orientation, -1 
if they are same but in different orientation, or 0 if they are different.

    $line1->compare($line2);

optional

    $line1->compare($line2,$min_dist);

where min_dist says how close points have to be together. If no min_dist
is given will default to distance of 1 unit.

=cut

sub compare {
    my $line1= shift @_;
    my $line2= shift @_;
    my $min_dist=shift @_;

    if (! $min_dist){
      $min_dist=1; # How far apart points can be to be considered the same
    }
    
    my ($p1,$p2)=$line1->points();
    my ($p3,$p4)=$line2->points();

    if ($p1->distance($p3)<$min_dist and $p2->distance($p4)<$min_dist){
      return 1;
    } elsif ($p1->distance($p4)<$min_dist and $p2->distance($p3)<$min_dist){
      return -1;
    } else {
      return 0;
    }
}

#####################
# close_to_parallel #
#####################

=head2 close_to_parallel

Method to say if two lines are approximately parrallel.
Returns 1 if lines are nearly parrallel or 0 if they are different.

    $line1->close_to_parallel($line2);

=cut

sub close_to_parallel {
    my $line1= shift @_;
    my $line2= shift @_;

    my $debug=0;
    my $min_diff=0.1;
    my $gradient1=$line1->gradient();
    my $gradient2=$line2->gradient();

    # Works well when lines are close to horizontal!
    if ($gradient1<0.0001 and $gradient1>-0.0001 and $gradient2<0.0001 and $gradient2>-0.0001){
	return 1;
    }

    # Works well except when lines are close to horizontal!
    my $fraction=$gradient1/$gradient2;
    if ($fraction<(1+$min_diff) and $fraction>(1-$min_diff)){
	if ($debug){
	    warn "GRADIENT1 $gradient1 GRADIENT2 $gradient2 FRACTION OF GRADIENTS $fraction";
	    $line1->write();
	    $line2->write();
	    warn "Gradients are too close and therefore too parrallel";
	}
	return 1;
    } else {
	return 0;
    }
}



1;  # says use was ok
__END__

