=head1 NAME

Model

=head1 DESCRIPTION

The Model object provides an interface to all the objects that model
lines polygons, sets and tilings.

These objects should ONLY be accessed by external programs through this interface!

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for Parameters
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Model;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use Tiling;
use Point;
use Polygon;
use Adult;
use Baby;
use Line;
use DualSet;
use PolygonSet;
use Patch;
use strict;
#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

#######
# new #
#######

=head1 new

Create new Model object

$model=new Model($geometry,$type,other arguments);

Other arguments can be one of 

  bbox_scale    => value   # This is really a View issue!
  tower_n       => integer value
  tower_array   => comma seperated value of decimal fractions e.g 0.2,0.3
  random        => value
  max_iteration => value

$type can be one of 'twist' or 'weave'

=cut
sub new {
  my $class = shift @_;
  my $geometry = shift @_;
  my $type = shift @_;

  my %args=@_;

  # Need to add code here to create new patch then tessellate it and store patch so
  # we can untessellate.

  my $tiling = new Tiling($geometry,$type,%args);
  $tiling->tessellate(%args);     # Populate $tiling object

  if ($args{'random'} and $args{'max_iteration'}){
      my $dualset=new DualSet();
      $dualset->populate($tiling);
      $dualset->randomise($args{'random'});

      my $cost=1000;
      my $oldcost=1000;

      my $iter=0;
      my $max_iter=$args{'max_iteration'};
      while ($cost>0.1 and $iter<$max_iter){
	  my $step=$oldcost/10;
	  if ($step>1){$step=1;}

	  $cost=$dualset->minimise($step);
	  warn "OVERALL COST: $cost ITERATION $iter STEP $step\n";
	  $oldcost=$cost;
	  $iter++;
      }
  }


  my $patch=$tiling->patch();
  if (! defined $patch){
      warn "This geometry appears to not have a patch";
  }


                               # @_ is only set for geometry 16!  
  my $self = {
	       "tiling"          => $tiling,
	       "crease_pattern"  => '', # Will store a copy of the original tiling
                                        # when vertexifying. This allows us to unorigamify
	       "patch"           => $patch, # Store a copy of the patch used to generate the original tiling.
                                        # This will allow us to untessellate
	       "geometry"        => $geometry,
	       "state"           => 'none', # Stores whether tiling, folded, unfolded etc
	       "type"            => $type,
               "view_list"       => [],
               "tag_list"        => [], # Contains mapping from tag list to object references
	      };

  
  bless $self, $class;
  $self->state('tiling');
  return $self;
}


#########
# state #
#########

=head1 state

get/set state

    $state=$model->state();

or 
    $model->state($state);

=cut

sub state {
  my ($self,$state);

  ################
  # Valid states #
  ################

  my %valid_state=("none","1",     # Temporary state at beginning
		   "patch","1",    # Tiling has not been generated from patch
		   "tiling","1",   # Tiling has not been vertexified
		   "unfolded","1", # Tiling has been vertexified
		   "folded","1",   # Tiling has negative angles
	      );

  my $n=@_;

  if ($n == 2){
    ($self,$state)=@_;
    $self->{'state'}=$state;
  } elsif ($n == 1){
    my $self= shift @_;
    $state=$self->{'state'};
  }

  if (! $valid_state{$state}){
    die "Sorry in Model::state your state [$state] is not a valid state!\n";
  }
  return $state;
}


##############
# bbox_scale #
##############

=head1 bbox_scale

Method to scale the bounding box of the Model. This method should be deprecated!

$model->bbox_scale('10');

=cut

sub  bbox_scale {
  my $self=shift @_;
  my $scale=shift @_;

  warn "This method should be deprecated!";

  my $tiling=$self->_tiling();
  $tiling->bbox_scale($scale);  
}

###############
# description #
###############

=head1 description

Get description of tiling

$model->description();

=cut

sub description {
  my $self=shift @_;

  my $tiling=$self->_tiling();
  my $description=$tiling->description();  
}

###########
# reverse #
###########

=head1 reverse

Method to reverse all crease assignemnts

$model->reverse();

=cut

sub  reverse {
  my $self=shift @_;

  my $tiling=$self->_tiling();
  $tiling->reverse();
}

###########
# stretch #
###########

=head1 stretch

Method to stretch tilings along x axis

$model->strect(0.5);

=cut

sub  stretch {
  my ($self,$stretch)=@_;

    my $tiling=$self->_tiling();  
    if ($tiling){
      $tiling->stretch($stretch);
    }
    my $crease_pattern=$self->_crease_pattern();  
    if ($crease_pattern){
      $crease_pattern->stretch($stretch);
    }
  $self->notify_view();
}


###########
# _tiling #
###########

=head1 _tiling

Internal method to get tiling object.  This should not
ever be made externally available! Tiling is the a copy
of the Tiling object before origamify has been carried out.
The _crease_pattern stores the object after origamify.

$tiling=$model->_tiling();

=cut

sub _tiling {
  my $self=shift @_;
  my $tiling=$self->{'tiling'};
  
  return $tiling;
}

###################
# _crease_pattern #
###################

=head1 _crease_pattern

Internal method to get tiling object of crease pattern.  This should
not ever be made externally available! So this stores the tiling
object after origamify. _tiling stores tiling object before origamify.

$tiling=$model->_crease_pattern();

=cut

sub _crease_pattern {
  my $self=shift @_;
  my $tiling=$self->{'crease_pattern'};
  
  return $tiling;
}


###########
# tag2ref #
###########

=head1 tag2ref

This method returns an object reference given a render stack tag id

$self->tag2ref($tag);

=cut

sub tag2ref {
    my ($self,$tag)= @_;

    # decrement as array starts at 0 whereas tags start at 1
    $tag--;
    my $ref=$self->{'tag_list'}[$tag];
    return $ref;
}

#############
# decompose #
#############

=head1 decompose

A fairly polymorphic method that can deal with decomposing a tile.

    $model->decompose($method,$model_tag, .... other parameters);

Possible $methods are crystal,instar and decompose. Each requires different parameters

=cut

sub decompose {
    my $self   = shift @_;
    my $method = shift @_;
    my $tag    = shift @_;

    if (! defined $tag){
	die "In Model::decompose tag not defined $self $method";
    }

    my $tiling=$self->_tiling();
    my $obj=$self->tag2ref($tag);

    # Convert tag into object reference!
    my ($set) = $tiling->each();
    my $gui_set = new Patch();

    if ($method eq 'crystal'){
	$gui_set = crystal Patch($obj,@_);
	$tiling->add($gui_set);
    } elsif ($method eq 'instar'){
	$gui_set = instar Patch($obj,@_);
	$tiling->add($gui_set);
    } elsif ($method eq 'decompose'){
	$gui_set->decompose($obj);
	$tiling->add($gui_set);
    } else {
    &report("Your decompose method $method is not in my list. Serious error!");
    return 0;
  }

    # Need to delete object that is acted upon.  This will mean
    # I need to loop over tiling to find object to remove
    foreach my $set ($tiling->each()){
	if ($set->remove($obj)){
	    last;
	}
    }


    #warn "Calling notify_view from decompose";
    $self->notify_view();

}

##########
# delete #
##########

=head1 delete

A method to delete a tile from a tiling using model tag

    $model->delete($model_tag);

=cut

sub delete {
    my $self   = shift @_;
    my $tag    = shift @_;

    if (! defined $tag){
	die "In Model::delete tag not defined";
    }

    my $tiling=$self->_tiling();
    my $obj=$self->tag2ref($tag);

    # Need to delete object that is acted upon.  This will mean
    # I need to loop over tiling to find object to remove
    foreach my $set ($tiling->each()){
	if ($set->remove($obj)){
	    last;
	}
    }
    $self->notify_view();
}

#############
# vertexify #
#############

=head1 vertexify

Vertexify model

$model->vertexify();

Will include edge babies as well if you use

$model->vertexify('edge' => 1);

This will give more accurate light pattern rendering at the edges of the paper.

=cut

sub vertexify {
  my $self=shift @_;

  my %args=@_;

  my $tiling=$self->_tiling();
  $self->{'crease_pattern'}=$tiling->copy();
  my $crease_pattern=$self->_crease_pattern();

  $crease_pattern->vertexify(%args);
  $self->state('unfolded');

  # Now notify views that Model has changed
  #warn "Calling notify_view from vertexify";
  $self->notify_view();
}


##################
# undo_vertexify #
##################

=head1 undo_vertexify

Will undo a vertexify. Will result in getting the original tiling
again

$model->undo_vertexify();

=cut

sub undo_vertexify {
  my $self=shift @_;

  my %args=@_;

  my $tiling=$self->_tiling();

  # Remove crease pattern object
  $self->{'crease_pattern'}=undef;

  $self->state('tiling');

  # Now notify views that Model has changed
  $self->notify_view();
}


##########
# rotate #
##########

=head1 rotate

This method will rotate model by an angle
To rotate all polygons about a specific point use

$model->rotate($rotation,$x_orig,$y_orig);

=cut

sub rotate {
    my ($self,$rotation,$x_orig,$y_orig)=@_;

    if (! $x_orig){
	$x_orig=0;
	$y_orig=0;
    }

    my $tiling=$self->_tiling();  
    if ($tiling){
      $tiling->rotate($rotation,$x_orig,$y_orig);
    }
    my $crease_pattern=$self->_crease_pattern();  
    if ($crease_pattern){
      $crease_pattern->rotate($rotation,$x_orig,$y_orig);
    }

    $self->notify_view();
}

####################
# reassign_creases #
####################

=head1 reassign_creases

Will reassign creases based on a focus.

$self->reassign_creases($x,$y)

=cut

sub reassign_creases {
    my ($self,$x,$y)=@_;

    my $tiling=$self->_tiling();  
    $self->reassign_creases($x,$y)
}

#############
# translate #
#############

=head1 translate

Translate model

$model->translate(x,y);

=cut

sub translate {
    my $self=shift @_;
    my $x=shift @_;
    my $y=shift @_;
    
    my $tiling=$self->_tiling();
    if ($tiling){    
      $tiling->translate($x,$y);
    }

    my $crease_pattern=$self->_crease_pattern();
    if ($crease_pattern){
	$crease_pattern->translate($x,$y);
    }
    
    # Now notify views that Model has changed
    $self->notify_view();
}

##########
# render #
##########

=head1 render

Method to return an XML render stack, that will be used
by View objects.  Also this method puts an array into
the model object which has object references in.  These correspond to
tag attributes for objects in XML.  This means external 
objects to model can refer to objects by these tags and
we can work out which object the external objects mean.

($stack)=$model->render();

$stack is XML stack for rendering
$tag_ref is a reference to the array of objects

=cut

sub render {
    my $self = shift @_;

    my %args=@_;

#    if (%args){
#	warn %args," in Model::render.";
#    }

    my $tiling;
    if (! defined $args{'show'}){
	carp "In Model::render don't know whether to show patch, tiling or creases!  Guessing you want creases!";
	$args{'show'}='creases';
    }

    if ($args{'show'} eq 'tiling' or $args{'show'} eq 'patch'){
	$tiling=$self->_tiling();
    } elsif ($args{'show'} eq 'creases'){
	$tiling=$self->_crease_pattern();
    } else {
	warn "In model::render show is $args{'show'}";
    }

    my ($stack,$tag_ref)=$tiling->render(%args);

    $self->{'tag_list'}=$tag_ref;

    return $stack;
}

##############
# parameters #
##############

=head1 parameters

Method to return parameters for a model.

%parameters=$model->parameters();

=cut

sub parameters {
    my $self = shift @_;

    my $tiling=$self->_tiling();
    my %hash=%{$tiling};

    return %hash;
}

#########
# title #
#########

=head1 title

Method to get or set title for model

$self->title($title)

=cut

sub title {
    my $self = shift @_;

    my $title=shift @_;
    my $tiling=$self->_tiling();
    if ($title){
	$tiling->title($title);
    }
    $title=$tiling->title();

    return $title;
}

########
# page #
########

=head1 page

Method to get or set page number for model

$self->page($title)

=cut

sub page {
    my $self = shift @_;

    my $page=shift @_;
    my $tiling=$self->_tiling();
    if ($page){
	$tiling->page($page);
    }
    $page=$tiling->page();

    return $page;
}

#############
# transform #
#############

=head1 transform

Transform model, give pleat angle and pleat ratio.  Also pass
second pleat angle for weave tessellations. This routine actually
changes the Model object to have these new properties.

$model->transform($pleat_ratio,$pleat_angle,$pleat_angle2);

=cut

sub transform {
    my $self = shift @_;
    my ($pleat_ratio,$pleat_angle,$pleat_angle2)=@_;

    if (defined $pleat_ratio and defined $pleat_angle){
	if ($pleat_angle < 0){
	    $self->state('folded');
	} else {
	    $self->state('unfolded');
	}
	
	my $crease_pattern=$self->_crease_pattern();

	if ($crease_pattern){
	  $crease_pattern->transform($pleat_ratio,$pleat_angle,$pleat_angle2);
	} else {
	  croak "Oh no in transform without a crease pattern being made!";

	}
    }
    # Now notify views that Model has changed
    $self->notify_view();
}

#########
# scale #
#########

=head1 scale

Scale model, use x and y as origin of scaling

$model->scale($scale,$x,$y);

=cut

sub scale {
  my $self=shift @_;
  my $scale=shift @_;
  my $x=shift @_;
  my $y=shift @_;

  my $tiling=$self->_tiling();
  $tiling->scale($scale,$x,$y);

  my $crease_pattern=$self->_crease_pattern();
  if ($crease_pattern){
      $crease_pattern->scale($scale,$x,$y);
  }

  my $patch=$self->{'patch'};
  if ($patch){
      $patch->scale($scale,$x,$y);
  }

  # Now notify views that Model has changed
  #warn "Calling notify_view from scale";
  $self->notify_view();
}

#############
# max_angle #
#############

=head1 max_angle

    Method to give maximum possible pleat angle.  This assumes
    that you are using the same twist for all polygons. The
    max angle is half of the internal angle for the most acute
    angle in the tiling.

    $max_angle=$model->max_angle();

=cut

sub max_angle {
  my $self=shift @_;

  my $tiling=$self->_tiling();

  my $max_angle=$tiling->max_angle();
  return $max_angle;
}

###############
# attach_view #
###############

=head1 attach_view

This method will attach a view object to a model object. The Model object will
then notify the view object when it has changed so it can update itself based
on the new Model.

  $model->attach_view($view);

These methods are for the observer pattern from design patterns, page 293.

=cut

sub attach_view {
  my ($self,$view)=@_;

  push (@{$self->{'view_list'}}, $view);
}

###############
# detach_view #
###############

=head1 detach_view

This method will detach a view object to a model object.

  $model->detach_view($view);

These methods are for the observer pattern from design patterns, page 293.

=cut

sub detach_view {
  my ($self,$view)=@_;

  die "Oh dear got tangled in knots";
}


###############
# notify_view #
###############

=head1 notify_view

This method will tell all observer view objects when Model has changed

  $model->notify_view();

These methods are for the observer pattern from design patterns, page 293.

This method is called in the following methods:
   vertexify
   translate
   scale
   transform


=cut

sub notify_view {
  my $self=shift @_;

  foreach my $view (@{$self->{'view_list'}}){
    $view->update();
  }
}

############
# x_expand #
############

=head1 x_expand

Get/set for x expansion

    $tiling->x_expand();

This is how many times patch is repeated along x axis.

=cut

sub x_expand {
  my $self = shift @_;
  my $value = shift @_;

  my $tiling=$self->_tiling();

  if ($value){
    $tiling->x_expand($value);
  } else {
    $value=$tiling->x_expand();
  }

  return $value;
}

############
# y_expand #
############

=head1 y_expand

Get/set for y expansion

    $tiling->y_expand();

This is how many times patch is repeated along y axis.

=cut

sub y_expand {
  my $self = shift @_;
  my $value = shift @_;

  my $tiling=$self->_tiling();

  if ($value){
    $tiling->y_expand($value);
  } else {
    $value=$tiling->y_expand();
  }

  return $value;
}





1;  # says use was ok

__END__



