
#
# Perl Module for PDF
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package PDF;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use View;
use Text::Wrap;
use lib '/nfs/team71/pfam/agb/lib';
use PDF::Create;


#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ('Exporter','View');

#######
# new #
#######

=head1 new

Creates new PDF object. Note that handle is actually a file rather
than a filehandle.  I couldn't get PDF::create to consistently 
write to a filehandle!

$view = new PDF($model,$filename);

=cut

sub new {
    my ($class,$model,$filehandle) = @_;

    my $self = new View($model,$filehandle);
    bless $self, $class;

    # Attach view object to model so it can notify it of updates.
    $model->attach_view($self);
    return $self;
}



#######
# pdf #
#######

=head2 pdf

Sets or get pdf object reference

    $pdf=$view->pdf($pdf);

=cut

sub pdf {
    my ($self,$value)=@_;

    if (defined $value){
      $self->{'pdf'}=$value;
    }
    return $self->{'pdf'};
}


##########
# header #
##########

=head1 header

Prints PDF header to filename

    $output->header(filename);

=cut

sub header {
  my ($self,$file,$parameters) = @_;

  warn "Creating PDF file $file";

#  my $pdf = new PDF::Create('fh'           => $FILEHANDLE,
  my $pdf = new PDF::Create('filename'     => $file,
			    'Version'      => 1.2,
			    'PageMode'     => 'UseOutlines',
			    'Author'       => 'Alex Bateman',
			    'Title'        => 'Origami Tessellation',
			    'CreationDate' => [ localtime ],
			    );

  $self->pdf($pdf);
}

##########
# footer #
##########

=head1 footer

Prints PDF footer

    $output->footer();

=cut

sub footer {
  my ($self) = @_;

  my $pdf=$self->pdf();
  $pdf->close();
  warn "In PDF footer";
}



########
# body #
########

=head1 body

Print body of postscript

    $output->body();

=cut

sub body {
    my ($self) = @_;

    my $pdf=$self->pdf();

    #my $x_tran=300;
    #my $y_tran=300;
    my $x_tran=0;
    my $y_tran=0;

    my $root = $pdf->new_page('MediaBox' => [ 0, 0, 612, 792 ]);
    # Add a page which inherits its attributes from $root
    my $page = $root->new_page;

    my ($stack,$tag_ref)=$self->{'model'}->render(%args);
    my @lines=split /^/,$stack;

    foreach my $line (@lines){
	my ($tag,$rest);
	if ($line =~ /\<(\S+)\s+(.*)/){
	    $tag=$1;
	    $rest=$2;
	}

	if ($tag eq 'line'){
	    if ($rest =~ /crease=\"(\S)\" x1=\"(\S+)\" y1=\"(\S+)\" x2=\"(\S+)\" y2=\"(\S+)\"/){
		my $x1=$2+$x_tran;
		my $y1=$3+$y_tran;
		my $x2=$4+$x_tran;
		my $y2=$5+$y_tran;

		if ($1 eq 'm'){
		    $page->set_width(1);
		    $page->line($x1, $y1, $x2, $y2);
		} else {
		    $page->set_width(2);
		    $page->line($x1, $y1, $x2, $y2);
		}
	    }
	}
    }
}

1;  # says use was ok


__DATA__

# page 1

Copyright (c) 2005 by Alex Bateman

All rights reserved.  The patterns in this E-book may not be resold or
redistributed in any form without written permission from the
author.

# page 2

The picture above is of a folded origami tessellation. A style of
folding developed by Yoshide Momatani, Shuzo Fujimoto and more
recently by Chris Palmer.  If they are folded from transparent paper
such as tracing paper (UK) or vellum (US) they show spectacular
patterns when held up to a bright light.

# page 3

So you would like to fold some Origami Tessellations. This book
contains a collection of designs I have made over the past ten years.
I have not used standard origami notation for these models, it just is
not possible to provide step-by-step instructions. To fold them
successfully I suggest the following method.


1. Photocopy the pattern onto a transparent or thin paper. 

2. Fold the pattern on a flat surface. 

3. Fold ALL the thin lines as valley folds. Then turn the the paper over and fold ALL the thick lines as valley folds (These thick lines might be visible through non-transparent paper). 

4. Gently include all the creases you have made, this makes parts of the model twist. 

5. Be patient. 

6. Flatten the model completely and hold up to the light to see the final pattern. 


# page 4

This is a simple example to help you understand how to collapse
a simple twist fold. Later on we will combine these simple elements
or tiles into an origami tessellation.

# page 5

This twist fold is called an open back twist fold.

# page 6

This twist fold is called an iso area twist fold. Some layers go in
front of the central hexagon whereas others go behind.

# page 7

This is called an closed back twist fold.


# page 8

This is the most complex twist fold you will see in this book.  So if you
can get this right you have all the basic skills you will need.

# page 9

Here is the iso area version of an octagon twist. 

# page 10

Now we put four twist folds together to make our first real tessellation.

# page 11

Make sure to crease all the thick lines first.  Then turn the paper over
and crease all the thin lines.  This will help to stop you getting
confused and making some creases in the wrong direction.


# page 12

The book Patterns and Tilings by Grunbaum and Shephard
(ISBN:0716719983) is the essential guide to tessellations. It is an
extraordinary book.  that gave me a lot of inspiration.


# page 13

You can find out more about Origami Tessellation at the following URL

www.sanger.ac.uk/Users/agb/Origami/Tessellation# page 14


# page 15

When you hold your tessellation up to a bright light you see the full beauty
of the model in the light pattern.  The various number of layers overlapping
back and forth give rise to completely unexpected shapes. These patterns are
why I fold tessellations.

# page 16

Here is a practice run for the hexagon swirl

# page 17

The hexagon swirl is my all time favourite tessellation.  I love the
way the network of triangles seem to be sitting above the rest of the paper.
