=head1 NAME

Patch

=head1 DESCRIPTION

B<Patch> contains a collection of polygon objects that can be
translated to make a complete tiling.

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for Patch
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Patch;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Polygon;
use Adult;
use Line;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );


#######
# new #
#######

=head1 new

Creates Patch object. This contains many attributes.
The weave attribute can be left or right.

$atch = new Patch();

=cut
sub new {
    my ($class,%parameters)=@_;

    my $set = {
	       "polygons"   => [],
	       "x1"         => $parameters{'x1'},
	       "y1"         => $parameters{'y1'},
	       "x2"         => $parameters{'x2'},
	       "y2"         => $parameters{'y2'},
	       "alternate1" => $parameters{'alternate1'},
	       "alternate2" => $parameters{'alternate2'},
	       "current_alpha"  => '1',
	       "current_theta"  => '0',
	       "current_alpha2" => '1', # Used in weave tilings
	       "current_theta2" => '0', # Used in weave tilings
	       };

    bless $set, $class;
    return $set;
}


########
# copy #
########

=head1 copy

Method to copy a patch object

    $new_patch = $patch->copy();

=cut
sub copy {
    my ($self)=@_;

    my $newself= new Patch(
			   'x1'         =>$self->{'x1'},
			   'x2'         =>$self->{'x2'},
			   'y1'         =>$self->{'y1'},
			   'y2'         =>$self->{'y2'},
			   'alternate1' =>$self->{'alternate1'},
			   'alternate2' =>$self->{'alternate2'},
			   );

    foreach my $object ($self->each()){
      my $new_object=$object->copy();
      $newself->add($new_object);
    }
    return $newself;
}

########
# each #
########

=head1 each

Loops over Patch and returns a list of polygon objects

    @polygons = $patch->each();

=cut

sub each {
  my($self)=@_;
  my @list=@{$self->{'polygons'}};
  return @list;
}

#######
# add #
#######

=head1 add

Add polygons to Patch object

    $patch->add($polygon1..$polygonN);

=cut
sub add {
  my $self = shift @_;
  push(@{$self->{'polygons'}},@_);
}

##########
# remove #
##########
=head1 remove

Will remove a polygon object from patch.  Will return 1 if removed
and 0 if not.

$patch->remove($polygon);

=cut
sub remove {
  my($self,$polygon)=@_;
  my $return=0;
  my @array;

  # Hmm got to be careful modifying arrays while looping over them.
  foreach my $tmp_polygon ($self->each()){
    if ($polygon eq $tmp_polygon){
      #print "Yep this polygon is in the polygonset $polygonset, removing\n";
      $return=1;
    } else {
      push (@array,$tmp_polygon);
    }
  }
  # Replace point list!
  #print "Replacing @array\n";
  $self->{'polygons'}=[];

  foreach my $element (@array){
    $self->add($element);
  }
  return $return;
}


###########
# add_set #
###########

=head1 add_set

Adds all polygons of a set to Set object

    $patch->add_set($polygonset2);

=cut
sub add_set {
    my ($set,$set2)=@_;

    foreach my $polygon ($set2->each()){
      push(@{$set->{'polygons'}},$polygon);
    }
}


#########
# scale #
#########

=head1 scale

    Method to scale each polygon in Patch using given x and y as
origin for scaling.

    $patch->scale($scale_factor,$x_origin,$y_origin);

If x and y are not given will use centre of each polygon as origin
for scaling.

=cut
sub scale {
    my ($set,$scale_factor,$x_origin,$y_origin)=@_;

    my $len=@_;

    if (! $scale_factor){
	die "In method Adult::scale, No scale factor given!\n";
    }


    if ($len == 2){
      foreach my $polygon ($set->each()){
	$polygon->scale($scale_factor);
      }
    } elsif ($len == 4){
      foreach my $polygon ($set->each()){
	$polygon->scale($scale_factor,$x_origin,$y_origin);
      }
    } else {
      die "Wrong number of args in Patch::scale\n";
    }
}



#############
# translate #
#############

=head1 translate

    Method translate each polygon in Patch by x and y

$polygonset -> translate($x,$y);

=cut
sub translate {
    my ($set,$x,$y)=@_;

    foreach my $polygon ($set->each()){
      eval {
	$polygon->translate($x,$y);
      };
      if ($@ eq "Cannot translate baby!\n"){ # Catch exception
      } elsif ($@){die "$@";}
    }
}

##########
# rotate #
##########

=head1 rotate

    Method to rotate each polygon in Patch by $rotation

$polygonset -> rotate($rotation);

or

$polygonset -> rotate($rotation,$x_orig,$y_orig);

=cut
sub rotate {
  my ($set,$rotation,$x_orig,$y_orig)=@_;

  my $len=@_;
  
  if ($len==2){
    foreach my $polygon ($set->each()){
      $polygon->rotate($rotation);
    }
  } elsif ($len==4){
    foreach my $polygon ($set->each()){
      $polygon->rotate($rotation,$x_orig,$y_orig);
    }
  } else {
    die "Wrong number args in Patch::rotate\n";
  }
}


################
# set_polygon1 #
################

=head1 set_polygon1

Dummy routine, in place to mimic PolygonSetSet object.  Required in Patch::transform
to recognise difference between patches and Vertex sets.

=cut
sub set_polygon1 {
    die "Would like to Depracate method";
    return '4';
}

################
# set_polygon2 #
################

=head1 set_polygon2

Dummy routine, in place to mimic PolygonSet object.  Required in Patch::transform
to recognise difference between patches and Vertex sets.

=cut

sub set_polygon2 {
    die "Would like to Depracate method";
    return '4';
}



###########
# reverse #
###########
=head1 reverse

    Method to reverse crease assignment for set

$set->reverse();

=cut

sub reverse {
    my ($self)=@_;

    foreach my $polygon ($self->each()){
	$polygon->reverse();
    }
}

###########
# reorder #
###########

=head1 reorder

Will take a polygonset and reorder point objects for each polygon if incorrect.
Only works on  convex polygons!

$polygonset->reorder();

=cut
sub reorder {
    my ($set)= @_;      # Get object

    foreach my $polygon ($set->each()){
	$polygon->reorder();
    }
    return $set;
}


#############
# transform #
#############

=head1 transform

Takes the absolute alphas and thetas as arguments.  Then calculates
the difference between absolute and current values of theta and alpha
and does scale and rotate.  type is either 'twist' or 'weave'

e.g.
  $set->transform($type,$alpha1,$alpha2,$theta1,$theta2);

=cut

sub transform {
    my ($self,$type,$alpha1,$alpha2,$theta1,$theta2) = @_;

    #warn "In transform with alpha1 $alpha1 alpha 2 $alpha2 theta1 $theta1 theta2 $theta2";
    
    if ($type eq 'twist'){
	    # Rotate and scale by difference in absolute and current values for theta and alpha
	    my $change=$theta1-$self->{'current_theta'};
	    
	    if (! $change){
		carp "No value for change in Patch::transform";
	    }
	    
	    $self->rotate($change);
	    $self->{'current_theta'}=$theta1;    
	    
	    my $change2=$alpha1/$self->{'current_alpha'};
	    $self->scale($change2);
	    $self->{'current_alpha'}=$alpha1;
    } elsif ($type eq 'weave'){
	##################
	# Weave geometry #
	##################
	
	# Rotate and scale by difference in absolute and current values for theta and alpha
        my $change_1_1=$theta1-$self->{'current_theta'};
        my $change_1_2=$alpha1/$self->{'current_alpha'};
	my $change_2_1=$theta2-$self->{'current_theta2'};
	my $change_2_2=$alpha2/$self->{'current_alpha2'};

	foreach my $polygon ($self->each()){
	    my $colour=$polygon->colour();

	    if ($colour eq 1){
	      $polygon->rotate($change_1_1);
	      $polygon->scale($change_1_2);
	    } elsif ($colour eq 2){
	      $polygon->rotate($change_2_1);
	      $polygon->scale($change_2_2);
	    } else {
		die "Shouldn't be trying to weave a tessellation with colour $colour";
	    }
	}
	$self->{'current_theta'}=$theta1;
	$self->{'current_alpha'}=$alpha1;
	$self->{'current_theta2'}=$theta2;
	$self->{'current_alpha2'}=$alpha2;

    } else {
	die "Shouldn't be here with type of $type";
    }
}

#########
# write #
#########

=head1 write

Debugging method: Write polygonset to STDERR

$polygonset->write();

=cut
sub write {
    my ($set)= @_;      # Get object

    print "Writing Patch $set\n";
    foreach my $polygon ($set->each()){
	$polygon->write();
    }
}

################
# make_species #
################

=head1 make_species

Generates a patch where polygon of vertex species specified

$patch->make_species(4,4,4,4)

=cut

sub make_species {
  my $self = shift @_;
  my @n    = @_;

  ###########################################
  # Check that vertex specified is possible #
  ###########################################
  my $total=0;
  foreach my $n (@n){
      $total=$total+(180*(1-2/$n));
  }

  if ($total > 360.001 or $total < 359.999){
      die "Patch::make_species your polygons add up to $total not 360!\n";
  }

  #######################
  # Generate the tiling #
  ####################### 

  $total=0;
  foreach my $n (@n){
      # Make new set
      my $polygon=Adult::new_ngon($n,1);
      # Do translate and rotates
      my $y=50*cos(3.1415927/$n)/sin(3.1415927/$n);
      $polygon->translate(50,-$y);
      $polygon->rotate(180*(1-2/$n),0,0);
      $polygon->rotate($total,0,0);
      $self->add($polygon);
      $total=$total+180*(1-2/$n);
  }



  return $self;
}


##########
# render #
##########
=head1 render

Method to return an XML render stack, that will be used
by View objects.  Also this method returns an array
which has object references in.  These correspond to
tag attributes for objects in XML.  THis means external 
objects to model can refer to objects by these tags and
we can work out which object the external objects mean.

($current_tag,$stack,$tag_ref)=$tiling->render($current_tag,$stack,\@tags);

$stack is XML stack for rendering
$tag_ref is a reference to the array of objects

=cut
sub render {
    my ($self,$current_tag,$stack_ref,$tag_ref) = @_;

    $current_tag++;
    push (@$tag_ref,$self);

    my $open_tag  = "  <polygonset tag=\"$current_tag\">\n";
    my $close_tag = "  <polygonset/>\n";

    $$stack_ref .= $open_tag;

    foreach my $element ($self->each()){
	($current_tag,$stack_ref,$tag_ref)=$element->render($current_tag,$stack_ref,$tag_ref);
    }
    $$stack_ref .= $close_tag;

    return ($current_tag,$stack_ref,$tag_ref);
}




#############
# decompose #
#############

=head1 decompose

Takes n-gon object and decomposes into n triangles.
Probably need to look at crease of each line to get correct creases.

$set->decompose($polygon,$handedness);

Handedness says whether layers swirl to 'left' or 'right'

=cut

sub decompose {
  my ($set,$polygon,$handedness) = @_;

  if (! $handedness){
      $handedness = 'left';
  }

  my $x=$polygon->dual_vertex()->x();
  my $y=$polygon->dual_vertex()->y();


  my @points = $polygon->each();
  my @lines=$polygon->eachline();

  my @triangles; # Store decomposed triangles
  my @theta; # Need these angles to find dual vertices of decomposed tiles
  my @orthogonal_lines; # Stores lines on which centres of decomposed triangles will lie

  my $n=@points;
  my $i;
  for ($i=0; $i<$n; $i++){
    # Centre of scaling is wrong here!
    my $triangle = new Adult(0,0,'1');

    my $crease=$lines[$i]->crease();
    my $next_crease=$lines[($i+1)%$n]->crease();

    my $point1 = new Point ($x,$y,$triangle);
    my $point2 = $points[$i]->copy($triangle);
    my $point3 = $points[($i+1)%$n]->copy($triangle);

    my ($line1,$line3);
    if ($handedness eq 'right'){
	$line1=new Line($point1,$point2->copy($triangle),$crease);
	$line3=new Line($point3,$point1->copy($triangle),-1*$next_crease);
    } elsif ($handedness eq 'left'){
	$line1=new Line($point1,$point2->copy($triangle),$crease);
	$line3=new Line($point3,$point1->copy($triangle),-1*$next_crease);
    } else {
	die "In Patch::decompose don't recognise handedness [$handedness]\n";
    }
#    my $line2=new Line($point2,$point3->copy($triangle),$crease); # Outer line
    my $line2=new Line($point2,$point3->copy($triangle),$crease); # Outer line

    $triangle->add($line1);
    $triangle->add($line2);
    $triangle->add($line3);

    if ($crease == 1){
	$triangle->colour(1);
    } elsif ($crease==-1){
	$triangle->colour(2);
    }


    # Make a line orthogonal to outside line of triangle $line2
    my $new_line = Line::orthogonal($line2,$polygon->dual_vertex());
    push (@orthogonal_lines,$new_line);

    # If this is the first orthogonal line scale by some factor
    if ($i == 0){
	my $factor=0.5;
	$new_line->scale($factor,$x,$y);
    }

    #$triangle->add($new_line);
    #$triangle->write();

    # Will need to store this. Also need angles
    my $theta1=$new_line->angle($line1);
    my $theta2=-180+$line3->angle($new_line);

    push (@theta,$theta1,$theta2);
    push (@triangles,$triangle);

    # Centre of scaling is corrected here!
    #$triangle->find_centre();
    # $triangle->find_centre2();  
    $set->add($triangle);
  }

  # Now fix all centre points
  $n=@triangles;

  my $pi = atan2(1,1)*4;
  $i=1;
  my @scale;
  while ($i<=$n){
      # Calculate scales for each centre
      if ($i == 1){
	  $scale[$i]=1;
      } else {
	  my $theta3=$pi*$theta[((($i-1)*2)-1)]/180;
	  my $theta4=$pi*$theta[(($i-1)*2)]/180;

	  my $temp_scale=$scale[$i-1]*cos ($theta3)/cos ($theta4);
	  if ($temp_scale<0){
	      die "Scale is less than zero! Should never happen. previous scale is $scale[$i-1], thetas are $theta[((($i-1)*2)-1)] and $theta[(($i-1)*2)] cos thetas are ",cos ($theta[((($i-1)*2)-1)])," and ",cos ($theta[(($i-1)*2)]),"\n";
	  }
	  $scale[$i]=$temp_scale;
      }
      $i++;
  }

  # OK now have all scales to get dual vertices
  # Loop one more time to do all scales
  $i=1;
  my $length=$orthogonal_lines[0]->length();
  while ($i<=$n){
      # Calculate scales for each centre
      my $current_line=$orthogonal_lines[$i-1];
      my $current_length=$current_line->length();

      # Scale to be same length as first orthogonal line
      $current_line->scale($length/$current_length,$x,$y);

      # Now apply scale factor
      $current_line->scale($scale[$i],$x,$y);
      my ($point1,$point2)=$current_line->points();

      # We now have the correct origin for the triangle!
      $triangles[$i-1]->dual_vertex($point2);
      $i++;
  }
  return $set;
}

####################
# double_decompose #
####################

=head1 double_decompose

Takes n-gon object and decomposes twice into 3n triangles.

$set->double_decompose($polygon);

=cut

sub double_decompose {
  my ($set,$polygon) = @_;

  my $tempset = new Patch();# Might cause bugs!
  $tempset->decompose($polygon);
  foreach my $newpolygon ($tempset->each()){
    my $tempset2 = new Patch();# Might cause bugs!
    $tempset2->decompose($newpolygon);
    $set->add_set($tempset2);
  }
  return $set;
}

############
# new_star #
############

=head1 new_star

Creates set of polygons that form a star.  This includes a 
central n-gon surrounded by n triangles

    $star_set = new_star($n,$angle,$creases,$alternate);

$n is number of sides
$angle is angle in degrees
$creases contains : separated list of creases for triangle
$alternate should be 'alternate' to swap crease assignment of triangles 

First point is vertically above centre.  The central n-gon
has its first side horizontal. The edges have length 100 units.

=cut

sub new_star {
    my ($n,$angle,$creases,$alternate)=@_;
    my $set = new Patch();

    my ($crease1,$crease2,$crease3)=split(/:/,$creases);

    my ($colour,$central_colour);
    if ($crease1 == 1 and $crease2 == 1 and $crease3 == 1){
	$colour=1;
	$central_colour=2;
    } elsif ($crease1 == -1 and $crease2 == -1 and $crease3 == -1){
	$colour=2;
	$central_colour=1;
    } else {
	$colour='3a';
	$central_colour='3b';
    }

    # Convert angle to radians
    my $pi = atan2(1,1)*4;
    $angle=$angle*$pi/180;

    my $ngon= new Adult(0,0,$central_colour);

    # Find first point
    my $x=100*sin($angle/2);
    my $y=$x*sqrt(1/(sin($pi/$n)*sin($pi/$n))-1);
    my $point = new Point(-$x,$y,$ngon);

    my @inner_points;
    push (@inner_points,$point);

    # Rotate point to make central Polygon.
    my $i=1;
    while ($i<$n){
	my $new_point=$point->copy();
	$new_point->rotate((360*$i)/$n);
	push (@inner_points,$new_point);
	$i++;
    }

    ###################################
    # Make polygon from @inner_points #
    ###################################
    $i=0;
    while($i<$n){
      my $point1 = $inner_points[$i];
      my $point2 = $inner_points[($i+1)%$n]->copy();

      my $crease=-1*$crease2;
      if (defined $alternate and $alternate eq 'alternate'){
	  $crease=$crease2**$i;
      }

      my $line= new Line($point1,$point2,$crease);
      $ngon->add($line);
      $i++;
    }

    $set->add($ngon);

    # Now make outer points to make triangles
    # Find first point
    $y=100*cos($angle/2)+$x*sqrt(1/(sin($pi/$n)*sin($pi/$n))-1);

    # Make new polygon object
    my $outer_polygon = new Adult(0,0,$central_colour);  # This polygon is just a temporary object
    my $outer_point = new Point(0,$y,$outer_polygon);
    my @outer_points;
    push (@outer_points,$outer_point);
    $i=1;
    while ($i<$n){
      my $new_point=$outer_point->copy();
      $new_point->rotate((360*$i)/$n);
      push (@outer_points,$new_point);
      $i++;
    }

    # Now make triangles from Inner and outer polygon
    $i=1;
    while ($i<=$n){
      my $triangle = new Adult(0,0,$colour);      

      my $point1=$outer_points[$i%$n]->copy($triangle);
      my $point2=$inner_points[$i-1%$n]->copy($triangle);
      my $point3=$inner_points[$i%$n]->copy($triangle);

      my $line1=new Line($point2,$point1->copy(),$crease1);
      my $line2=new Line($point3,$point2->copy(),$crease2);
      my $line3=new Line($point1,$point3->copy(),$crease3);

      $triangle->add($line1,$line2,$line3);

      if (defined $alternate and $alternate eq 'alternate' and (-1)**$i>0){
	  $triangle->reverse();
      }

      # Add centre to triangle
      $triangle->find_centre2();

      $set->add($triangle);
      $i++;
    }
    return $set;
}



###########
# crystal #
###########

=head1 crystal

Takes n-gon object and decomposes by insetting a smaller version of the
polygon and join up the vertices. Returns central polygon object and set
of surrounding objects.

($set)= crystal Patch($polygon,$reduction1 .. $reductionn);

$reduction How much to shrink inset polygon

=cut

sub crystal {
  my $class = shift @_;
  my $polygon = shift @_;

  # Make array of creases based on polygons
  my @creases;
  foreach my $line ($polygon->eachline()){
      push (@creases,$line->crease());
  }

  my $set = new Patch();
  my $inset_polygon;

  while (@_){
    my $shrink=shift @_;
    if ($shrink<0 || $shrink >1){
      die "Sorry \$shrink not specified in Patch crystal method\n ";
    }

    $inset_polygon=$polygon->copy();
    $inset_polygon->scale($shrink);

    my @points=$polygon->each();

    my @inset_points = $inset_polygon->each();
    my $n=@points;
    
    my ($quad,$point1,$point2,$point3,$point4,$i);
    for ($i=0; $i<$n; $i++){
      # Centre of scaling is wrong here!
      my $quad = new Adult(0,0,'1');
      
      my $point1 = $points[$i]->copy($quad);
      my $point2 = $points[($i+1)%$n]->copy($quad);
      my $point3 = $inset_points[($i+1)%$n]->copy($quad);
      my $point4 = $inset_points[$i]->copy($quad);

      my $line1=new Line($point1,$point2->copy(),$creases[$i]);
      my $line2=new Line($point2,$point3->copy(),1);
      my $line3=new Line($point3,$point4->copy(),$creases[$i]*(-1));
      my $line4=new Line($point4,$point1->copy(),-1);

      $quad->add($line1,$line2,$line3,$line4);
      
      # Centre of scaling should be corrected here!
      #    $triangle->find_centre();  
      $quad->find_centre2();  
      $set->add($quad);
    }
    $polygon=$inset_polygon; # cunning switch to allow recursion!
  }

  $set->add($inset_polygon);
  return $set;
}




###############
# quarterise #
###############
=head1 quarterise

Takes n-gon object and decomposes by making lines from midpoint of all lines to centre.
for a square the result would be four smaller squares.

my $set = quarterise Patch($polygon);

=cut


sub quarterise {
  my $class = shift @_;
  my $polygon = shift @_;

  die "Commented out this method 18/11/02";

  ####################################
  # Check number of sides is correct #
  ####################################
  my @lines=$polygon->eachline();
  my $n=@lines;

  if ($n != 4){
      die "In Patch::quarterise, can only accept polygons with 4 sides, yours has $n!\n";
  }

  # Foreach line get midpoint and put in an array.
  my (@midpoints,@cornerpoints);
  foreach my $line (@lines){
      push (@midpoints,$line->midpoint());
      my ($p1,$p2)=$line->points();
      push (@cornerpoints,$p1);
  }

  my $centre=$polygon->dual_vertex()->copy();

  ###########################
  # Make quartered polygons #
  ###########################

  my $set=new Patch();

  my $i=0;
  while($i<$n){
      my $line1=new Line($cornerpoints[$i]->copy(),$midpoints[$i]->copy(),1);
      my $line2=new Line($midpoints[$i]->copy(),$centre->copy(),1);
      my $line3=new Line($centre->copy(),$midpoints[($i-1)%$n]->copy(),1);
      my $line4=new Line($midpoints[($i-1)%$n]->copy(),$cornerpoints[$i]->copy(),1);

      ###############################
      # Calulate centre for polygon #
      ###############################
      my $dummy_line=new Line($cornerpoints[$i],$centre,1);
      my $mpoint = $dummy_line->midpoint();
      my $x=$mpoint->x();
      my $y=$mpoint->y();

      my $qpolygon=new Adult($x,$y,1); 
      $qpolygon->add($line1,$line2,$line3,$line4);
      $set->add($qpolygon);
      $i++;
  }
  return $set;
}



1;  # says use was ok
__END__
