=head1 NAME

Point

=head1 DESCRIPTION

The Point object contains a cartesian coordinate and a pointer
to a polygon object that owns the point object.

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for Point
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Point;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

#######
# new #
#######

=head2 new

Create new point object. Object contains x and y coordinates.

    $point = new Point($x,$y);

=cut

sub new {
    my ($class,$x,$y)=@_;

    croak "x or y is not defined for new point" unless defined $x and defined $y;

    my $point = {
    };

    bless $point, $class;

    $point->x($x);
    $point->y($y);

    return $point;
} 

########
# copy #
########

=head2 copy

Create new point object from old point.

    $new_point = $point->copy();

=cut

sub copy {
    my $point = shift @_;
    my $new_point = new Point($point->x(),$point->y());
    return $new_point;
}


#########
# label #
#########

=head2 label

Label point object for debugging purposes only. Label will be displayed in output.

    $point->label('some text');

=cut

sub label {
    my $len=@_;
    if ($len == 2){
	my $self  = shift @_;
	my $text  = shift @_;
	$self->{'label'}=$text; 
	return $text;
    } else {
	my $self   = shift @_;
	if (! defined $self->{'label'}){
	  return "";
	} else {
	    my $text=$self->{'label'};
	    return $text;
	}
    }
    die"Should never get here\n!";
}

##########
# rotate #
##########

=head2 rotate

Takes point and rotates about specified point by $angle in degrees.
If no x,y value is given rotation will be about the origin. 

    $point->rotate($angle,$x,$y);

=cut

sub rotate {
    my ($point,$angle,$xr,$yr)=@_;
    $angle=(3.1415927*$angle)/180;           # Convert degrees to radians
    
    if (not defined $xr){
      $xr=0;
      $yr=0;
    }

    # Do rotation of point by $angle
    my $xdiff = $point->x()-$xr;
    my $ydiff = $point->y()-$yr;

    my $cos=cos $angle;
    my $sin=sin $angle;

    $point->x($xr+$xdiff*$cos-$ydiff*$sin);
    $point->y($yr+$xdiff*$sin+$ydiff*$cos);
}

#############
# translate #
#############

=head2 translate

Takes point and translates by $x and $y

    $point->translate($x,$y);

=cut

sub translate {
    my ($point,$x,$y)=@_;

    $point->x($point->x()+$x);
    $point->y($point->y()+$y);
}

#########
# scale #
#########

=head2 scale

Takes point and uniformly scales by factor about x,y value,
will default to origin.

    $point->scale($factor,$x,$y);

=cut

sub scale {
    my ($point,$factor,$x,$y,$xcache,$ycache)=@_;

    # Potentially cache values of $x*(1-$factor)
    if ($ycache){
	$point->x($point->x()*$factor+$xcache);
	$point->y($point->y()*$factor+$ycache);
    } else {
	$point->x($point->x()*$factor+$x*(1-$factor));
	$point->y($point->y()*$factor+$y*(1-$factor));
    }
}

##########
# render #
##########
=head1 render

Method to return an XML render stack, that will be used
by View objects.  Also this method returns an array
which has object references in.  These correspond to
tag attributes for objects in XML.  THis means external 
objects to model can refer to objects by these tags and
we can work out which object the external objects mean.

($current_tag,$stack,$tag_ref)=$tiling->render($current_tag,$stack,\@tags);

$stack is XML stack for rendering
$tag_ref is a reference to the array of objects

=cut
sub render {
    my ($self,$current_tag,$stack,$tag_ref) = @_;
    my @tags=@{$tag_ref};

    $current_tag++;
    push (@tags,$self);

    my $x=int($self->x());
    my $y=int($self->y());

    my $open_tag  = "        <point tag=\"$current_tag\" x=\"$x\" y=\"$y\"/>\n";

    $stack .= $open_tag;

    return ($current_tag,$stack,\@tags);
}

#########
# write #
#########

=head2 write

Debugging method: prints point object data to STDERR

    $point-> write();

=cut

sub write {
    my ($point)=@_;

    print STDERR "Writing $point\n";
    printf STDERR "x:%8.1f y:%8.1f\n",$point->xy();
}

############
# distance #
############

=head2 distance

Returns distance between point objects

    $point1->distance($point2);

=cut

sub distance {
  my ($p1,$p2)=@_;

  my $xdist=$p1->x()-$p2->x();
  my $ydist=$p1->y()-$p2->y();  
  my $distance=sqrt ($xdist**2+$ydist**2);
  return $distance;
}


############
# proximal #
############

=head2 proximal

Will see if a point is in the proximity of a list of points. This routine
can be used as a speed up test for other routines.

    $point->proximal(@points);

returns 1 if point is within both a and y ranges of the list of points.

=cut

sub proximal {
  my $point= shift @_;

  my @points=@_;

  ######################
  # Find xmin and xmax #
  ######################
  my $xmin= 10000000;
  my $xmax=-10000000;
  my $ymin= 10000000;
  my $ymax=-10000000;

  foreach my $test_point (@points){
      my ($x,$y)=$test_point->xy();

      if ($x<$xmin){$xmin=$x;}
      elsif ($x>$xmax){$xmax=$x;}
      if ($y<$ymin){$ymin=$y;}
      elsif ($y>$ymax){$ymax=$y;}
  }

  ####################
  # Find if proximal #
  ####################
  my ($x,$y)=$point->xy();

  if ($x>$xmin and $x<$xmax and $y>$ymin and $y<$ymax){
      return 1;
  } else {return 0}
}


#####
# x #
#####

=head2 x

Sets x of point and returns value

    $x=$point->x();

=cut

sub x {
    my ($point,$x)=@_;

    if (defined $x){
      $point->{'x'}=$x;
    }
    return $point->{'x'};
}

#####
# y #
#####

=head2 y

Sets y of point and returns value

    $y=$point->y();

=cut

sub y {
    my ($point,$y)=@_;

    if (defined $y){
      $point->{'y'}=$y;
    }
    return $point->{'y'};
}

######
# xy #
######

=head2 xy

Returns x and y values of point. A bit faster than calling x and y seperately
Cannot be used to set values.

    ($x,$y)=$point->xy();

=cut

sub xy {
    my $point = shift @_;

    return $point->{'x'},$point->{'y'};
}


1;  # says use was ok
__END__
