=head1 NAME

Polygon

=head1 DESCRIPTION

Description for B<Polygon>

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for Polygon
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Polygon;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Adult;
use Point;
use Edge;
use Vertex;
use Line;


#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );


#######
# new #
#######

=head1 new

Creates empty polygon object.
coordinates for dual vertex (centre of scaling and rotation) is passed as an argument that
is converted into a Point object.

$polygon = new Polygon($x,$y);

=cut

sub new {
  my($class,$x,$y)=@_;

  confess "x or y is not defined for centre of new polygon" unless defined $x and defined $y;

  my $self = {
		 "lines"  => [],
		 "centre" => undef,
		};
  
  bless $self, $class;

  my $centre= new Point ($x,$y);
  $self->dual_vertex($centre);

  return $self;
}

########
# copy #
########

=head1 copy

DUMMY METHOD
Create new polygon object from old polygon.  Note that new point also belongs
to the same polygon, unless new polygon is passed as an argument.

$new_point = $point->copy($new_polygon);

=cut

sub copy {
    my $polygon=shift @_;
    $polygon->write();
    die "You can only copy Adults, use method there instead for $polygon\n";
}

##########
# layers #
##########

=head1 layers

Get set method for number of layers in polygon.  Used when rendering light
patterns.

    $polygon->layers(1);

=cut

sub layers {
    my $n=@_;

    if ($n == 1){
	my $polygon=shift @_;
	my $layers=$polygon->{'layers'};
	return $layers;
    } elsif ($n ==2){
	my $polygon=shift @_;
	my $layers=shift @_;
	$polygon->{'layers'}=$layers;
	return $layers;
    } else {
	die "In Polygon::layers, reached unreachable code!\n";
    }
}

##########
# centre #
##########

=head1 centre

Get method for centre point of polygon

    $point=$polygon->centre();

=cut

sub centre {
    my $self = shift @_;

    my $centre=$self->{'centre'};
    return $centre;
}

#########
# label #
#########

=head1 label

Label polygon object for debugging purposes only. Label will be displayed in output.

    $polygon->label('some text');

=cut

sub label {
    my $len=@_;
    if ($len == 2){
	my $self  = shift @_;
	my $text  = shift @_;

	$self->{'label'}=$text; 
	return $text;
    } else {
	my $self   = shift @_;
	my $text=$self->{'label'};
	return $text;
    }
    die"Should never get here\n!";
}


#########
# sides #
#########

=head1 sides

Returns number of sides of polygon

    $polygon->sides();

=cut

sub sides {
    my $polygon=shift@_;

    my @lines=$polygon->eachline();
    my $n=@lines;
    return $n;
}


############
# eachline #
############

=head1 eachline
  
Loops over polygon and returns a list of line objects.
  
@lines = $polygon->eachline();
=cut

sub eachline {
  my($polygon)= shift @_;
  my @list=@{$polygon->{'lines'}};
  return @list;
}

###########
# reverse #
###########

=head1 reverse

reverses crease assignment of polygon

$polygon->reverse();
=cut

sub reverse {
  my $polygon = shift @_;

  warn "Using this reverse for $polygon";
  foreach my $line ($polygon->eachline()){
    $line->reverse();
  }
}


########
# each #
########

=head1 each

Loops over polygon and returns a list of point objects

@points = $polygon->each();
=cut

sub each {
  my $polygon = shift @_;

  my @list;
  foreach my $line ($polygon->eachline()){
    my ($p1)=$line->points(); # Will get first point of line
    push(@list,$p1);
  }
  return @list;
}

#########
# split #
#########

=head1 split

Polygon intersection algorithm

    @split_polygons=$polygon1->split($polygon2);

Return values    Meaning
       0         No overlap
      -1         polygon2 completely lies in polygon1
      -2         polygon1 completely lies in polygon2

Otherwise return array of split polygons

=cut

sub split {
    my($polygon1,$polygon2)=@_;

    my $debug=0;
    if ($debug){
	print STDERR  "In Polygon::split with $polygon1 $polygon2\n\n";
	print STDERR "POLYGON 1";
	$polygon1->write();
	print STDERR "POLYGON 2";
	$polygon2->write();
    }

    my ($x_centre,$y_centre)=$polygon2->dual_vertex()->xy();

    warn "CENTRES $x_centre,$y_centre\n";

    # Short circuit if no overlap between polygons
    my $overlap=$polygon1->overlap($polygon2);
    if (! $overlap){
	print STDERR "In Polygon::split found no overlap\n";
	return 0;
    } elsif ($overlap == -1){
	warn "Completely overlapping polygons, 1 contains 2!\n";
	$polygon2->layers($polygon1->layers()+$polygon2->layers());	
	my @split_polygons=($polygon1,$polygon2);
	#return (@split_polygons);
	return -1;
    } elsif ($overlap == -2){
	warn "Completely overlapping polygons! 2 contains 1\n";
	$polygon1->layers($polygon1->layers()+$polygon2->layers());	
	my @split_polygons=($polygon2,$polygon1);
	#return (@split_polygons);
	return -2;
    }

    # Make output polygons
    my $outer_polygon1 = new Adult($x_centre,$y_centre,'1');
    my $outer_polygon2 = new Adult($x_centre,$y_centre,'1');
    my $inner_polygon  = new Adult($x_centre,$y_centre,'1');

    #############
    # Polygon 1 #
    #############
    foreach my $line ($polygon1->eachline()){
        # Check number of times line intersects with other polygon
	my @results=$polygon2->intersect($line);

        # Remove points on lines
	my @newresults;
	foreach my $thing (@results){
	    if ($thing != -1){
		push (@newresults,$thing);
	    }
	}
	@results=@newresults;

	my $intersections=@results;
	my $intersect_point=$results[0];

	if ($intersect_point == -1){
	    die;
	}


	my ($p1,$p2)=$line->points();

	my $inside1=$polygon2->contains_point($p1);
	my $inside2=$polygon2->contains_point($p2);

	print STDERR "Found $intersections intersections for $line and in outs of [$inside1] [$inside2]\n";

	if ($inside1 == -1 and $inside2 == -1 and $intersections == 0){
	    print STDERR "Better fix inouts!\n";
	    $inside1=$inside2=0;
	    print STDERR "NEW: Found $intersections intersections for $line and in outs of [$inside1] [$inside2]\n";
	} elsif ($inside1 == -1){
	    print STDERR "Better fix inouts!\n";
	    $inside1=($inside2-$intersections)**2;
	    print STDERR "NEW: Found $intersections intersections for $line and in outs of [$inside1] [$inside2]\n";
	} elsif ($inside2 == -1){
	    print STDERR "Better fix inouts!\n";
	    $inside2=($inside1-$intersections)**2;
	    print STDERR "NEW: Found $intersections intersections for $line and in outs of [$inside1] [$inside2]\n";
	}

        # Workout if odd or even number of intersectioons
	my $even_odd;
	if (int($intersections/2) == $intersections/2){ # Even
	    $even_odd=0;
	} else { # Odd
	    $even_odd=1;
	}


	if ($even_odd != ($inside1-$inside2)**2){
	    warn "********************\n";
	    $line->write();
	    $polygon2->write();
	    die "Your intersections = [$intersections] do not tally with in and outs!\n";
	}

	if ($inside1 and $inside2){
            # Seems to be a bug here when intersections == 0
	    if ($intersections == 0){
		die "Seems to be a bug with this case!\n";
	    }

            # catenate both line endpoints, and the intersection points. BTW number of intersection
            # points should be even!
	    my @point_array=($p1,$p2,@results);

	    my @datakeys;

	    my $gradient=$line->gradient();
	    if ($gradient>999 or $gradient<-999){
		# Order list of points by y coordinates.
		foreach my $point (@point_array){
		    push (@datakeys,$point->y());
		}
	    } else {
		# Order list of points by x coordinates.
		foreach my $point (@point_array){
		    push (@datakeys,$point->x());
		}
	    }
	    my @sort_data=@point_array[sort {$datakeys[$a] <=> $datakeys[$b]} $[..$#point_array];

            # Foreach point of array, check $flag and make new line and add to inner_polygon
            # and correct outer polygon.
	    my $n=@sort_data;

            # Keep state by using $flag=1 or $flag=-1, initialise $flag=1
	    my $flag=1;
	    my $i=1;
	    while ($i<$n){
		if ($flag == '1'){
		    print STDERR "Flag 1 Adding line to outer2 and inner\n";
		    my $line=new Line($sort_data[$i-1]->copy(),$sort_data[$i]->copy(),'1');
		    $outer_polygon2->add($line);
		    $inner_polygon->add($line->copy());
		} elsif ($flag == '-1'){
		    print STDERR "Flag -1 Adding line to outer1\n";
		    my $line=new Line($sort_data[$i-1]->copy(),$sort_data[$i]->copy(),'1');
		    $outer_polygon1->add($line);
		}
		$i++;
		$flag=$flag*(-1);

	    }
	} elsif (not $inside1 and not $inside2){
            # catenate both line endpoints, and the intersection points. BTW number of intersection
            # points should be even!
	    my @point_array=($p1,$p2,@results);

            # Order list of points by x coordinates.
	    my @datakeys;
	    foreach my $point (@point_array){
		push (@datakeys,$point->x());
	    }
	    my @sort_data=@point_array[sort {$datakeys[$a] <=> $datakeys[$b]} $[..$#point_array];

            # Foreach point of array, check $flag and make new line and add to inner_polygon
            # and correct outer polygon.
	    my $n=@sort_data;

            # Keep state by using $flag=1 or $flag=-1, initialise $flag=1
	    my $flag=1;
	    my $i=1;
	    while ($i<$n){
		if ($flag == '1'){
		    my $line=new Line($sort_data[$i-1]->copy(),$sort_data[$i]->copy(),'1');
		    $outer_polygon1->add($line);
		} elsif ($flag == '-1'){
		    #my $innerline1=new Line($sort_data[$i-1]->copy(),$sort_data[$i]->copy(),'1');
		    my $line=new Line($sort_data[$i-1]->copy(),$sort_data[$i]->copy(),'1');
		    $outer_polygon2->add($line);
		    $inner_polygon->add($line->copy());
		}
		$flag=$flag*(-1);
		$i++;
	    }
	} elsif (not $inside1 and $inside2 and $intersections == 1){
	    print STDERR "$p2,$intersect_point\n";

	    my $innerline1=new Line($p2->copy(),$intersect_point->copy(),'1');
	    my $innerline2=$innerline1->copy();
	    my $outerline =new Line($p1->copy(),$intersect_point->copy(),'1');

	    $outer_polygon2->add($innerline1);
	    $inner_polygon->add($innerline2);
	    $outer_polygon1->add($outerline);
	} elsif ($inside1 and not $inside2 and $intersections == 1){
	    my $innerline1=new Line($p1->copy(),$intersect_point->copy(),'1');
	    my $innerline2=$innerline1->copy();
	    my $outerline =new Line($p2->copy(),$intersect_point->copy(),'1');

	    $outer_polygon2->add($innerline1);
	    $inner_polygon->add($innerline2);
	    $outer_polygon1->add($outerline);
	} elsif ($intersections == 0){
	    die "Shouldn't really get here, but due to fudging it is possible!\n";
	} else {
	    foreach my $obj (@results){
		$obj->write();
	    }
	    die "EHHH In Polygon::split found a case I don't deal with @results [$intersections]\n";
	}
    }

    #############
    # Polygon 2 #
    #############
    foreach my $line ($polygon2->eachline()){
        # Check number of intersections with polygon
	my @results=$polygon1->intersect($line);

        # Remove points on lines
	my @newresults;
	foreach my $thing (@results){
	    if ($thing != -1){
		push (@newresults,$thing);
	    }
	}
	@results=@newresults;

	my $intersections=@results;
	my $intersect_point=$results[0];

	my ($p1,$p2)=$line->points();
	my $inside3=$polygon1->contains_point2($p1);
	my $inside4=$polygon1->contains_point2($p2);
#	my $inside3=$polygon1->contains_point($p1);
#	my $inside4=$polygon1->contains_point($p2);

	print STDERR "Found $intersections intersections for $line and in outs of [$inside3] [$inside4]\n";


	if ($inside3 == -1 and $inside4 == -1 and $intersections == 0){
	    print STDERR "Better fix inouts!\n";
	    $inside3=$inside4=0;
	    print STDERR "NEW: Found $intersections intersections for $line and in outs of [$inside3] [$inside4]\n";
	} elsif ($inside3 == -1){
	    print STDERR "Better fix inouts!\n";
	    $inside3=($inside4-$intersections)**2;
	    print STDERR "NEW: Found $intersections intersections for $line and in outs of [$inside3] [$inside4]\n";
	} elsif ($inside4 == -1){
	    print STDERR "Better fix inouts!\n";
	    $inside4=($inside3-$intersections)**2;
	    print STDERR "NEW: Found $intersections intersections for $line and in outs of [$inside3] [$inside4]\n";
	}

        # Workout if odd or even number of intersectioons
	my $even_odd;
	if (int($intersections/2) == $intersections/2){ # Even
	    $even_odd=0;
	} else { # Odd
	    $even_odd=1;
	}

	if ($even_odd != ($inside3-$inside4)**2){
	    warn "********************\n";
	    $line->write();
	    $polygon1->write();
	    die "Your intersections [$intersections] do not tally with in and outs!\n";
	}

	if ($inside3 and $inside4){
            # catenate both line endpoints, and the intersection points. BTW number of intersection
            # points should be even!
	    my @point_array=($p1,$p2,@results);

            # Order list of points by x coordinates.
	    my @datakeys;

	    my $gradient=$line->gradient();
	    if ($gradient>999 or $gradient<-999){
		# Order list of points by y coordinates.
		foreach my $point (@point_array){
		    push (@datakeys,$point->y());
		}
	    } else {
		# Order list of points by x coordinates.
		foreach my $point (@point_array){
		    push (@datakeys,$point->x());
		}
	    }
	    my @sort_data=@point_array[sort {$datakeys[$a] <=> $datakeys[$b]} $[..$#point_array];

            # Foreach point of array, check $flag and make new line and add to inner_polygon
            # and correct outer polygon.
	    my $n=@sort_data;

            # Keep state by using $flag=1 or $flag=-1, initialise $flag=1
	    my $flag=1;
	    my $i=1;
	    while ($i<$n){
		if ($flag == '1'){
		    print STDERR "Flag 1 Adding line to outer1 and inner\n";
		    my $line=new Line($sort_data[$i-1]->copy(),$sort_data[$i]->copy(),'1');
		    $outer_polygon1->add($line);
		    $inner_polygon->add($line->copy());
		} elsif ($flag == '-1'){
		    #my $innerline1=new Line($sort_data[$i-1]->copy(),$sort_data[$i]->copy(),'1');
		    print STDERR "Flag 1 Adding line to outer2\n";
		    my $line=new Line($sort_data[$i-1]->copy(),$sort_data[$i]->copy(),'1');
		    $outer_polygon2->add($line);
		}
		$flag=$flag*(-1);
		$i++;
	    }
	} elsif (not $inside3 and not $inside4){
            # catenate both line endpoints, and the intersection points. BTW number of intersection
            # points should be even!
	    my @point_array=($p1,$p2,@results);

            # Order list of points by x coordinates.
	    my (@datakeys);
	    foreach my $point (@point_array){
		my $x=$point->x();
		push (@datakeys,$x);

		#print STDERR "NN POINT $point\n";
		#$point->write();
	    }
	    my @sort_data=@point_array[sort {$datakeys[$a] <=> $datakeys[$b]} $[..$#point_array];

            # Foreach point of array, check $flag and make new line and add to inner_polygon
            # and correct outer polygon.
	    my $n=@sort_data;
	    #print STDERR "DATAKEYS",@datakeys,"\n";
	    #print STDERR "SORTDATA",@sort_data,"\n";

	    #print "N is $n\n";
            # Keep state by using $flag=1 or $flag=-1, initialise $flag=1
	    my $flag=1;
	    my $i=1;
	    while ($i<$n){
		if ($flag == '1'){
		    my $line=new Line($sort_data[$i-1]->copy(),$sort_data[$i]->copy(),'1');
		    $outer_polygon2->add($line);
		    #print STDERR "Writing line to polygon2";
		    $line->write();
		} elsif ($flag == '-1'){
		    my $line=new Line($sort_data[$i-1]->copy(),$sort_data[$i]->copy(),'1');
		    $outer_polygon1->add($line);
		    $inner_polygon->add($line->copy());
		}
		$flag=$flag*(-1);
		$i++;
	    }
	} elsif (not $inside3 and $inside4 and $intersections == 1){
	    my $innerline1=new Line($p2->copy(),$intersect_point->copy(),'1');
	    my $innerline2=$innerline1->copy();
	    my $outerline =new Line($p1->copy(),$intersect_point->copy(),'1');

	    $outer_polygon1->add($innerline1);
	    $inner_polygon->add($innerline2);
	    $outer_polygon2->add($outerline);
	} elsif ($inside3 and not $inside4 and $intersections == 1){
	    my $innerline1=new Line($p1->copy(),$intersect_point->copy(),'1');
	    my $innerline2=$innerline1->copy();
	    my $outerline =new Line($p2->copy(),$intersect_point->copy(),'1');

	    $outer_polygon1->add($innerline1);
	    $inner_polygon->add($innerline2);
	    $outer_polygon2->add($outerline);
	} elsif ($intersections == 0){
	    die "Shouldn't really get here, but due to fudging it is possible!\n";
	} else {
	    foreach my $obj (@results){
		$obj->write();
	    }
	    die "In Polygon::split found a case I don't deal with @results [$intersections]\n";
	}
    }

    #####################################
    # Correct depth for overlap polygon #
    #####################################
    if ($polygon1->layers){
	$outer_polygon1->layers($polygon1->layers);
    } else {
	warn "Polygon $polygon1 has no layers!\n";
	$outer_polygon1->layers(1);
    }

    if ($polygon2->layers){
	$outer_polygon2->layers($polygon2->layers);
    } else {
	warn "Polygon $polygon2 has no layers!\n";
	$outer_polygon2->layers(1);
    }


    my $inner_layers=$outer_polygon1->layers()+$outer_polygon2->layers();
    warn "*** inner polygon has $inner_layers layers\n";
    $inner_polygon->layers($inner_layers);

    my @split_polygons;
    push (@split_polygons,$outer_polygon1->regularise());
    push (@split_polygons,$outer_polygon2->regularise());
    push (@split_polygons,$inner_polygon->regularise());
    return (@split_polygons);
}

###########
# overlap #
###########

=head1 overlap

Will detect if two polygons have any overlap. 

Returns 1 for partial overlap of polygons
        0 for no overlap
       -1 for complete overlap. Polygon1 completely contains polygon2.
       -2 for complete overlap. Polygon2 completely contains polygon1.

    $polygon1->overlap($polygon2);

=cut

sub overlap {
    my ($p1,$p2)=@_;

    # Intersection tests
    foreach my $line ($p1->eachline()){
	foreach my $test_line ($p2->eachline()){
	    my $result=$line->intersect($test_line);
	    if ($result == -1){next;} elsif ($result == 0){next} else {return 1}
	}
    }

    print STDERR "In overlap found no intersections!\n";

    # If we got here then either all points should be inside or outside of the
    # polygon

    # Containment tests
    foreach my $point ($p1->each()){
	#Polygon 2 completely contains polygon 1!
	if ($p2->contains_point2($point) == 1){
	    return -2;}
	elsif ($p2->contains_point2($point) == -1){next;}
    }
    foreach my $point ($p2->each()){
	#Polygon 1 completely contains polygon 2!
	if ($p1->contains_point2($point) == 1){return -1}
	elsif ($p1->contains_point2($point) == -1){next;}
    }
    # If we got here no overlap
    return 0;
}


##############
# regularise #
##############

=head1 regularise

When using the split method some polygons become unordered and split into
multiple unnconnected polygons.  This method will take one of these and
reorder it and return extra polygons if needed.

    @extra_polygons=$polygon->regularise();

=cut

sub regularise {
    my $polygon=shift @_;

    my ($x_centre,$y_centre)=$polygon->dual_vertex()->xy();

    print STDERR "\nBefore regularisationwith $polygon";
    $polygon->write();
    my @lines=$polygon->eachline();

    # Strip out very short lines which could cause problems
    my @templines;
    foreach my $line (@lines){
	my ($p1,$p2)=$line->points();
	if (! $p1->distance($p2)<1){
	    push(@templines,$line);
	} else {
	    warn "Stripped out short line $line\n";
	}
    }
    @lines=@templines;
    my @new_polygons;

    if (! @lines){
	return @new_polygons;
    }

    my $first_line=shift @lines;
    my @newlines;
    push (@newlines,$first_line);

    my ($firstpoint,$lastpoint)=$first_line->points();

    my $num_lines;
    while (@lines){
        my $old_num_lines=$num_lines;
	$num_lines=@lines;

	if ($old_num_lines == $num_lines){
	    die "Got stuck!\n";
	}

	# Check if last point joins to first point
	# If so and no more lines then exit
	# Else make a new polygon object and start again with next line
	my $i;
	print STDERR "In regularise @lines\n";
	my $found;
	for ($i=0; $i<$num_lines; $i++){
	    my $current_line=$lines[$i];
	    $found=0;
#	    print STDERR "CURRENT LINE IS $current_line\n";
	    my ($p1,$p2)=$current_line->points();

	    my $dist1=$p1->distance($lastpoint);
#	    print STDERR "Distance to point is $dist1\n";
	    if ($dist1<1){
	        $found=1;
		push(@newlines,splice(@lines,$i,1)); # Add onto fixed lines!
		$lastpoint=$p2;
		print STDERR "Adding $current_line line! 1\n";

		if ($firstpoint->distance($lastpoint)<1){
		    # Finished polygon
		    my $new_polygon=new Adult($x_centre,$y_centre,1);
		    $new_polygon->layers($polygon->layers());
		    $new_polygon->add(@newlines);

		    warn "Finished polygon!\n";
		    $new_polygon->write();
		    push(@new_polygons,$new_polygon);

		    if (! $new_polygon->eachline()){
			die "$new_polygon has no lines!\n";
		    }
		    
		    # If there are any lines left need to start a new polygon
		    if (@lines){
			print STDERR "Haven't finished yet. Reinitialising stuff\n";
			$first_line=shift @lines;
			@newlines=();
			push (@newlines,$first_line);
			($firstpoint,$lastpoint)=$first_line->points();
		    }
		}
		last;
	    } elsif ($p2->distance($lastpoint)<1){
	        $found=1;
		$current_line->swap();
		push(@newlines,splice(@lines,$i,1));
		#my $newline=new Line($p2,$p1,1);
		$lastpoint=$p1;
		#splice(@lines,$i,1);
		#push (@newlines,$newline);
		print STDERR "Adding $current_line line! 2\n";

		if ($firstpoint->distance($lastpoint)<1){
		    # Finished polygon
		    my $new_polygon=new Adult($x_centre,$y_centre,1);
		    $new_polygon->layers($polygon->layers());
		    $new_polygon->add(@newlines);
		    warn "Finished polygon!\n";
		    $new_polygon->write();
		    push(@new_polygons,$new_polygon);

		    
		    # If there are any lines left need to start a new polygon
		    if (@lines){
			print STDERR "Haven't finished yet. Reinitialising stuff\n";
			$first_line=shift @lines;
			@newlines=();
			push (@newlines,$first_line);
			($firstpoint,$lastpoint)=$first_line->points();
		    }
		}
		last;
	    }
	}
	if (! $found){
  	    warn "Found nothing this round!\n";
	}
    }
    return @new_polygons;
}


#############
# intersect #
#############

=head1 intersect

@intersects=$polygon->intersect($line);

will return $point objects for each
line in the polygon that intersects with the argument line.

=cut

sub intersect {
    my($polygon,$testline)=@_;

    my @results;
    foreach my $line ($polygon->eachline()){

	if ($testline==$line){
	    warn "In Polygon::intersect testline is in polygon! Skipping comparison!\n";
	    next;
	}

	my $point=$line->intersect($testline);

	if ($point == 0){
	    # warn "No intersection! Skip ...\n";
	} elsif ($point == -1){
	    #warn "Odd result in Polygon::intersect got -1 result!\n";
	    return -1;
	} elsif ($point){
	    push(@results,$point);
	}
    }

    #my $n=@results;
    #if ($n>1){warn "Found $n for this polygon @results!\n";}
    return @results;
}

#######
# cog #
#######

=head1 cog

Finds centre of gravity of polygon and returns point object

  $cog=$self->cog();

=cut

sub cog {
    my($self)=@_;

    my $n; # Number of lines
    my $x_tot=0;
    my $y_tot=0;

    foreach my $line ($self->eachline()){
      $n++;
      my $p1=$line->points();
      my ($x,$y)=$p1->xy();
      $x_tot=$x_tot+$x;
      $y_tot=$y_tot+$y;
    }

    my $point=new Point($x_tot/$n,$y_tot/$n);
    return $point;
}


###############
# dual_vertex #
###############

=head1 dual_vertex

Returns dual tililngs vertex that corresponds with this polygon.
Returns this as a point object.  This point is used as the centre
of scaling and rotation.

Get or set method

 $point=$polygon->dual_vertex();
 $polygon->dual_vertex($point);

=cut

sub dual_vertex {
    my($self,$centre)=@_;

    if (defined $centre){
      $self->{'centre'}=$centre;
    } else {
      $centre=$self->{'centre'};
    }
    return $centre;
}

#######
# add #
#######
=head1 add
Method to add line objects to a polygon object

$polygon->add($line1,$line2...);

=cut
sub add {
  my $polygon=shift @_;
  push(@{$polygon->{'lines'}},@_);
}

###################
# adjacent_points #
###################
=head1 adjacent_points

Method to find the two adjacent points in a polygon

($point1,$point2)=$polygon->adjacent_points($point);

=cut
sub adjacent_points {
  my ($polygon,$point)=@_;

  # Check if $point is in polygon!
  my ($i,$found);

  my(@points) = $polygon->each();
  my($n_sides) = @points;
  for ($i=0;$i<$n_sides;$i++){
    if ($point eq $points[$i]){
      # This is the one
      $found=1;
      last;
    }
  }
  if ($found){
    # Return adjacents
    my $point1 = $points[$i-1%$n_sides];
    my $point2 = $points[$i+1%$n_sides];
    return ($point1,$point2);
  } else {
    die "The point you passed to adjacent_points is not in polygon!\n";
  }
}

##################
# contains_point #
##################
=head1 contains_point

Method to find if a point is within a polygon. Returns true if point is
in polygon

if ($polygon->contains_point($point)){
}

=cut
sub contains_point {
  my ($polygon,$testpoint) = @_;
  my @pointarray = $polygon->each();
  my @linearray  = $polygon->eachline();

  my $length = @linearray;

  my $i = 0;
  while($i<$length){
    my $point1 = $pointarray[($i)%$length];#
    my $point2 = $pointarray[($i+1)%$length];#
    my $point3 = $pointarray[($i+2)%$length];

    # Find gradient between points 1 and 2
    my $gradient  = ($point2->y()-$point1->y())/($point2->x()-$point1->x());#
    my $intercept = $point1->y()-$gradient*$point1->x();#
#    my $gradient  = $linearray[$i]->gradient();
#    my $intercept = $linearray[$i]->intercept();

    # If line is horizontal gradient calculation breaks down
    if (($point1->x() eq $point2->x())){#
      die "Can't calculate gradient for this pair\n";#
    }#

    # Now check if test point and point 3 are on same side of line
    # For derivation see rough book page 19.
    unless (($gradient*$point3->x()+$intercept-$point3->y())*($gradient*$testpoint->x()+$intercept-$testpoint->y())>0){
      #print STDERR "Point is outside Polygon\n";
      return 0;
    }
    $i++;
  }
  return (1);
}


###################
# contains_point2 #
###################
=head1 contains_point2

Method to find if a point is within a polygon. 

Returns 1 if point is in polygon
Returns 0 if point is outside polygon
Returns -1 if point appears to be on edge of polygon

This uses the odd-even rule!

$polygon->contains_point2($point)

=cut
sub contains_point2 {


  die "This routine seems to be seriously bugged! In some obvious cases it seems to call a point as outside polygon!";

  my ($polygon,$testpoint,$rotation) = @_;

  # Make a new line - this is test ray!
  my $y=$testpoint->y();
  my $dummy=new Adult(0,0,1);
  my $p1=new Point(-1000000,$y,$dummy);
  my $testline=new Line($p1,$testpoint);

  if ($rotation){
      $testline->rotate($rotation,$testpoint->x(),$y),
  }

  # Count intersections
  my ($total_intersects,$n);
  #print STDERR "-----------\n";
  foreach my $line ($polygon->eachline()){
      #print STDERR "\nLooking at next line\n";
      my @result=$line->intersect($testline);

      if ($result[0] == 0){
	  next;
      } elsif ($result[0] == -1){ # It cannot be determined whether point is inside or outside
	  return (-1);
	  warn "Uh oh, possible error in Line::intersect that might cause incorrect result in Polygon::contains_point2!\n";
	  print STDERR "Oh, dodgy attempt in contains_polygon2, rotating by ",$rotation+2,"\n";
	  if ($rotation == 10){
	      warn "Oh dear rotating is making no difference! return -1\n";
	      return -1;
	  }
	  my $done=$polygon->contains_point2($testpoint,$rotation+2);
	  return $done;
      }
	  
      my ($p1,$p2)=$line->points();
      # Check if point is very close to any of the lines
      my $newline1= new Line($p1,$testpoint);
      my $newline2= new Line($testpoint,$p2);
      my $grad1=$newline1->gradient();
      my $grad2=$newline2->gradient();
      
      if (($grad1-$grad2)**2 < 0.1){
	  warn "Test point is very close line. In contains_point2 may give an error\n";
	  print STDERR "Oh, dodgy attempt in contains_polygon2, rotating by ",$rotation+2,"\n";
	  if ($rotation == 10){
	      warn "Oh dear rotating is making no difference! return -1\n";
	      return -1;
	  }
#	  my $done=$polygon->contains_point2($testpoint,$rotation+2);
	  my $done=$polygon->contains_point2($testpoint,$rotation+2);
	  return $done;
      } else {
	  #print STDERR "Gradients are different $grad1 , $grad2\n";
      }
      
      # Check if result is close to either point in line.
      # If this is the case we can't be certain if this routine will
      # give the correct answer.
      foreach my $intersect_point (@result){
	  if ($intersect_point->distance($p1)<1){
	      #die;
	      # Rotate testline by ten degrees and try again
	      print STDERR "Oh, dodgy attempt in contains_polygon2, rotating\n";
	      $testline->rotate(10,$testpoint->x(),$y),
	  } elsif ($intersect_point->distance($p2)<1){
	      #die;
	      # Rotate testline by ten degrees and try again
	      print STDERR "Oh, dodgy attempt in contains_polygon2, rotating\n";
	      $testline->rotate(10,$testpoint->x(),$y),
	  }
      }
      
      $n=@result;
      $total_intersects=$total_intersects+$n;
  }
  
  #print STDERR "\nTOTAL INTERSECTS $total_intersects\n\n";

  if ($total_intersects == 0){ # point is outside
      #print STDERR "Point is outside!\n";
      return 0;
  } else {
      #print STDERR "Total intersects is $total_intersects\n";
  }
  
  my $result=(-1)**$total_intersects;
  #print STDERR "[$result] $total_intersects\n";
  if ($result==1){ # Point is outside
      return 0;
  } elsif ($result == -1){ # Point is inside
      return 1;
  }
  die "Polygon::overlap, Unreachable code\n";
}



###################
# contains_point3 #
###################
=head1 contains_point3

Method to find if a point is within a polygon. Experimental
routine looking for large speed up.

Returns 1 if point is in polygon
Returns 0 if point is outside polygon

This uses the odd-even rule!

$polygon->contains_point3($point)

=cut
sub contains_point3 {
  my ($polygon,$testpoint) = @_;
#  die "This routine is very broken!\n";
  # Make a new line - this is test ray!
  my $y=$testpoint->y();

  my $total; # Counts intersections
  foreach my $line ($polygon->eachline()){
  # Check whether line crosses ray

      my ($p1,$p2)=$line->points();
      if ($p1->y() > $y and $p2->y() < $y){
           # Yes they intersect
	  $total++;
      } elsif ($p2->y() > $y and $p1->y() < $y){
           # Yes they intersect
	  $total++;
      } else {
           # No lines don't intersect
      }
  }

  # Now check whether total is odd (in polygon) or even (outside polygon)
  print STDERR "$total\n";
  if (int($total/2) == $total/2){
      # Must be even
      return 0;
  } else {
      # Must be odd
      print STDERR "Got here\n";
      return 1;

  }
}


####################
# contains_polygon #
####################
=head1 contains_polygon

Method to find if any point of a polygon is within a polygon.
Returns true if any point is in polygon (or on edge of polygon!).

if ($polygon->contains_polygon($polygon2)){
}

=cut
sub contains_polygon {
  my ($polygon,$testpolygon) = @_;

  my $debug=0;
  if ($debug){
      print STDERR "Checking if $polygon contains $testpolygon\n";
      $polygon->write();
      $testpolygon->write();
      print STDERR "____________________\n";
  }

  foreach my $point ($testpolygon->each()){
    # Check is in Polygon
    #my $result2=$polygon->contains_point2($point);
    my $result=$polygon->contains_point($point);

    #print "     Testing $point. Result is $result or result2 is $result2\n";
#    $point->write();

    if ($result){ # This will return true if -1 or 1 is returned!

	if ($debug){
	    my $result=$polygon->contains_point2($point);
	    print STDERR "Yes polygon inside: $result\n";
	}
      return 1;
    }
  }
  if ($debug){
      print STDERR "No polygon outside\n";
  }
  return 0;
}

#########
# write #
#########
=head1 write

Debugging method: write out polygon data to STDERR

$polygon->write();

=cut
sub write {
  my $polygon = shift @_;
  
  my @lines=$polygon->eachline();
  my $number_sides=@lines;

  my $layers=$polygon->layers;


  my $centre=$polygon->{'centre'};
  print STDERR "\nwriting $polygon with $number_sides sides. Origin point object:\n";
  $centre->write();

  if ($layers){print STDERR " $layers Layers.\n";}

  foreach my $line (@lines){
    $line->write();
  }
  print STDERR "\n";
}


###############
# shared_edge #
###############
=head1 shared_edge

Method to find if two polygons share an edge.  If they do will create an edge baby object;

$polygon1->shared_edge($polygon2);

=cut
sub shared_edge {
  my ($polygon1,$polygon2) = @_;

  if ($polygon1 == $polygon2){
      return 0;
  }

  # Do proximity checks
  my $do_proximity=0;
  if ($do_proximity){
      my @points1=$polygon1->each();
      my @points2=$polygon2->each();
      
      my $proximal=0;
      foreach my $point (@points1){
	  if ($point->proximal(@points2)){
	      $proximal=1;
	      last;
	  }
      }
      if (! $proximal){
	  return 0; # Polygons are not close!
      }
  }


  my $thresh=1; # min distance to be considered same point
  foreach my $line ($polygon1->eachline()){
    foreach my $test_line ($polygon2->eachline()){
	my ($p1,$p2)=$line->points();
	my ($p3,$p4)=$test_line->points();

     if ($p1->distance($p3)<$thresh and $p2->distance($p4)<$thresh){
	my $line1=new Line($p1,$p2,'1');
	my $line2=new Line($p2,$p4,'1');
	my $line3=new Line($p4,$p3,'1');
	my $line4=new Line($p3,$p1,'1');

	my $mpoint=$line1->midpoint();

	my $polygon=new Edge($mpoint->x(),$mpoint->y());
	$polygon->add($line1,$line2,$line3,$line4);
	return $polygon;
      } elsif ($p2->distance($p3)<$thresh and $p1->distance($p4)<$thresh){
	my $line1=new Line($p1,$p4,'1');
	my $line2=new Line($p4,$p3,'1');
	my $line3=new Line($p3,$p2,'1');
	my $line4=new Line($p2,$p1,'1');

	my $mpoint=$line1->midpoint();

	my $polygon=new Edge($mpoint->x(),$mpoint->y());
	$polygon->add($line1,$line2,$line3,$line4);
	return $polygon ;
      }
    }
  }
}

1;  # says use was ok

__END__

