=head1 NAME

PolygonSet

=head1 DESCRIPTION

B<PolygonSet> contains a collection of polygon objects. Currently now
only used for vertex babies.

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for PolygonSet
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package PolygonSet;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Polygon;
use Adult;
use Line;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );


#######
# new #
#######

=head1 new

Creates PolygonSet object. This contains many attributes.
The weave attribute can be left or right.

$polygonset = new PolygonSet($theta,$alpha$n1,$n2,$type);

=cut
sub new {
    my ($class,$theta,$alpha,$n1,$n2,$colour)=@_;

    my $set = {"polygons"      => [],
	       "current_theta" => $theta,
	       "current_alpha" => $alpha,
	       "n1"            => $n1,     # Polygon 1 
	       "n2"            => $n2,     # Polygon 2
	       "colour"        => $colour, # Only used by weave tessellations, colour1, colour2 etc
	       };

    if (! $set->{'colour'} =~ /colour/){
      die "Sorry you must specify colour for polygonset not ",$set->{'colour'},"\n";
    }

    bless $set, $class;
    return $set;
}


########
# copy #
########

=head1 copy

Method to copy a polygonset object

    $new_set = $set->copy();

=cut
sub copy {
    my ($set)=@_;

    my $newset= new PolygonSet($set->{'current_theta'},$set->{'current_alpha'},$set->{'n1'},$set->{'n2'},$set->{'colour'});

    foreach my $object ($set->each()){
      my $new_object=$object->copy();
      $newset->add($new_object);
    }
    return $newset;
}




################
# set_polygon1 #
################

=head1 set_polygon1

Used to set polygon1 transform parameter. This holds number
of sides of polygon. This is a get set method.

Set

    $polygonset->set_polygon1("$n1");

Get

    $n1=$polygonset->set_polygon1();

=cut
sub set_polygon1 {
  my $polygonset = shift @_;
  my $n1 = shift @_;

  if ($n1){
    $polygonset->{'n1'}=$n1;
  } else {
    $n1=$polygonset->{'n1'};
  }
  if (! $n1){die "No $n1 set in set_polygon1\n";}
  return $n1;
}

################
# set_polygon2 #
################

=head1 set_polygon2

Used to set polygon2 transform parameter. This holds number
of sides of polygon. This is a get set method.

Set

$polygonset->set_polygon2("n2");

Get
$n2=$polygonset->set_polygon2();

=cut

sub set_polygon2 {
  my $polygonset = shift @_;
  my $n2 = shift @_;

  if ($n2){
    $polygonset->{'n2'}=$n2;
  } else {
    $n2=$polygonset->{'n2'};
  }
  return $n2;
}


##########
# colour #
##########

=head1 colour

Get method to tell what colour set is 

    $colour=$polygonset->colour();

=cut

sub colour {
  my $self= shift @_;
  my $colour=$self->{'colour'};

  return $colour;
}


########
# each #
########

=head1 each

Loops over polygonset and returns a list of polygon objects

    @polygons = $polygonset->each();

=cut

sub each {
  my($polygonset)=@_;
  my @list=@{$polygonset->{'polygons'}};
  return @list;
}


##########
# instar #
##########

=head1 instar

Takes a polygon and insets and rotates smaller version, to create
an inset star, or instar.

$angle is angle in degrees to rotate inset polygon

    $instar_set = instar PolygonSet($polygon,$angle,$shrink,$n1,$n2);

=cut

sub instar {
  my ($class,$polygon,$shrink,$n1,$n2)=@_;

  die "Sorry instar method is not yet working!";
  my $set = new PolygonSet(0,1,$n1,$n2,'colour1');

}


############
# new_star #
############

=head1 new_star

Creates set of polygons that form a star.  This includes a 
central n-gon surrounded by n triangles

    $star_set = new_star($n,$angle,$n1,$n2,$creases,$alternate);

$n is number of sides
$angle is angle in degrees
$creases contains : separated list of creases for triangle
$alternate should be 'alternate' to swap crease assignment of triangles 

First point is vertically above centre.  The central n-gon
has its first side horizontal. The edges have length 100 units.

=cut

sub new_star {
    my ($n,$angle,$n1,$n2,$creases,$alternate)=@_;
    my $set = new PolygonSet(0,1,$n1,$n2,'colour1');

    my ($crease1,$crease2,$crease3)=split(/:/,$creases);

    my ($colour,$central_colour);
    if ($crease1 == 1 and $crease2 == 1 and $crease3 == 1){
	$colour=1;
	$central_colour=2;
    } elsif ($crease1 == -1 and $crease2 == -1 and $crease3 == -1){
	$colour=2;
	$central_colour=1;
    } else {
	$colour='3a';
	$central_colour='3b';
    }

    # Convert angle to radians
    my $pi = atan2(1,1)*4;
    $angle=$angle*$pi/180;

    my $ngon= new Adult(0,0,$central_colour);

    # Find first point
    my $x=100*sin($angle/2);
    my $y=$x*sqrt(1/(sin($pi/$n)*sin($pi/$n))-1);
    my $point = new Point(-$x,$y,$ngon);

    my @inner_points;
    push (@inner_points,$point);

    # Rotate point to make central Polygon.
    my $i=1;
    while ($i<$n){
	my $new_point=$point->copy();
	$new_point->rotate((360*$i)/$n);
	push (@inner_points,$new_point);
	$i++;
    }

    ###################################
    # Make polygon from @inner_points #
    ###################################
    $i=0;
    while($i<$n){
      my $point1 = $inner_points[$i];
      my $point2 = $inner_points[($i+1)%$n]->copy();

      my $crease=-1*$crease2;
      if ($alternate eq 'alternate'){
	  $crease=$crease2**$i;
      }

      my $line= new Line($point1,$point2,$crease);
      $ngon->add($line);
      $i++;
    }

    $set->add($ngon);

    # Now make outer points to make triangles
    # Find first point
    $y=100*cos($angle/2)+$x*sqrt(1/(sin($pi/$n)*sin($pi/$n))-1);

    # Make new polygon object
    my $outer_polygon = new Adult(0,0,$central_colour);  # This polygon is just a temporary object
    my $outer_point = new Point(0,$y,$outer_polygon);
    my @outer_points;
    push (@outer_points,$outer_point);
    $i=1;
    while ($i<$n){
      my $new_point=$outer_point->copy();
      $new_point->rotate((360*$i)/$n);
      push (@outer_points,$new_point);
      $i++;
    }

    # Now make triangles from Inner and outer polygon
    $i=1;
    while ($i<=$n){
      my $triangle = new Adult(0,0,$colour);      

      my $point1=$outer_points[$i%$n]->copy($triangle);
      my $point2=$inner_points[$i-1%$n]->copy($triangle);
      my $point3=$inner_points[$i%$n]->copy($triangle);

      my $line1=new Line($point2,$point1->copy(),$crease1);
      my $line2=new Line($point3,$point2->copy(),$crease2);
      my $line3=new Line($point1,$point3->copy(),$crease3);


      $triangle->add($line1,$line2,$line3);

      if (defined $alternate and $alternate eq 'alternate' and (-1)**$i>0){
	  $triangle->reverse();
      }

      # Add centre to triangle
      $triangle->find_centre2();

      $set->add($triangle);
      $i++;
    }
    return $set;
}


################
# new_star_dev #
################

=head1 new_star_dev

Creates set of polygons that form a star.  This includes a 
central n-gon surrounded by n triangles
$n is number of sides
$angle is angle in degrees
First point is vertically above centre.  The central n-gon
has its first side horizontal. The edges have length 100 units.

    ($centre_set,$star_set) = new_star($n,$angle,$rotation);

=cut

sub new_star_dev {
    my($n,$angle,$theta,$alpha,$n1,$n2,$colour)=@_;
    my ($i,$line,$x,$y,$point);

    my $set  = new PolygonSet($theta,$alpha,$n1,$n2,'colour1');
    my $set2 = new PolygonSet($theta,$alpha,$n1,$n2,'colour2');

    # Convert angle to radians
    my $pi = atan2(1,1)*4;
    $angle=$angle*$pi/180;


    my $ngon= new Adult(0,0);
    # Find first point
    $x=100*sin($angle/2);
    $y=$x*sqrt(1/(sin($pi/$n)*sin($pi/$n))-1);
    $point = new Point(-$x,$y,$ngon);
    $ngon->add($point);

    # Rotate point to make central Polygon.
    my ($new_point);
    $i=1;
    while ($i<$n){
	$new_point=$point->copy();
	$new_point->rotate((360*$i)/$n);
	$ngon->add($new_point);
	$i++;
    }
    my(@inner_points)  = @{$ngon->{'points'}};
    $set2->add($ngon);

    # Now make outer points to make triangles
    # Find first point
    $y=100*cos($angle/2)+$x*sqrt(1/(sin($pi/$n)*sin($pi/$n))-1);

    # Make new polygon object
    my $outer_polygon = new Adult(0,0,'1'); # This polygon is just a temporary object
    my $outer_point = new Point(0,$y,$outer_polygon);
    $outer_polygon->add($outer_point);
    $i=1;
    while ($i<$n){
      $new_point=$outer_point->copy();
      $new_point->rotate((360*$i)/$n);
      $outer_polygon->add($new_point);
      $i++;
    }

    my(@outer_points) = @{$outer_polygon->{'points'}};

    # Now make triangles from Inner and outer polygon
    $i=1;
    while ($i<=$n){
      my $triangle = new Adult(0,0);      

      my $point1=$outer_points[$i%$n]->copy($triangle);
      my $point2=$inner_points[$i-1%$n]->copy($triangle);
      my $point3=$inner_points[$i%$n]->copy($triangle);

      my $line1=new Line($point1,$point2->copy(),1);
      my $line2=new Line($point2,$point3->copy(),1);
      my $line3=new Line($point3,$point1->copy(),1);

      $triangle->add($line1);
      $triangle->add($line2);
      $triangle->add($line3);

      # Add centre to triangle
      # $triangle->find_centre();
      $triangle->find_centre2();

      $set->add($triangle);
      $i++;
    }
    return ($set2,$set);
}




#############
# decompose #
#############

=head1 decompose

Takes n-gon object and decomposes into n triangles.
Probably need to look at crease of each line to get correct creases.

$set->decompose($polygon,$handedness);

Handedness says whether layers swirl to 'left' or 'right'

=cut

sub decompose {
  my ($set,$polygon,$handedness) = @_;

  if (! $handedness){
      $handedness = 'left';
  }

  my $x=$polygon->dual_vertex()->x();
  my $y=$polygon->dual_vertex()->y();


  my @points = $polygon->each();
  my @lines=$polygon->eachline();

  my @triangles; # Store decomposed triangles
  my @theta; # Need these angles to find dual vertices of decomposed tiles
  my @orthogonal_lines; # Stores lines on which centres of decomposed triangles will lie

  my $n=@points;
  my $i;
  for ($i=0; $i<$n; $i++){
    # Centre of scaling is wrong here!
    my $triangle = new Adult(0,0,'1');

    my $crease=$lines[$i]->crease();
    my $next_crease=$lines[($i+1)%$n]->crease();

    my $point1 = new Point ($x,$y,$triangle);
    my $point2 = $points[$i]->copy($triangle);
    my $point3 = $points[($i+1)%$n]->copy($triangle);

    my ($line1,$line3);
    if ($handedness eq 'right'){
	$line1=new Line($point1,$point2->copy($triangle),$crease);
	$line3=new Line($point3,$point1->copy($triangle),-1*$next_crease);
    } elsif ($handedness eq 'left'){
	$line1=new Line($point1,$point2->copy($triangle),$crease);
	$line3=new Line($point3,$point1->copy($triangle),-1*$next_crease);
    } else {
	die "In PolygonSet::decompose don't recognise handedness [$handedness]\n";
    }
#    my $line2=new Line($point2,$point3->copy($triangle),$crease); # Outer line
    my $line2=new Line($point2,$point3->copy($triangle),$crease); # Outer line

    $triangle->add($line1);
    $triangle->add($line2);
    $triangle->add($line3);

    if ($crease == 1){
	$triangle->colour(1);
    } elsif ($crease==-1){
	$triangle->colour(2);
    }


    # Make a line orthogonal to outside line of triangle $line2
    my $new_line = Line::orthogonal($line2,$polygon->dual_vertex());
    push (@orthogonal_lines,$new_line);

    # If this is the first orthogonal line scale by some factor
    if ($i == 0){
	my $factor=0.5;
	$new_line->scale($factor,$x,$y);
    }

    #$triangle->add($new_line);
    #$triangle->write();

    # Will need to store this. Also need angles
    my $theta1=$new_line->angle($line1);
    my $theta2=-180+$line3->angle($new_line);

    push (@theta,$theta1,$theta2);
    push (@triangles,$triangle);

    # Centre of scaling is corrected here!
    #$triangle->find_centre();
    # $triangle->find_centre2();  
    $set->add($triangle);
  }

  # Now fix all centre points
  $n=@triangles;

  my $pi = atan2(1,1)*4;
  $i=1;
  my @scale;
  while ($i<=$n){
      # Calculate scales for each centre
      if ($i == 1){
	  $scale[$i]=1;
      } else {
	  my $theta3=$pi*$theta[((($i-1)*2)-1)]/180;
	  my $theta4=$pi*$theta[(($i-1)*2)]/180;

	  my $temp_scale=$scale[$i-1]*cos ($theta3)/cos ($theta4);
	  if ($temp_scale<0){
	      die "Scale is less than zero! Should never happen. previous scale is $scale[$i-1], thetas are $theta[((($i-1)*2)-1)] and $theta[(($i-1)*2)] cos thetas are ",cos ($theta[((($i-1)*2)-1)])," and ",cos ($theta[(($i-1)*2)]),"\n";
	  }
	  $scale[$i]=$temp_scale;
      }
      $i++;
  }

  # OK now have all scales to get dual vertices
  # Loop one more time to do all scales
  $i=1;
  my $length=$orthogonal_lines[0]->length();
  while ($i<=$n){
      # Calculate scales for each centre
      my $current_line=$orthogonal_lines[$i-1];
      my $current_length=$current_line->length();

      # Scale to be same length as first orthogonal line
      $current_line->scale($length/$current_length,$x,$y);

      # Now apply scale factor
      $current_line->scale($scale[$i],$x,$y);
      my ($point1,$point2)=$current_line->points();

      # We now have the correct origin for the triangle!
      $triangles[$i-1]->dual_vertex($point2);
      $i++;
  }
  return $set;
}

####################
# double_decompose #
####################

=head1 double_decompose

Takes n-gon object and decomposes twice into 3n triangles.

$set->double_decompose($polygon);

=cut

sub double_decompose {
  my ($set,$polygon) = @_;

  my $tempset = new PolygonSet(0,1,4,4,'TWIST','colour1');# Might cause bugs!
  $tempset->decompose($polygon);
  foreach my $newpolygon ($tempset->each()){
    my $tempset2 = new PolygonSet(0,1,4,4,'TWIST','colour1');# Might cause bugs!
    $tempset2->decompose($newpolygon);
    $set->add_set($tempset2);
  }
  return $set;
}



###########
# crystal #
###########

=head1 crystal

Takes n-gon object and decomposes by insetting a smaller version of the
polygon and join up the vertices. Returns central polygon object and set
of surrounding objects. #Must pass in the context of the tiling as $n1 and $n2.

($set)= crystal PolygonSet($polygon,$n1,$n2,$reduction1 .. $reductionn);

$reduction How much to shrink inset polygon

=cut

sub crystal {
  my $class = shift @_;
  my $polygon = shift @_;
  my $n1 = shift @_;
  my $n2 = shift @_;

  # Make array of creases based on polygons
  my @creases;
  foreach my $line ($polygon->eachline()){
      push (@creases,$line->crease());
  }

  warn "You need to sort out creases for crystal geometries\n";

  my $set = new PolygonSet(0,1,$n1,$n2,'colour1');
  my $inset_polygon;

  while (@_){
    my $shrink=shift @_;
    if ($shrink<0 || $shrink >1){
      die "Sorry \$shrink not specified in PolygonSet.pm crystal method\n ";
    }
    print STDERR "SHRINK:$shrink\n";

    $inset_polygon=$polygon->copy();
    $inset_polygon->scale($shrink);

    my @points = $polygon->each();

    my @inset_points = $inset_polygon->each();
    my $n=@points;
    
    my ($quad,$point1,$point2,$point3,$point4,$i);
    for ($i=0; $i<$n; $i++){
      # Centre of scaling is wrong here!
      my $quad = new Adult(0,0,'1');
      
      my $point1 = $points[$i]->copy($quad);
      my $point2 = $points[($i+1)%$n]->copy($quad);
      my $point3 = $inset_points[($i+1)%$n]->copy($quad);
      my $point4 = $inset_points[$i]->copy($quad);

      my $line1=new Line($point1,$point2->copy(),$creases[$i]);
      my $line2=new Line($point2,$point3->copy(),1);
      my $line3=new Line($point3,$point4->copy(),$creases[$i]*(-1));
      my $line4=new Line($point4,$point1->copy(),-1);

      $quad->add($line1,$line2,$line3,$line4);
      
      # Centre of scaling should be corrected here!
      #    $triangle->find_centre();  
      $quad->find_centre2();  
      $set->add($quad);
    }
    $polygon=$inset_polygon; # cunning switch to allow recursion!
  }

  $set->add($inset_polygon);
  return $set;
}




###############
# quarterise #
###############
=head1 quarterise

Takes n-gon object and decomposes by making lines from midpoint of all lines to centre.
for a square the result would be four smaller squares.

my $set = quarterise PolygonSet($polygon);

=cut


sub quarterise {
  my $class = shift @_;
  my $polygon = shift @_;

  die "Commented out this method 18/11/02";

  ####################################
  # Check number of sides is correct #
  ####################################
  my @lines=$polygon->eachline();
  my $n=@lines;

  if ($n != 4){
      die "In PolygonSet quarterise, can only accept polygons with 4 sides, yours has $n!\n";
  }

  # Foreach line get midpoint and put in an array.
  my (@midpoints,@cornerpoints);
  foreach my $line (@lines){
      push (@midpoints,$line->midpoint());
      my ($p1,$p2)=$line->points();
      push (@cornerpoints,$p1);
  }

  my $centre=$polygon->dual_vertex()->copy();

  ###########################
  # Make quartered polygons #
  ###########################

  my $set=new PolygonSet(0,1,4,4,'TWIST',1);

  my $i=0;
  while($i<$n){
      my $line1=new Line($cornerpoints[$i]->copy(),$midpoints[$i]->copy(),1);
      my $line2=new Line($midpoints[$i]->copy(),$centre->copy(),1);
      my $line3=new Line($centre->copy(),$midpoints[($i-1)%$n]->copy(),1);
      my $line4=new Line($midpoints[($i-1)%$n]->copy(),$cornerpoints[$i]->copy(),1);

      ###############################
      # Calulate centre for polygon #
      ###############################
      my $dummy_line=new Line($cornerpoints[$i],$centre,1);
      my $mpoint = $dummy_line->midpoint();
      my $x=$mpoint->x();
      my $y=$mpoint->y();

      my $qpolygon=new Adult($x,$y,1); 
      $qpolygon->add($line1,$line2,$line3,$line4);
      $set->add($qpolygon);
      $i++;
  }
  return $set;
}

#######
# add #
#######

=head1 add

Add polygons to Set object

    $polygonset->add($polygon1..$polygonN);

=cut
sub add {
  my $set = shift @_;
  push(@{$set->{'polygons'}},@_);
}

##########
# remove #
##########
=head1 remove

Will remove a polygon object from polygonset.  Will return 1 if removed
and 0 if not.

$polygonset->remove($polygon);

=cut
sub remove {
  my($polygonset,$polygon)=@_;
  my $return=0;
  my @array;

  # Hmm got to be careful modifying arrays while looping over them.
  foreach my $tmp_polygon ($polygonset->each()){
    if ($polygon eq $tmp_polygon){
      #print "Yep this polygon is in the polygonset $polygonset, removing\n";
      $return=1;
    } else {
      push (@array,$tmp_polygon);
    }
  }
  # Replace point list!
  #print "Replacing @array\n";
  $polygonset->{'polygons'}=[];

  foreach my $element (@array){
    $polygonset->add($element);
  }
  return $return;
}


###########
# add_set #
###########

=head1 add_set

Adds all polygons of a set to Set object

    $polygonset->add_set($polygonset2);

=cut
sub add_set {
    my ($set,$set2)=@_;

    foreach my $polygon ($set2->each()){
      push(@{$set->{'polygons'}},$polygon);
    }
}


#########
# scale #
#########

=head1 scale

    Method to scale each polygon in PolygonSet using given x and y as
origin for scaling.

$polygonset -> scale($scale_factor,$x_origin,$y_origin);

If x and y are not given will use centre of each polygon as origin
for scaling.

=cut
sub scale {
    my ($set,$scale_factor,$x_origin,$y_origin)=@_;

    my $len=@_;

    if (! $scale_factor){
	die "In method Adult::scale, No scale factor given!\n";
    }


    if ($len == 2){
      foreach my $polygon ($set->each()){
	$polygon->scale($scale_factor);
      }
    } elsif ($len == 4){
      foreach my $polygon ($set->each()){
	$polygon->scale($scale_factor,$x_origin,$y_origin);
      }
    } else {
      die "Wrong number of args in PolygonSet::scale\n";
    }
}


#############
# translate #
#############

=head1 translate

    Method translate each polygon in PolygonSet by x and y

$polygonset -> translate($x,$y);

=cut
sub translate {
    my ($set,$x,$y)=@_;

    foreach my $polygon ($set->each()){
      eval {
	$polygon->translate($x,$y);
      };
      if ($@ eq "Cannot translate baby!\n"){ # Catch exception
      } elsif ($@){die "$@";}
    }
}

##########
# rotate #
##########

=head1 rotate

    Method to rotate each polygon in PolygonSet by $rotation

$polygonset -> rotate($rotation);

or

$polygonset -> rotate($rotation,$x_orig,$y_orig);



=cut
sub rotate {
  my ($set,$rotation,$x_orig,$y_orig)=@_;
  my $len=@_;
  
  if ($len==2){
    foreach my $polygon ($set->each()){
      $polygon->rotate($rotation);
    }
  } elsif ($len==4){
    foreach my $polygon ($set->each()){
      $polygon->rotate($rotation,$x_orig,$y_orig);
    }
  } else {
    die "Wrong number args in PolygonSet::rotate\n";
  }
}

###########################
# multiple_rectangle_grid #
###########################
=head1 multiple_rectangle_grid

Method takes a polygonset and does rectangle grid for each tile.

$outputset->rectangle_grid($set,$xbegin,$ybegin,$xstep,$ystep,$parameters,$bound_box);

=cut

sub multiple_rectangle_grid {
  my ($outputset,$set,$xbegin,$ybegin,$x_step,$y_step,$parameters,$bound_box,$alternate) = @_;

  # Foreach polygon of set do rectangle grid.
  foreach my $polygon ($set->each()){
    my $tempset = new PolygonSet();
    $tempset->rectangle_grid($polygon,$xbegin,$ybegin,$x_step,$y_step,$parameters,$bound_box,$alternate);

    # Take set generated and add to output set 
    foreach my $outputpolygon ($tempset->each()){
      # Add to output set
      $outputset->add($outputpolygon);
    }
  }
  return ($outputset);
}


##################
# rectangle_grid #
##################
=head1 rectangle_grid

method to generate a rectangular grid of polygons. Will have the
ability to take a polygon to use as a bounding box.

SHOULD COMPLAIN VIGOUROUSLY IF POLYGON STARTS OUT OF BOUNDS.

    $set->rectangle_grid($polygon,$xbegin,$ybegin,$x_step,$y_step,$bound_box, 'alternate');

$polygon      Is polygon object
$xbegin       Is where to place centre of first polygon on x axis
$ybegin       Is where to place centre of first polygon on y axis
$x_step       Is how far to move each step on x axis
$y_step       Is how far to move each step on y axis
$bound_box    Polygon for bounding box

optional 'alternate'  means that creases are reversed in a checkerboard fashion

=cut
sub rectangle_grid {
  my ($set,$polygon,$xbegin,$ybegin,$x_step,$y_step,$bound_box,$alternate) = @_;
  my ($x_init,$x_start,$x_end,$y_start,$y_end,$new_polygon);

  # Should add scale in this routine.  The actual scaling however will not occur until
  # The output stage.  To do scaling we will divide the $x_step, $y_step, $xbegin and
  # $ybegin parameters by scale.  But only do this while working out how many repeats
  # to do.

  if ($set->{'scale'}){
    # Reset variables
    $xbegin=$xbegin/$set->{'scale'};
    $ybegin=$ybegin/$set->{'scale'};
    $x_step=$x_step/$set->{'scale'};
    $y_step=$y_step/$set->{'scale'};
    $polygon->scale(1/$set->{'scale'}); # At origin
  }

  # Need to calculate these given the bounding box.
  
  # Algorithm should start at $xbegin and decrement until none of the polygons
  # Points belong in box. Then increment until not in box etc.

  # Move polygon to correct start
  $polygon->translate($xbegin,$ybegin); # Bug here?!



  # Set up bounding box to be large enough to include polygon at least once
  # Serious bug here for crystal geometry and others. Seems that contains_polygon
  # is failing!
  my $scale=1;
  my $scale_increment=1.5;
  until ($bound_box->contains_polygon($polygon)){
    $bound_box->scale(1.5);
    #warn "Uh oh, started off out of bounds! Expanding bounding box\nScale is $scale\n";
    #warn "\nBOUND BOX\n\n";
    $bound_box->scale($scale_increment);
    $scale=$scale*$scale_increment;
    if ($scale>100){
      die "We appear to have got into an infinite loop!  Serious known unfixed bug!\nSeem to be scaling bounding box inapropriately\n";
    }  
  }


  # Find $x_start
  my $i=0;
  my $okay=1; # This variable stores whether polygon has been translated out of bounding box
              # and therefore we have got our x_start variable.
  while ($okay){
    $new_polygon = $polygon->copy();
    $new_polygon->translate($i*$x_step,0);
    if ($bound_box->contains_polygon($new_polygon)){
        #print STDERR "BOUND BOX contains new polygon decrementing! I is $i\n";
	$i--;
    } else {
      $okay=0;
    }
  }
  $x_start=$i+1;

  # Find $x_end
  $i=0;
  $okay=1;
  while ($okay){
    $new_polygon = $polygon->copy();
    $new_polygon->translate($i*$x_step,0);
    if ($bound_box->contains_polygon($new_polygon)){
      $i++;
    } else {
      $okay=0;
    }
  }
  $x_end=$i;


  # Find $y_start
  $i=0;
  $okay=1;
  while ($okay){
    $new_polygon = $polygon->copy();
    $new_polygon->translate(0,$i*$y_step);
    if ($bound_box->contains_polygon($new_polygon)){
      $i--;
    } else {
      $okay=0;
    }
  }
  $y_start=$i+1;

  # Find $y_end
  $i=0;
  $okay=1;
  while ($okay){
    $new_polygon = $polygon->copy();
    $new_polygon->translate(0,$i*$y_step);
    if ($bound_box->contains_polygon($new_polygon)){
      $i++;
    } else {
      $okay=0;
    }
  }
  $y_end=$i;

  $x_init=$x_start;

  # Move polygon back to origin
  $polygon->translate(-$xbegin,-$ybegin);

  # Reset variables if scale has been used
  if ($set->{'scale'}){
    die "Using scale in rectangle_grid\n";
    $polygon->scale($set->{'scale'});
    $xbegin=$xbegin*$set->{'scale'};
    $ybegin=$ybegin*$set->{'scale'};
    $x_step=$x_step*$set->{'scale'};
    $y_step=$y_step*$set->{'scale'};
  }

  # Move polygon to correct start
  $polygon->translate($xbegin,$ybegin);
	
  $i=-1;
  while ($y_start<$y_end){
    $i=$i*-1;
    my $j=1;

    $x_start=$x_init;

    while ($x_start<$x_end){
      # Make new polygon
      $new_polygon = $polygon->copy();

      $j=$j*-1;

      if (defined $alternate and $alternate eq 'alternate' and $i*$j>0){
	  $new_polygon->reverse();
      }

      $new_polygon->translate($x_start*$x_step,$y_start*$y_step);  
      $set->add($new_polygon);
      $x_start++;
    }
    $y_start++;
  }

  # Would be good to split polygons that are on the on the edge of
  # the bounding box

  # This code will not work!  This needs to happen after vertexify step!
  my $stupid=0;
  if ($stupid){
      foreach my $p ($set->each()){
	  # Test
	  my @return=$bound_box->split($p);
	  if ($return[0] !=0 and $return[0] !=-1 and $return[0] !=-2){
	      # Must have returned split polygons
	      
	      # Remove $p from set and add one of split polygons!
	      $set->remove($p);
	      $return[0]->layers(0);
	      $return[1]->layers(0);
	      $return[2]->layers(0);
	      #	  $set->add($return[0]);
	      my $x=$return[2]->dual_vertex()->x();
	      my $y=$return[2]->dual_vertex()->y();
	      
	      $set->add($return[2]);

	      #	  die "Return of @return\n";
	  }
	  
      }
  }

  return $set;
}

###########
# reverse #
###########
=head1 reverse

    Method to reverse crease assignment for set

$set->reverse();

=cut

sub reverse {
    my ($self)=@_;

    foreach my $polygon ($self->each()){
	$polygon->reverse();
    }
}

###########
# reorder #
###########

=head1 reorder

Will take a polygonset and reorder point objects for each polygon if incorrect.
Only works on  convex polygons!

$polygonset->reorder();

=cut
sub reorder {
    my ($set)= @_;      # Get object

    foreach my $polygon ($set->each()){
	$polygon->reorder();
    }
    return $set;
}

##############
# to_polygon #
##############
=head1 to_polygon

Takes polygonset of baby objects and converts to polygon objects

$new_set = $polygonset->to_polygon();

=cut

sub to_polygon {
  my ($set)= @_;      # Get object
  
  my $new_set = new PolygonSet();
  foreach my $baby ($set->each()){
    my $polygon = $baby->to_polygon();
    $new_set->add($polygon);
  }
  return $new_set;
}

#############
# transform #
#############

=head1 transform

Now an unused method. Only called when PolygonSet of Vertex babies is
added to tiling and the transorm method is called.

=cut

sub transform {
    return 0;
}

#########
# write #
#########

=head1 write

Debugging method: Write polygonset to STDERR

$polygonset->write();

=cut
sub write {
    my ($set)= @_;      # Get object

    print "Writing PolygonSet $set\n";
    foreach my $polygon ($set->each()){
	$polygon->write();
    }
}

##########
# render #
##########
=head1 render

Method to return an XML render stack, that will be used
by View objects.  Also this method returns an array
which has object references in.  These correspond to
tag attributes for objects in XML.  THis means external 
objects to model can refer to objects by these tags and
we can work out which object the external objects mean.

($current_tag,$stack,$tag_ref)=$tiling->render($current_tag,$stack,\@tags);

$stack is XML stack for rendering
$tag_ref is a reference to the array of objects

=cut
sub render {
    my ($self,$current_tag,$stack_ref,$tag_ref) = @_;

    $current_tag++;
    push (@$tag_ref,$self);

    my $open_tag  = "  <polygonset tag=\"$current_tag\">\n";
    my $close_tag = "  <polygonset/>\n";

    $$stack_ref .= $open_tag;

    foreach my $element ($self->each()){
	($current_tag,$stack_ref,$tag_ref)=$element->render($current_tag,$stack_ref,$tag_ref);
    }
    $$stack_ref .= $close_tag;

    return ($current_tag,$stack_ref,$tag_ref);
}

1;  # says use was ok
__END__

