=head1 NAME

PrimaryPatch

=head1 DESCRIPTION

B<PrimaryPatch> contains a collection of polygon objects. THe only
purpose for the PrimaryPatch object is in the calculation of
parameters for the tiling.  Calling the method calculate_parameters
will return the absolute scales and rotations needed for the model.
This object is contained by a tiling object.  The Model object is
unaware of its existance!

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for PolygonSet
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package PrimaryPatch;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Polygon;
use Adult;
use Line;
use Patch;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter', 'Patch' );


#######
# new #
#######

=head1 new

Creates Patch object. This contains many attributes.
The weave attribute can be left or right.

$atch = new Patch();

=cut
sub new {
    my ($class,%parameters)=@_;

    my $set = {
	       "polygons"   => [],
	       "x1"         => $parameters{'x1'},
	       "y1"         => $parameters{'y1'},
	       "x2"         => $parameters{'x2'},
	       "y2"         => $parameters{'y2'},
	       "alternate1" => $parameters{'alternate1'},
	       "alternate2" => $parameters{'alternate2'},
	       "current_alpha" => '1',
	       "current_theta" => '0',
	       };

    bless $set, $class;
    return $set;
}




########################
# calculate_parameters #
########################

=head1 calculate_parameters

This method takes pleat ratio and pleat angles.  Will then calculate alphas and
scalings and return them.

=cut

sub calculate_parameters {
  my ($self,$pleat_ratio,$pleat_angle1,$pleat_angle2) = @_;

  if (! $pleat_ratio){
      warn "No pleat ratio so will not render! [$pleat_ratio,$pleat_angle1,$pleat_angle2]";
      return 0;
  }

  if (! $pleat_angle1){
      warn "No pleat angle so will not render! [$pleat_ratio,$pleat_angle1,$pleat_angle2]";
      return 0;
  }

  my @polygons = $self->each();
  my $primary=$polygons[0];
  my $secondary=$polygons[1];

  # Check that secondary exists
  if (! $secondary){
      die "Oh no. The patch for this geometry only has one tile!";
  }

  # Find distance between primary and secondary polygons
  my $centre1=$primary->centre();
  my $centre2=$secondary->centre();
  my $original_distance=$centre1->distance($centre2);

  my $pi = atan2(1,1)*4;

  # Calculate absolute values of theta and alpha.
  my $psi_1              = $pleat_angle1;
  my $psi_2              = $pleat_angle2;

  # Don't forget to convert angle to radians
  $psi_1=$psi_1*$pi/180;
  $psi_2=$psi_2*$pi/180;

  # Relative scale of secondary polygon compared to primary polygon (see page 68 of notebook)
  # If not a weave type tiling this should equal 1.
  my $relative_scale=sin($psi_1)/sin($pi-$psi_2); 


  # Find difference in thetas for primary and secondary polygon (see page 68 of notebook)
  # If not a weave type tiling this should equal 0.
  my $theta_difference=$pleat_angle1-$pleat_angle2;

  # Now we need to construct primary and secondary polygons connected by
  # the pleat of correct length.  We can use this to find the distance 
  # between the new centres and an additional scale factor for both polygons.
  # We also use this to find theta1

  my $primary_copy=$primary->copy();
  my $secondary_copy=$secondary->copy();

  my $primary_centre=$primary_copy->centre();
  my $secondary_centre=$secondary_copy->centre();
  my $original_line=new Line($primary_centre->copy(),$secondary_centre->copy());

  # Find common edges
  my ($final_distance,$final_line,$adjacent);
  foreach my $line1 ($primary_copy->eachline()){
      foreach my $line2 ($secondary_copy->eachline()){
	  my $return_value=$line1->compare($line2);
	  if($return_value == 1){
	      $adjacent=1;

	      my $pleat_line=$line1->copy();
	      # Scale by Pleat ratio
	      my ($p1,$p2)=$pleat_line->points();
	      $pleat_line->scale($pleat_ratio,$p2->xy());

	      # Rotate by pleat angle 1
	      $pleat_line->rotate($pleat_angle1,$p2->xy());

	      # Scale copy of secondary polygon about point 4
	      my ($p3,$p4)=$line2->points();
	      $secondary_copy->scale($relative_scale,$p4->xy()); # Change p4 to p3 for return of -1!

	      # Rotate copy of secondary by difference in thetas
	      $secondary_copy->rotate($theta_difference);

	      # Now translate copy of secondary polygon
	      ($p1,$p2)=$pleat_line->points();
	      my $x_tran=$p1->x()-$p2->x();
	      my $y_tran=$p1->y()-$p2->y();

	      $secondary_copy->translate($x_tran,$y_tran);

	      # Find distance between primary and secondary polygons
	      my $centre3=$primary_copy->centre();
	      my $centre4=$secondary_copy->centre();
	      $final_distance=$centre3->distance($centre4);
	      $final_line=new Line($centre3->copy(),$centre4->copy());
	  } elsif ($return_value == -1){
	      $adjacent=1;

	      my $pleat_line=$line1->copy();
	      # Scale by Pleat ratio
	      my ($p1,$p2)=$pleat_line->points();
	      $pleat_line->scale($pleat_ratio,$p2->xy());

	      # Rotate by pleat angle 1
	      $pleat_line->rotate($pleat_angle1,$p2->xy());

	      # Scale copy of secondary polygon about point 3
	      my ($p3,$p4)=$line2->points();
	      $secondary_copy->scale($relative_scale); # Change p3 to p4 for return of 1!

	      # Rotate copy of secondary by difference in thetas
	      $secondary_copy->rotate($theta_difference);

	      # Now translate copy of secondary polygon
	      ($p1,$p2)=$pleat_line->points();
	      my $x_tran=$p1->x()-$p2->x()-$p3->x()+$p2->x();
	      my $y_tran=$p1->y()-$p2->y()-$p3->y()+$p2->y();

	      $secondary_copy->translate($x_tran,$y_tran);

	      # Find distance between primary and secondary polygons
	      my $centre3=$primary_copy->centre();
	      my $centre4=$secondary_copy->centre();
	      $final_distance=$centre3->distance($centre4);
	      $final_line=new Line($centre3->copy(),$centre4->copy()); 
	  }
      }
  }

  if (! $adjacent){
      die "Oh no!  It appears that the primary and secondary tile in the patch are not adjacent";
  }

  if (! $final_line){
      die "Oh no could not make a final line!";
  }
  if (! $final_distance){
      die "Oh no could not calculate a final distance!";
  }

  # So calculate scales for each polygon
  my $scale1=$original_distance/$final_distance;
  my $scale2=$scale1*$relative_scale;

  # Now calculate thetas for polygons
  my $theta1=$original_line->angle($final_line);
  my $theta2=$theta1+$theta_difference;

  my $theta_phi=$theta1+$pleat_angle1;

  warn "For pleat ratio $pleat_ratio angle1 $pleat_angle1 angle2 $pleat_angle2\nwe get scale1 $scale1 scale2 $scale2 theta1 $theta1 theta2 $theta2 original distance $original_distance final distance $final_distance relative scale $relative_scale theta diff $theta_difference\n**** THETA + PHI $theta_phi\n";

  return($scale1,$scale2,$theta1,$theta2);
}



1;  # says use was ok
__END__

