=head1 NAME

Tiling

=head1 DESCRIPTION

B<Tiling> object contains polygonset objects.  The methods on tiling objects
includes transiently generating vertex and edge objects.  Also the
tiling object contains all the tiling generation code, each of which is
stored in an undocumented method.

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for Tiling
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Tiling;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Point;
use Polygon;
use Adult;
use Baby;
use Line;
use PolygonSet;
use Patch;
use PrimaryPatch;
use Vertex;
use Edge;
use Math::Trig;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

#######
# new #
#######

=head1 new

Creates tile object

$tiling = new Tiling();

=cut

sub new {

  my $class = shift @_;
  my $geometry  = shift @_;
  my $type = shift @_;

  my %args = @_;

  my $bbox_scale;

  foreach my $arg (keys %args){
      if ($arg eq 'bbox_scale'){
	  $bbox_scale = $args{'bbox_scale'};
      }
  }

  my $bbox  = Adult::new_ngon('4','1');

  if (! $bbox_scale){
      $bbox_scale=9;
  }

  my $self = {
		  "sets"           => [],
		  "geometry"       => $geometry,
		  "type"           => $type,
		  "bbox"           => $bbox,
		  "description"    => '',
		  "unfolded_tiling"=> '',
		  "patch"          => '',    # Stores the original patch
		  "bbox_scale"     => '1',   # This stores absolute value of bounding box
		  "absolute_scale" => '1',   # This stores the absolute scaling
		  "x_translate"    => '0',   # This stores the absolute x translation
		  "y_translate"    => '0',   # This stores the absolute y translation
		  "pleat_ratio"    => '0',   # This stores the pleat_ratio
		  "pleat_angle"    => '0',   # This stores the pleat angle
		  "pleat_angle2"   => '0',   # This stores the pleat angle for weave tessellations
		  "title"          => '',    # Title for tiling
		  "page"           => '',    # Page number if used in book
		  "x_expand"       => '6',   # Number of repetitions on x axis
		  "y_expand"       => '6',   # Number of repetitions on x axis
		  };

  if (! $self->{'type'}){
    die "Sorry I need to know whether tiling is type=twist or weave\n!";
  }

  bless $self, $class;

  $self->bbox_scale($bbox_scale);

  return $self;
}

########
# copy #
########

=head1 copy

Method to copy a tiling object

    $copy = $self->copy();

Note patch is same between both tiling and copied tiling!

=cut

sub copy {
    my ($self)=@_;

    my $copy = new Tiling($self->{'geometry'},$self->{'type'});

    $copy->{'title'}=$self->{'title'};
    $copy->{'page'}=$self->{'page'};
    $copy->{'description'}=$self->{'description'};

    my $patch=$self->{'patch'};
    if ($self->{'patch'}){
	$copy->{'patch'}=$self->{'patch'};
    }
    my $copypatch=$copy->{'patch'};

    foreach my $object ($self->each()){
      my $new_object=$object->copy();
      $copy->add($new_object);
    }
    return $copy;
}


########
# bbox #
########

=head1 bbox

Returns bounding box

$bbox = $tiling->bbox();

=cut

sub bbox {
  my $tiling = shift @_;

  my $bbox=$tiling->{'bbox'};
  return $bbox;
}

##############
# bbox_scale #
##############

=head1 bbox_scale

Scales bounding box

    $tiling->bbox_scale($scale);

$scale is the absolute value of the bounding box

=cut

sub bbox_scale {
  my $self = shift @_;
  my $scale= shift @_;

  my $bbox_scale_current=$self->{'bbox_scale'};
  my $bbox=$self->bbox();

  $bbox->scale($scale/$bbox_scale_current);
  $self->{'bbox_scale'}=$scale;
}

############
# x_expand #
############

=head1 x_expand

Get/set for x expansion

    $tiling->x_expand();

This is how many times patch is repeated along x axis.

=cut

sub x_expand {
  my $self = shift @_;
  my $value = shift @_;

  if ($value){
    $self->{'x_expand'}=$value;
  } else {
    $value=$self->{'x_expand'};
  }

  return $value;
}

############
# y_expand #
############

=head1 y_expand

Get/set for y expansion

    $tiling->y_expand();

This is how many times patch is repeated along y axis.

=cut

sub y_expand {
  my $self = shift @_;
  my $value = shift @_;

  if ($value){
    $self->{'y_expand'}=$value;
  } else {
    $value=$self->{'y_expand'};
  }

  return $value;
}

###############
# description #
###############

=head1 description

Get set of description for tiling

    $tiling->description();

=cut

sub description {
  my $self = shift @_;
  my $desc= shift @_;

  if ($desc){
      $self->{'bbox_scale'}=$desc;
  } else {
      $desc=$self->{'bbox_scale'};
  }
  return $desc;
}

#########
# patch #
#########

=head1 patch

Get set of primary patch for tiling

    $self->patch();


=cut

sub patch {
  my $self = shift @_;
  my $value= shift @_;

  if ($value){
      $self->{'patch'}=$value;
  } else {
      $value=$self->{'patch'};
  }
  return $value;
}

########
# each #
########

=head1 each

Loops over tiling and returns a list of polygonset objects

@polygonsets = $tiling->each();

=cut

sub each {
  my $tiling = shift @_;
  my @list;

  foreach my $polygonset (@{$tiling->{'sets'}}){
    push(@list,$polygonset);
  }
  return @list;
}

#######
# add #
#######

=head1 add

Add polygonsets to tile object

    $tiling->add($set1,$set2...);

=cut

sub add {
  my $tiling = shift @_;
  push(@{$tiling->{'sets'}},@_);
}




#############
# transform #
#############

=head1 transform

Method to take gui parameters and calculate the absolute scales and
rotations applied to the tiles. This uses the primary patch object and
a construction method for the calculation rather than an analytical
solution. This is then applied to all the patch objects in the Tiling.

$tiling->transform($pleat_ratio,$pleat_angle1,$pleat_angle2,$type);

=cut

sub transform {
    my ($self,$pleat_ratio,$pleat_angle1,$pleat_angle2)=@_;

    if (! $pleat_ratio){
	carp "No parameters for Tiling::transform";
    }

    $self->{'pleat_ratio'}=$pleat_ratio;
    $self->{'pleat_angle'}=$pleat_angle1;
    $self->{'pleat_angle2'}=$pleat_angle2;

    my $type=$self->{'type'};
    my $primary_patch=$self->patch();

    my ($alpha1,$alpha2,$theta1,$theta2);
    if ($type eq 'twist'){
	($alpha1,$alpha2,$theta1,$theta2)=$primary_patch->calculate_parameters($pleat_ratio,$pleat_angle1,$pleat_angle1);
    } else {
	($alpha1,$alpha2,$theta1,$theta2)=$primary_patch->calculate_parameters($pleat_ratio,$pleat_angle1,$pleat_angle2);
    }


    if ($alpha1){ # calculate parameters returns 0 if pleat angles were zero.  This happens
	# sometimes when using scroll bars in gui.
	#  warn "Transforming set $type,$alpha1,$alpha2,$theta1,$theta2";
	foreach my $set ($self->each()){
	    $set->transform($type,$alpha1,$alpha2,$theta1,$theta2);
	}
    }
}

#########
# scale #
#########

=head1 scale

    This is a seriously polymorphic method to scale each polygon in Tiling.

To scale each polygon about a specific point use

$tiling -> scale($scale_factor,$x,$y);

This invocation will change the absolute_scale attribute of the tiling.

If no x or y given will use centre of each polygon.

=cut

sub scale {
    my ($self,$scale_factor,$x_origin,$y_origin)=@_;
    
    my $len=@_;
    if (! $scale_factor){
	die "In method Tiling::scale, No scale factor given!\n";
    }
    
    my $absolute_scale=$self->{'absolute_scale'};
    foreach my $polygonset ($self->each()){
	if ($len == 2){
	    $polygonset->scale($scale_factor);
	} elsif ($len == 4){
	    $polygonset->scale($scale_factor/$absolute_scale,$x_origin,$y_origin);
	} else {
	    die "Wrong number of args in Tiling::scale\n";
	}
    }
    $self->{'absolute_scale'}=$scale_factor;
    return $self;
}


###########
# reverse #
###########

=head1 reverse

Method to reverse crease assignment for tiling

    $tiling->reverse();

=cut

sub reverse {
    my ($self)=@_;

    foreach my $set ($self->each()){
	$set->reverse();
    }
}

#############
# translate # 
#############

=head1 translate

Method to translate each polygonset in tile. x and y are absolute
values.  So if you translate x by 150, then do 150 again it will not
actually move the tiling!

    $tiling->translate(x,y);

=cut

sub translate {
    my ($self,$x,$y)=@_;

    my $x_translate=$self->{'x_translate'};
    my $y_translate=$self->{'y_translate'};
  
    foreach my $polygonset ($self->each()){
	my $x_change=$x-$x_translate;
	my $y_change=$y-$y_translate;
	$polygonset->translate($x_change,$y_change);
    }
}

#############
# max_angle #
#############

=head1 max_angle

    Method to give maximum possible pleat angle.  This assumes
    that you are using the same twist for all polygons. The
    max angle is half of the internal angle for the most acute
    angle in the tiling.

$max_angle=$tiling->max_angle();

=cut

sub max_angle {
  my ($tiling)=@_;

  print STDERR "Calculating the maximum pleat angle for the tiling\n";

  my $max_angle=180;
  foreach my $set ($tiling->each()){
      foreach my $p ($set->each()){
	  my @lines=$p->eachline();
	  my $n=@lines;

	  if ($n<3){next;} # Ignore these, they can't be a true polygon

          # Loop over all lines
	  my $i=0;
	  while($i<$n){
	      my $angle=$lines[$i]->angle($lines[($i+1)%$n]);

	      if ($angle>180){
		  $angle=($angle-360)*-1;
	      }

	      #print STDERR "ANGLE $angle\n";
	      if ($angle < 1){
		  #print STDERR "Problem polygon ignoring!\n";
		  #$p->write();
		  #$lines[$i]->write();
		  #$lines[($i+1)%$n]->write();
		  #print STDERR "N $n\n";
		  $i++;
		  next;
	      }

	      if ($angle<$max_angle){
		  $max_angle=$angle;
	      }
	      $i++;
	  }
      }
  }
  $max_angle=$max_angle/2;
  print STDERR "Max angle for tiling is $max_angle\n";
  return $max_angle;
}

########
# trim #
########

=head1 trim

    Method to trim tiling with a bounding box

    $tiling->trim($box);

=cut

sub trim {
    my $self = shift @_;
    my $box  = $self->bbox();

    foreach my $set ($self->each()){
	foreach my $p ($set->each()){
	    # Test
	    my @return=$box->split($p);
	    if ($return[0] !=0 and $return[0] !=-1 and $return[0] !=-2){
		# Must have returned split polygons
		
		# Remove $p from set and add one of split polygons!
		$set->remove($p);
		$return[0]->layers(0);
		$return[1]->layers(0);
		$return[2]->layers(0);
		#	  $set->add($return[0]);
		my $x=$return[2]->dual_vertex()->x();
		my $y=$return[2]->dual_vertex()->y();
	      
		$set->add($return[2]);
		
		#	  die "Return of @return\n";
	    }
	}
    }
 

}

###########
# maekawa #
###########

=head1 maekawa

Checks vertices that sum of crease parities is +2 or -2.  Will ignore
vertices with less than four creases.  This condition is necessary for
flat foldability.

    @points=$tiling->maekawa();

Returns an array of points that do not satisfy Maekawa's theorem.

=cut

sub maekawa {
    my $tiling=shift @_;

    my (%data,@data,%owner,%owner_crease,@points,%number_of_lines);
    foreach my $polygonset ($tiling->each()){
	foreach my $polygon ($polygonset->each()){
	    foreach my $line ($polygon->eachline()){
		my ($p1,$p2)=$line->points();

		my $crease=$line->crease();
		if (! $crease){
		    die "No crease parity for $line\n";
		}

		# Have to do this to take account of baby objects where one point may
		# have a parity for both the adult and baby polygon
		$owner_crease{$p1}=$owner_crease{$p1}+$crease; # Get parity of crease for each line
		$owner_crease{$p2}=$owner_crease{$p2}+$crease; # Get parity of crease for each line

		# Keep this so we can count number of lines keyed off point
		$number_of_lines{$p1}++;
		$number_of_lines{$p2}++;

		###############################
		# List of all points in $data #
		###############################
		if (! $data{$p1}){
		    push(@data,$p1);
		    $data{$p1}=1;
		}
		
		if (! $data{$p2}){
		    push(@data,$p2);
		    $data{$p2}=1;
		}

		###################
		# Do I need this? #
		###################
		$owner{$p1}=$polygon;
		$owner{$p2}=$polygon;
	    }
	}
    }

    ###############################################
    # Sort all points by x coordinate to speed up #
    ###############################################
    my @datakeys;
    foreach my $point (@data){
	push (@datakeys,$point->x());
    }
    my @sortdata=@data[sort {$datakeys[$a] <=> $datakeys[$b]} $[..$#data];
    
    my %crease_parity;
    
    # Maybe I shouldn't shift elements! #

    while(@sortdata){       # Not all vertices assigned
	# Shift first element of list into vertex object
	my $point=shift @sortdata;
	
	if (! $point){die "No point!\n";}
	
	my @pointlist;
	push(@pointlist,$point);
	
	######################################################    
	# Traverse list checking if next point is close with #
	# (x1-x2)^2+(y1-y2)^2 < distance^2                   #
	######################################################
	
	my $nf;                          # Element of array @sortdata
	for ($nf=0; $nf<@sortdata; $nf++){
	    my $test_point=$sortdata[$nf];
	    my $xdist=$point->x()-$test_point->x();
	    my $ydist=$point->y()-$test_point->y();
	    if ($xdist<-1){ # Short circuit speed up
		last;
	    }
	    if ($ydist*$ydist<1 and $xdist*$xdist<1){
		push(@pointlist,splice @sortdata,$nf,1);
		$nf--;
	    }
	}

	########################################
	# OK now we can count number of points #
	# and tot up crease parities           #
	########################################
	
      	if (! $number_of_lines{$point}){
	    die "No number of lines for $point!!!!\n";
	}
	
	# Get overall parity of vertex
	my $number_of_lines;
	my $crease_total;
	foreach my $point (@pointlist){
	    $number_of_lines=$number_of_lines+$number_of_lines{$point};

	    if ($owner_crease{$point}){
		$crease_total = $crease_total+$owner_crease{$point};
	    } else {
		die "No crease for point $point\n";;
	    }
	}
	

	#print "Vertex has $number_of_lines lines and parity of $crease_total\n";
	
	################################################################
	# Can loop over all points and return ones with wrong parities #
	################################################################
	foreach my $point (@pointlist){
	    $crease_parity{$point}=$crease_total;
	    if ($crease_total == 2 or $crease_total == -2){
	    } else {
		if ($number_of_lines>3){
		    #print STDERR  "Got here $crease_total , $number_of_lines\n";
		    push (@points,$point);
		}

	    }
	}
    }
    print STDERR "points @points\n";
    return @points;
}

############
# kawasaki #
############

=head1 kawasaki

Method to check each vertex of origami is flat foldable,
so sum of alternate angles must be 180 degrees.
    
@points=$tiling->kawasaki();

returns a list of points that do not satisfy kawasakis theorem.

=cut

sub kawasaki {
    my ($tiling)=@_;

    # Get all lines and find where they join.  This will be similar to
    # find create_vertex_set

    # Make an array of all lines anda hash of angles of all lines
    my (@lines,%line_angle);
    foreach my $set ($tiling->each()){
	foreach my $p ($set->each()){
	    foreach my $line ($p->eachline()){
		push(@lines,$line);
		$line_angle{$line}=$line->angle();
	    }
	}
    }
}

################
# make_species #
################

=head1 make_species

Generates a tiling where each set is a polygon of vertex species
specified

$tiling = make_species Tiling(4,4,4,4)

=cut

sub make_species {
  my @n = @_;

  ###########################################
  # Check that vertex specified is possible #
  ###########################################
  my $total=0;
  foreach my $n (@_){
      $total=$total+(180*(1-2/$n));
  }

  if ($total > 360.001 or $total < 359.999){
      die "Tiling::make_species your polygons add up to $total not 360!\n";
  }

  #######################
  # Generate the tiling #
  ####################### 
  my $tiling=new Tiling('0','twist');
  $total=0;
  foreach my $n (@_){
      # Make new set
      my $set=new PolygonSet();
      my $polygon=Adult::new_ngon($n,1);
      # Do translate and rotates
      my $y=50*cos(3.1415927/$n)/sin(3.1415927/$n);
      $polygon->translate(50,-$y);
      $polygon->rotate(180*(1-2/$n),0,0);
      $polygon->rotate($total,0,0);
      $tiling->add($set);
      $set->add($polygon);
      $total=$total+180*(1-2/$n);
  }
  return $tiling;
}

####################
# reassign_creases #
####################

=head1 reassign_creases

This method will reassign creases of a tiling based on a focal point.
edges from 0 to 180 degrees will be assigned mountains 180 to 360 as valleys.

Arguments are x and y position of focus.

$tiling->reassign_creases($x,$y)

=cut

sub reassign_creases {
  my ($tiling,$x,$y)=@_;

  my $focus= new Point($x,$y);

  foreach my $set ($tiling->each()){
    foreach my $polygon ($set->each()){
      foreach my $line ($polygon->eachline()){
	my ($p1,$p2)=$line->points();

	my $line1=new Line($focus,$p1);
	my $line2=new Line($focus,$p2);

	my $angle=$line1->angle($line2);

	$angle=$angle-180;

	if ($angle >= 0){
	  $line->crease(-1);
	} else {
	  $line->crease(1);
	}
      }
    }
  }
}

##########
# rotate #
##########

=head1 rotate

    This is a seriously polymorphic method to rotate each polygon in
    Tiling by $rotation

To rotate polygon about its own centre of rotation invoke as

$tiling -> rotate($rotation);

To rotate all polygons about a specific point use

$tiling -> rotate($rotation,$x_orig,$y_orig);

=cut

sub rotate {
  my ($tiling,$rotation,$x_orig,$y_orig)=@_;

  my $len=@_;
  
  if ($len==2){
    foreach my $set ($tiling->each()){
      $set->rotate($rotation);
    }
  } elsif ($len==4){
    foreach my $set ($tiling->each()){
      $set->rotate($rotation,$x_orig,$y_orig);
    }
  } else {
    die "Wrong number args in Tiling::rotate\n";
  }
}


#############
# vertexify #
#############

=head1 vertexify

Make vertex babies for tiling.

$tiling->vertexify();

$tiling->vertexify('edge' => 1);

This will give more accurate light pattern rendering at the edges of the paper.

=cut

sub vertexify {
  my $tiling = shift @_;
  my %args=@_;

  if ($args{'edge'}){
      my $edge_polygon_set   = Tiling::create_edge_set($tiling);
      $tiling->add($edge_polygon_set);
  }

  my $vertex_polygon_set = Tiling::create_vertex_set($tiling);  
  $tiling->add($vertex_polygon_set);

  return $tiling;
}

#####################
# create_vertex_set #
#####################

=head1 create_vertex_set
  
  Method to create polygonset object from vertices of polygons in tiling object.
  Returns polygonset of baby vertex objects.
  
  $vertex_polygon_set = $tiling -> create_vertex_set();

=cut

sub create_vertex_set {
  my $tiling = shift @_;

  print STDERR "Creating vertex set\n";

  my ($xdist,$ydist,$vertex,$vertex_polygon,$test_point);
  my $vertex_polygonset = new PolygonSet(0,0,0,0,'colour0');  # Set to fill with new polygons
  
  # Make list of point objects ordered by x coordinate - Borrowed
  # from Perl 4 Camel book page 249.
  my (@data,%owner,%owner_crease);
  foreach my $polygonset ($tiling->each()){
    foreach my $polygon ($polygonset->each()){
      foreach my $line ($polygon->eachline()){
	my ($foo,$point)=$line->points();
	$owner_crease{$point}=$line->crease(); # Get parity of crease for each line
	push(@data,$point);
	$owner{$point}=$polygon;
      }
    }
  }

  ###############################################
  # Sort all points by x coordinate to speed up #
  ###############################################
  my (@datakeys);
  foreach my $point (@data){
    push (@datakeys,$point->x());
  }
  my @sortdata=@data[sort {$datakeys[$a] <=> $datakeys[$b]} $[..$#data];

  my $distance=0.1;

  while(@sortdata){       # Not all vertices assigned
    # Shift first element of list into vertex object
    my $point=shift @sortdata;
    
    my @pointlist;
    push(@pointlist,$point);

    ######################################################    
    # Traverse list checking if next point is close with #
    # (x1-x2)^2+(y1-y2)^2 < distance^2                   #
    ######################################################

    my $nf=0;                          # Element of array @sortdata
    for ($nf=0; $nf<@sortdata; $nf++){
      $test_point=$sortdata[$nf];
      $xdist=$point->x()-$test_point->x();
      $ydist=$point->y()-$test_point->y();
      if ($xdist<-1){
	last;
      }
      if ($ydist*$ydist<$distance and $xdist*$xdist<$distance){
	push(@pointlist,splice @sortdata,$nf,1);
	$nf--;
      }
    }


    ##############################################
    # Need to order point objects correctly now. #
    # otherwise line creation cocks up!          #
    ##############################################

    my @data_tmp=@pointlist;
    my @datakeys_tmp;
    foreach my $point (@data_tmp){
      # Need to find the centre of polygon that point belongs to!
      my $polygon=$owner{$point};
      
      if (! $polygon){
	die "Can't find owner for $point in Tiling::create_vertex_set\n";
      }

      # Use centre of gravity as centres of rotation are not correct
      # once stretches are used!
      my $px = $polygon->cog()->x();
      my $py = $polygon->cog()->y();
      my ($bx,$by) = $point->xy();
      
      # Calculate angle in next line
      my $angle=atan2($py-$by,$px-$bx);
      push (@datakeys_tmp,$angle);
    }
    my @sortdata_tmp=@data_tmp[sort {$datakeys_tmp[$a] <=> $datakeys_tmp[$b]} $[..$#data_tmp];
    @pointlist=@sortdata_tmp;

    ################################################
    # Okay now make lines and add to vertex object #
    ################################################

    my $tmppoint=$pointlist[0];
    my ($x,$y)=$tmppoint->xy();

    my $vertex = new Vertex($x,$y);
    my $length = @pointlist;
    my $i=0;
    while($i<$length){
      # These polygons are babies, so don't copy!
      my $point1 = $pointlist[$i];
      my $point2 = $pointlist[($i+1)%$length];

      my $crease=($owner_crease{$point1})*(-1);

      if ($point1 ne $point2){
	  my $line= new Line($point1,$point2,$crease);
	  $vertex->add($line);
      }

      $i++;
    }
    $vertex_polygonset->add($vertex);
  }
  return ($vertex_polygonset);
}

###################
# create_edge_set #
###################

=head1 create_edge_set

Method to create polygonset object from edges of tiling object.
Returns polygonset of baby objects.

$edge_set = create_edge_set($tiling);

=cut

sub create_edge_set {
  my $tiling = shift @_;

  print STDERR "Creating edge set\n";
  my $edge_polygonset = new PolygonSet(0,0);  # Set to fill with new polygons
  
  my @polygonlist;
  foreach my $set ($tiling->each()){
      push @polygonlist,$set->each();
  }

  # Add proximity check
  foreach my $polygon (@polygonlist){
    foreach my $test_polygon (@polygonlist){


      my $edge_polygon=$polygon->shared_edge($test_polygon);

      if ($edge_polygon){
	$edge_polygonset->add($edge_polygon);
      }
    }
  }
  if (! $edge_polygonset){die "No edge polygonset created\n";}
  return $edge_polygonset;
}

#########
# title #
#########

=head1 title

Method to get or set title for tiling

$self->title($title)

=cut

sub title {
    my $self = shift @_;

    my $title=shift @_;
    if ($title){
	$self->{'title'}=$title;
    }
    $title=$self->{'title'};
    return $title;
}

########
# page #
########

=head1 page

Method to get or set page number for tiling

$self->page($title)

=cut

sub page {
    my $self = shift @_;

    my $page=shift @_;
    if ($page){
	$self->{'page'}=$page;
    }
    $page=$self->{'page'};
    return $page;
}

##########
# render #
##########

=head1 render

Method to return an XML render stack, that will be used
by View objects.  Also this method returns an array
which has object references in.  These correspond to
tag attributes for objects in XML.  This means external 
objects to model can refer to objects by these tags and
we can work out which object the external objects mean.

($stack,$tag_ref)=$tiling->render();

$stack is XML stack for rendering
$tag_ref is a reference to the array of objects

=cut

sub render {
    my $self = shift @_;

    my %args=@_;

    my ($tag_ref,$stack_ref);
    my $current_tag;

    $current_tag++;
    push (@$tag_ref,$self);

    my $options="";
    if ($self->page()){
	my $page=$self->page();
	$options .= " page=\"$page\"";
    }
    if ($self->title()){
	my $title=$self->title();
	$options .= " title=\"$title\"";
    }

    my $open_tag  = "<tiling tag=\"$current_tag\" $options>\n";
    my $close_tag = "<tiling/>\n";

    $$stack_ref .= $open_tag;

    # Render light pattern
    if ($args{'light'}){
	my $multiplier=1;
	my $resolution=$multiplier*$args{'light'};
	($current_tag,$stack_ref,$tag_ref)=$self->render_light_pattern($current_tag,$stack_ref,$resolution,$tag_ref);
    }


    if ($args{'show'} eq 'patch'){
	my $patch=$self->patch();
	warn "SELF $self PATCH $patch";

	foreach my $element ($self->patch()->each()){
	    ($current_tag,$stack_ref,$tag_ref)=$element->render($current_tag,$stack_ref,$tag_ref);
	}
    } elsif ($args{'show'} eq 'tiling' or $args{'show'} eq 'creases') {
	foreach my $element ($self->each()){
	    ($current_tag,$stack_ref,$tag_ref)=$element->render($current_tag,$stack_ref,$tag_ref);
	}
    }

    $$stack_ref .= $close_tag;

    my $stack=$$stack_ref;
    return ($stack,$tag_ref);
}

##############
# tessellate #
##############

=head1 tessellate

Make initial tiling.

    $tiling->tessellate();

=cut

sub tessellate {
  my $tiling = shift @_;
  my %args=@_;


  #$args{'rotate'}=35;
  #$args{'stretch'}=0.8;

  if ($tiling->{'geometry'} eq 1){
    $tiling->make_4_4_4_4();
  } elsif ($tiling->{'geometry'} eq 2){
    $tiling->make_6_6_6();
  } elsif ($tiling->{'geometry'} eq 3){
    $tiling->make_3_3_3_3_3_3();
  } elsif ($tiling->{'geometry'} eq 4){
    $tiling->make_4_8_8();
  } elsif ($tiling->{'geometry'} eq 5){
    $tiling->make_3_6_3_6();
  } elsif ($tiling->{'geometry'} eq 6){
    $tiling->make_3_4_6_4();
  } elsif ($tiling->{'geometry'} eq 7){
    $tiling->make_4_6_12();
  } elsif ($tiling->{'geometry'} eq 8){
    $tiling->make_3_4_3_12__3_12_12();
  } elsif ($tiling->{'geometry'} eq 9){
    $tiling->make_decomposed_4_4_4_4_test();
  } elsif ($tiling->{'geometry'} eq 10){
    $tiling->make_9_9_6_4pi9_test();
  } elsif ($tiling->{'geometry'} eq 11){
    $tiling->make_4_6_4pi_6_6();
  } elsif ($tiling->{'geometry'} eq 12){
    $tiling->make_8_4pi_4_8_4pi_4();
  } elsif ($tiling->{'geometry'} eq 13){
    $tiling->make_decomposed_4_8_8_test();
  } elsif ($tiling->{'geometry'} eq 14){
    $tiling->make_decomposed_9_9_6_4pi9__1_test();
  } elsif ($tiling->{'geometry'} eq 15){
    $tiling->make_6_6_6_crystal();
  } elsif ($tiling->{'geometry'} eq 16){
    $tiling->make_double_decomposed_4_4_4_4();
  } elsif ($tiling->{'geometry'} eq 17){
    $tiling->make_decomposed_4_8_8__2_test();
  } elsif ($tiling->{'geometry'} eq 18){
    $tiling->make_tower_test(%args);
  } elsif ($tiling->{'geometry'} eq 19){
    $tiling->make_kites_test();
  } elsif ($tiling->{'geometry'} eq 20){
    $tiling->make_log_spiral(%args);
  } elsif ($tiling->{'geometry'} eq 21){
    $tiling->make_spiral(%args);
  } elsif ($tiling->{'geometry'} eq 22){
    $tiling->make_3_12_12();
  } elsif ($tiling->{'geometry'} eq 23){
    $tiling->make_3_3_3_4_4();
  } elsif ($tiling->{'geometry'} eq 24){
    $tiling->make_square_weave();
  } elsif ($tiling->{'geometry'} eq 25){
    $tiling->make_instar();
  } elsif ($tiling->{'geometry'} eq 26){
    $tiling->make_decomposed_3_3_3_3_3_3();
  } elsif ($tiling->{'geometry'} eq 27){
    $tiling->make_3_3_3_3_3_3__3_3_4_3_4();
  } elsif ($tiling->{'geometry'} eq 28){
    $tiling->make_3_3_3_4_4__4_4_4_4__1();
  } elsif ($tiling->{'geometry'} eq 29){
    $tiling->make_rosette1(%args);
  } elsif ($tiling->{'geometry'} eq 30){
    $tiling->make_rosette2(%args);
  } elsif ($tiling->{'geometry'} eq 31){
    $tiling->make_windmill();
  } elsif ($tiling->{'geometry'} eq 32){
    $tiling->make_3_6_3_6__3_3_6_6();
  } elsif ($tiling->{'geometry'} eq 33){
    $tiling->make_3_3_3_3_3_3__3_3_6_6();
  } elsif ($tiling->{'geometry'} eq 34){
    $tiling->make_3_3_4_3_4();
  } elsif ($tiling->{'geometry'} eq 35){
    $tiling->make_decomposed_9_9_6_4pi9__2_test();
  } elsif ($tiling->{'geometry'} eq 36){
    $tiling->make_3_4_4_6__3_6_3_6__1_test();
  } elsif ($tiling->{'geometry'} eq 37){
    $tiling->make_3_4_4_6__3_6_3_6__2_test();
  } elsif ($tiling->{'geometry'} eq 38){
    $tiling->make_squarex_1();
  } elsif ($tiling->{'geometry'} eq 39){
    $tiling->make_squarex_2();
  } elsif ($tiling->{'geometry'} eq 40){
    $tiling->make_squarex_3();
  } elsif ($tiling->{'geometry'} eq 41){
    $tiling->make_squarex_4();
  } elsif ($tiling->{'geometry'} eq 42){
    $tiling->make_butterfly();
  } elsif ($tiling->{'geometry'} eq 43){
    $tiling->make_squave();
  } elsif ($tiling->{'geometry'} eq 44){
    $tiling->make_3_6_3_6_crystal();
  } elsif ($tiling->{'geometry'} eq 45){
    $tiling->make_kepler();
  } elsif ($tiling->{'geometry'} eq 46){
    $tiling->make_3_3_3_4_4__4_4_4_4__2();
  } elsif ($tiling->{'geometry'} eq 47){
    $tiling->make_3_4_3_12__3_12_12();
  } elsif ($tiling->{'geometry'} eq 48){
    $tiling->make_3_3_3_4_4__3_4_6_4();
  } elsif ($tiling->{'geometry'} eq 49){
    $tiling->make_3_3_4_3_4__3_4_6_4();
  } elsif ($tiling->{'geometry'} eq 50){
    $tiling->make_3_3_3_4_4__3_3_4_3_4__3_4_6_4();
  }  elsif ($tiling->{'geometry'} eq 51){
    $tiling->make_3_4_4_6__3_4_6_4();
  } elsif ($tiling->{'geometry'} eq 52){
    $tiling->make_3_3_3_3_3_3__3_3_4_12();
  } elsif ($tiling->{'geometry'} eq 99){
    $tiling->make_twist(%args);
  } elsif ($tiling->{'geometry'} eq 100){
    $tiling->make_test2();
  }

  # Check arguments to see if tiling should be transformed in any way

  if ($args{'rotate'}){
    $tiling->rotate($args{'rotate'},0,0);
  }

  if ($args{'stretch'}){
    $tiling->stretch($args{'stretch'});
  }

  # $tiling->reassign_creases(0,0);

  return $tiling;
}



#####################
# make_square_weave #
#####################
sub make_square_weave {
  my $tiling = shift @_;

  print STDERR "Using square_weave geometry\n";
  my $polygon1 = Adult::new_ngon('4');

  my $polygon2 = new Adult();
  my $p1=new Point(50,50,$polygon2);
  my $p2=new Point(50,-50,$polygon2);
  my $p3=new Point(150,50,$polygon2);
  $polygon2->add($p2,$p3,$p1);
  $polygon2->find_centre2();

  $polygon2->write();
  my $polygon3= $polygon2->copy();
  $polygon3->rotate(90,0,0);

  my $polygon4= $polygon2->copy();
  $polygon4->rotate(180,0,0);

  my $polygon5= $polygon2->copy();
  $polygon5->rotate(-90,0,0);

  my $polygon6 = Adult::new_ngon('8');
  $polygon6->scale(sqrt(2));
  $polygon6->translate(300,200);

  my $set= new PolygonSet(0,1,4,4,'colour1');
  $set->add($polygon1,$polygon2,$polygon3,$polygon4,$polygon5,$polygon6);
  $tiling->add($set);
}

###############################
# make_3_3_3_3_3_3__3_3_4_3_4 #
###############################
sub make_3_3_3_3_3_3__3_3_4_3_4 {
  my  $tiling = shift @_;

  my $description="3.3.3.3.3.3; 3.3.4.3.4";
  $tiling->description($description);

  print STDERR "Using $description geometry\n";

  my $polygon1  = Adult::new_ngon('6','1');
  my $polygon2  = Adult::new_ngon('6','1');
  my $polygon3  = Adult::new_ngon('4','1');
  my $polygon4  = Adult::new_ngon('4','1');
  $polygon4->rotate(30);
  my $polygon5  = Adult::new_ngon('4','1');
  my $polygon6  = Adult::new_ngon('4','1');
  $polygon6->rotate(-30);
  my $polygon7  = Adult::new_ngon('4','1');
  $polygon7->rotate(30);
  my $polygon8  = Adult::new_ngon('4','1');
  $polygon8->rotate(-30);
  my $polygon9  = Adult::new_ngon('3','2');
  $polygon9->rotate(-30);
  my $polygon10 = Adult::new_ngon('3','2');
  $polygon10->rotate(30);
  my $polygon11 = Adult::new_ngon('3','2');
  $polygon11->rotate(-30);
  my $polygon12 = Adult::new_ngon('3','2');
  $polygon12->rotate(30);

  my $set1  = new PolygonSet(0,1,4,6,'colour1');
  my $set2  = new PolygonSet(0,1,4,6,'colour1');
  my $set3  = new PolygonSet(0,1,4,6,'colour2');
  my $set4  = new PolygonSet(0,1,4,6,'colour2');
  my $set5  = new PolygonSet(0,1,4,6,'colour2');
  my $set6  = new PolygonSet(0,1,4,6,'colour2');
  my $set7  = new PolygonSet(0,1,4,6,'colour2');
  my $set8  = new PolygonSet(0,1,4,6,'colour2');
  my $set9  = new PolygonSet(0,1,4,6,'colour1');
  my $set10 = new PolygonSet(0,1,4,6,'colour1');
  my $set11 = new PolygonSet(0,1,4,6,'colour1');
  my $set12 = new PolygonSet(0,1,4,6,'colour1');

  my $decomposed_set1 = new PolygonSet(0,1,4,6,'colour1');
  $decomposed_set1->decompose($polygon1);
  $set1->multiple_rectangle_grid($decomposed_set1,0,0,473.2,273.2,$tiling->bbox());

  my $decomposed_set2 = new PolygonSet(0,1,4,6,'colour1');
  $decomposed_set2->decompose($polygon2);
  $set2->multiple_rectangle_grid($decomposed_set2,236.6,136.6,473.2,273.2,$tiling->bbox());

#  $set1->rectangle_grid($polygon1,0,0,473.2,273.2,$tiling->bbox());
#  $set2->rectangle_grid($polygon2,236.6,136.6,473.2,273.2,$tiling->bbox());
  $set3->rectangle_grid($polygon3,0,136.6,473.2,273.2,$tiling->bbox());
  $set4->rectangle_grid($polygon4,118.3,68.3,473.2,273.2,$tiling->bbox());
  $set5->rectangle_grid($polygon5,236.6,0,473.2,273.2,$tiling->bbox());
  $set6->rectangle_grid($polygon6,354.9,68.3,473.2,273.2,$tiling->bbox());
  $set7->rectangle_grid($polygon7,354.9,204.9,473.2,273.2,$tiling->bbox());
  $set8->rectangle_grid($polygon8,118.3,204.9,473.2,273.2,$tiling->bbox());
  $set9->rectangle_grid($polygon9,78.86,136.6,473.2,273.2,$tiling->bbox());
  $set10->rectangle_grid($polygon10,157.74,0,473.2,273.2,$tiling->bbox());
  $set11->rectangle_grid($polygon11,315.5,0,473.2,273.2,$tiling->bbox());
  $set12->rectangle_grid($polygon12,394.3,136.6,473.2,273.2,$tiling->bbox());

  $tiling->add($set1,$set2,$set3,$set4,$set5,$set6,$set7,$set8,$set9,$set10,$set11,$set12);
  return $tiling;
}

##############
# make_kites #
##############
sub make_kites {
  my $tiling = shift @_;

  my $description="Kites";
  $tiling->description($description);

  print STDERR "Using $description geometry\n";
  my $set1 = new PolygonSet(0,1,4,4,'colour1');
  my $set2 = new PolygonSet(0,1,4,4,'colour2');

  my @kite=("0 -57.74 50 28.87 1",
	    "50 28.87 0 57.74 1",
	    "0 57.74 -50 28.87 1",
	    "-50 28.87 0 -57.74 1");

  my $colour=1;
  my $polygon1 = Adult::new_from_old_data($colour,@kite);
  my $polygon2 = $polygon1->copy();
  $polygon2->reverse();

  $polygon2->rotate(180);

  $set1->rectangle_grid($polygon1,0,0,100,115.5,$tiling->bbox());
  $set2->rectangle_grid($polygon2,50,-28.87,100,115.5,$tiling->bbox());
  $tiling->add($set1,$set2);
  return $tiling;
}

################
# make_crystal #
################
sub make_crystal {
  my $tiling = shift @_;

  my $description="6.6.6 Crystal";
  $tiling->description($description);

  print STDERR "Using $description geometry\n";
  my $crystal_set = new PolygonSet(0,1,4,4,'colour1');
  my $polygon = Adult::new_ngon('6','3a');
  $crystal_set = crystal PolygonSet($polygon,6,6,0.3);

  my $set1 = new PolygonSet(0,1,6,6,'colour1');
  my $set2 = new PolygonSet(0,1,6,6,'colour1');
  my $set3 = new PolygonSet(0,1,6,6,'colour1');
  my $set4 = new PolygonSet(0,1,6,6,'colour1');

  $set1->multiple_rectangle_grid($crystal_set,0,0,300,4*86.60254,$tiling->bbox());
  $set2->multiple_rectangle_grid($crystal_set,150,86.6,300,4*86.60254,$tiling->bbox());
  $set3->multiple_rectangle_grid($crystal_set,0,173.2,300,4*86.60254,$tiling->bbox());
  $set4->multiple_rectangle_grid($crystal_set,150,-86.6,300,4*86.60254,$tiling->bbox());

  $tiling->add($set1,$set2,$set3,$set4);
  return $tiling;
}






###############################
# make_decomposed_3_3_3_3_3_3 #
###############################
sub make_decomposed_3_3_3_3_3_3 {
  my $tiling = shift @_;

  my $description="3.3.3.3.3.3 Part Decomposed";
  $tiling->description($description);

  print STDERR "Using $description geometry\n";
  
  my $set1 = new PolygonSet(0,1,3,3,'colour1');
  my $set2 = new PolygonSet(0,1,3,3,'colour2');
  my $set3 = new PolygonSet(0,1,3,3,'colour1');
  my $set4 = new PolygonSet(0,1,3,3,'colour2');
  
  my $triangle2 = Adult::new_ngon('3','1');
  $set3->rectangle_grid($triangle2,50,86.6,100,173.2,$tiling->bbox());
  
  my $triangle1 = Adult::new_ngon('3','1');
  $set1->rectangle_grid($triangle1,0,0,100,173.2,$tiling->bbox());
  
  
  my $triangle3 = Adult::new_ngon('3','2');
  $triangle3->rotate(60);
  #$set2->rectangle_grid($triangle3,0,57.7,100,173.2,$tiling->bbox());
  my $decomposed_set1 = new PolygonSet(0,1,3,3,'colour1');
  $decomposed_set1->decompose($triangle3);
  $set2->multiple_rectangle_grid($decomposed_set1,0,57.7,100,173.2,$tiling->bbox());
  
  my $triangle4 = Adult::new_ngon('3','2');
  $triangle4->rotate(60);
  my $decomposed_set2 = new PolygonSet(0,1,3,3,'colour1');
  $decomposed_set2->decompose($triangle4);
  $set4->multiple_rectangle_grid($decomposed_set2,50,144.3,100,173.2,$tiling->bbox());

  $tiling->add($set1,$set2,$set3,$set4);
}

##################################
# make_double_decomposed_4_4_4_4 #
##################################
sub make_double_decomposed_4_4_4_4 {
  my $tiling = shift @_;

  my $description="4.4.4.4 Double Decomposed";
  $tiling->description($description);

  print STDERR "Using $description geometry\n";
#  die "This geometry has been depracated as it is not foldable with current centres\n";
  my $set = new PolygonSet(0,1,4,4,'colour1');
  my $polygon = Adult::new_ngon('4','1');
  $polygon -> scale('2');
  my $decomposed_set = new PolygonSet(0,1,4,4,'colour1');
  $decomposed_set->double_decompose($polygon);

  $set->multiple_rectangle_grid($decomposed_set,0,0,200,200,$tiling->bbox(),'alternate');
  $tiling->add($set);
}

#########################
# make_decomposed_4_8_8 #
#########################
sub make_decomposed_4_8_8 {
  my $tiling = shift @_;

  my $description="Decomposed 4.8d.8d";
  $tiling->description($description);

  print STDERR "Using $description geometry\n";

  my $octagon = Adult::new_ngon('8','3a');
  my $square  = Adult::new_ngon('4','3b');

  my $octagon_set    = new PolygonSet(0,1,8,4,'colour1');
  my $square_set     = new PolygonSet(0,1,8,4,'colour1');
  my $decomposed_set = new PolygonSet(0,1,8,4,'colour1');
  $decomposed_set->decompose($octagon);
      
  # Rotate square
  $square->rotate('45');

  $octagon_set->multiple_rectangle_grid($decomposed_set,0,0,241.4,241.4,$tiling->bbox(),'alternate');
  $square_set ->rectangle_grid($square,120.7,120.7,241.4,241.4,$tiling->bbox(),'alternate');

  $tiling->add($octagon_set,$square_set);
}

############################
# make_decomposed_4_8_8__2 #
############################
sub make_decomposed_4_8_8__2 {
  my $tiling = shift @_;

  my $description="Decomposed 4d.8d.8d";
  $tiling->description($description);
  print STDERR "Using $description geometry\n";

  my $octagon1 = Adult::new_ngon('8','3a');
  my $octagon2 = Adult::new_ngon('8','3b');
  my $octagon3 = Adult::new_ngon('8','3a');
  my $octagon4 = Adult::new_ngon('8','3b');
  my $square   = Adult::new_ngon('4','3a');

  # Rotate square
  $square->rotate('45');

  my $octagon_set1    = new PolygonSet(0,1,8,4,'colour1');
  my $octagon_set2    = new PolygonSet(0,1,8,4,'colour2');
  my $octagon_set3    = new PolygonSet(0,1,8,4,'colour1');
  my $octagon_set4    = new PolygonSet(0,1,8,4,'colour2');
  my $square_set      = new PolygonSet(0,1,8,4,'colour1');
  my $decomposed_set1 = new PolygonSet(0,1,8,4,'colour1');
  my $decomposed_set2 = new PolygonSet(0,1,8,4,'colour1');
  my $decomposed_set3 = new PolygonSet(0,1,8,4,'colour1');
  my $decomposed_set4 = new PolygonSet(0,1,8,4,'colour1');
  my $decomposed_set5 = new PolygonSet(0,1,8,4,'colour1');
  
  $decomposed_set1->decompose($octagon1);
  $decomposed_set2->decompose($octagon2);
  $decomposed_set3->decompose($octagon3);
  $decomposed_set4->decompose($octagon4);
  $decomposed_set5->decompose($square);
  
  $octagon_set1->multiple_rectangle_grid($decomposed_set1,0,0,2*241.4,2*241.4,$tiling->bbox());
  $octagon_set2->multiple_rectangle_grid($decomposed_set2,241.4,0,2*241.4,2*241.4,$tiling->bbox());
  $octagon_set3->multiple_rectangle_grid($decomposed_set3,241.4,241.4,2*241.4,2*241.4,$tiling->bbox());
  $octagon_set4->multiple_rectangle_grid($decomposed_set4,0,241.4,2*241.4,2*241.4,$tiling->bbox());
  $square_set ->multiple_rectangle_grid($decomposed_set5,120.7,120.7,241.4,241.4,$tiling->bbox(),'alternate');

  $tiling->add($octagon_set1,$octagon_set2,$octagon_set3,$octagon_set4,$square_set);
}


##############################
# make_decomposed_9_9_6_4pi9 #
##############################
sub make_decomposed_9_9_6_4pi9 {
    my $tiling = shift @_;

    my $description="9.9.6*4pi/9 Decomposed ";
    $tiling->description($description);

    print STDERR "Using $description geometry\n";

    # Star sets
    my $star_set1 = PolygonSet::new_star(6,80,9,9,'1:-1:-1','alternate');
    my $star_set2 = PolygonSet::new_star(6,80,9,9,'1:-1:-1','alternate');

    # nonagon sets
    my $nonagon1  = Adult::new_ngon('9','2');
    $nonagon1->rotate(-10);
    my $nonagon2  = Adult::new_ngon('9','1');
    $nonagon2->rotate(10);
    my $nonagon3  = Adult::new_ngon('9','1');
    $nonagon3->rotate(10);
    my $nonagon4  = Adult::new_ngon('9','2');
    $nonagon4->rotate(-10);

    my $set1 = new PolygonSet(0,1,9,9,'colour2');
    my $set2 = new PolygonSet(0,1,9,9,'colour1');
    my $set3 = new PolygonSet(0,1,9,9,'colour1');
    my $set4 = new PolygonSet(0,1,9,9,'colour2');
    my $set5 = new PolygonSet(0,1,9,9,'colour3');
    my $set6 = new PolygonSet(0,1,9,9,'colour3');
    my $decomposed_set = new PolygonSet(0,1,9,9,'TWIST',$tiling->{'scale'});
    $decomposed_set->decompose($nonagon1);
    my $decomposed_set2 = new PolygonSet(0,1,9,9,'TWIST',$tiling->{'scale'});
    $decomposed_set2->decompose($nonagon4);
    
    $set6->multiple_rectangle_grid($star_set1,0,0,824.4,475.8,$tiling->bbox());
    $set5->multiple_rectangle_grid($star_set2,412.2,238.0,824.5,475.8,$tiling->bbox());

    $set1->multiple_rectangle_grid($decomposed_set,274.8,0,824.4,475.8,$tiling->bbox());
    
    $set2->rectangle_grid($nonagon2,549.6,0,824.4,475.8,$tiling->bbox());
    $set3->rectangle_grid($nonagon3,137.4,238.0,824.4,475.8,$tiling->bbox());
    
    $set4->multiple_rectangle_grid($decomposed_set2,687,238.0,824.4,475.8,$tiling->bbox());
    $tiling->add($set1,$set2,$set3,$set4,$set5,$set6);
}

###################
# make_9_9_6_4pi9 #
###################
sub make_9_9_6_4pi9 {
  my $tiling = shift @_;

  my $description="9.9.6*4pi/9 Decomposed ";
  $tiling->description($description);

  print STDERR "Using $description geometry\n";

  # Star sets
  my $star_set1 = PolygonSet::new_star(6,80,9,9,'1:-1:-1','alternate');
  my $star_set2 = PolygonSet::new_star(6,80,9,9,'1:-1:-1','alternate');

  # nonagon sets
  my $nonagon1  = Adult::new_ngon('9','2');
  $nonagon1->rotate(-10);
  my $nonagon2  = Adult::new_ngon('9','1');
  $nonagon2->rotate(10);
  my $nonagon3  = Adult::new_ngon('9','1');
  $nonagon3->rotate(10);
  my $nonagon4  = Adult::new_ngon('9','2');
  $nonagon4->rotate(-10);

  my $set1 = new PolygonSet(0,1,9,9,'colour2');
  my $set2 = new PolygonSet(0,1,9,9,'colour1');
  my $set3 = new PolygonSet(0,1,9,9,'colour1');
  my $set4 = new PolygonSet(0,1,9,9,'colour2');
  my $set5 = new PolygonSet(0,1,9,9,'colour3');
  my $set6 = new PolygonSet(0,1,9,9,'colour3');

  $set6->multiple_rectangle_grid($star_set1,0,0,824.4,475.8,$tiling->bbox());
  $set5->multiple_rectangle_grid($star_set2,412.2,238.0,824.5,475.8,$tiling->bbox());
  $set1->rectangle_grid($nonagon1,274.8,0,824.4,475.8,$tiling->bbox());
  $set2->rectangle_grid($nonagon2,549.6,0,824.4,475.8,$tiling->bbox());
  $set3->rectangle_grid($nonagon3,137.4,238.0,824.4,475.8,$tiling->bbox());
  $set4->rectangle_grid($nonagon4,687,238.0,824.4,475.8,$tiling->bbox());

  $tiling->add($set1,$set2,$set3,$set4,$set5,$set6);
}





########################
# render_light_pattern #
########################

=head1 render_light_pattern

To make light patterns use sort of ray tracing.  This version uses a linesweep
algorithm to make a large speedup.

You must specifiy the pixel size, a value of 1 will give high resolution,
10 will give a very lower resolution light pattern.

Prints output object to FILEHANDLE in postscript format

    $self->render_light_pattern($current_tag,$stack_ref,$pixel_size,$tag_ref);

=cut

sub render_light_pattern {
    my ($self,$current_tag,$stack_ref,$pixel_size,$tag_ref) = @_;
    my @tags=@{$tag_ref};

    warn "Why on earth is this code here and not in View::Postscript?";

    if (! $pixel_size){
	die "In Tiling::render_light_pattern no pixel size specified\n";
    }

    #warn "Printing light pattern to XML stack with pixel size of $pixel_size ...\n";

    my $hp=$pixel_size/2; # Half pixel

    # Bounding_box parameters This is probably not good, should get them
    # externally.  What will happen when I change the values?
#    my $bbox_startx = -275;
#    my $bbox_endx   = 275;
#    my $bbox_starty = -275;
#    my $bbox_endy   = 275;
    my $bbox_startx = -1275;
    my $bbox_endx   = 1275;
    my $bbox_starty = -1275;
    my $bbox_endy   = 1275;

    #warn "Bounding box param are $bbox_startx,$bbox_endx,$bbox_starty,$bbox_endy\n";

    ############################
    # Make list of all polygon #
    ############################    
    my @data;
    foreach my $polygonset ($self->each()){
	foreach my $polygon ($polygonset->each()){
	    push(@data,$polygon);
	}
    }
    
    ###################################################
    # Make hashes to store min and max x and y values #
    # for each polygon to speed up                    #
    ###################################################
    my (@datakeys, %polygon_xmin,%polygon_xmax,%polygon_ymin,%polygon_ymax,);
    foreach my $polygon (@data){
	my $xminvalue= 1000000;
	my $xmaxvalue=-1000000;
	my $yminvalue= 1000000;
	my $ymaxvalue=-1000000;
	foreach my $point ($polygon->each()){ 
	    if ($point->x()<$xminvalue){
		$xminvalue=$point->x();
	    } 
	    if ($point->x()>$xmaxvalue){
		$xmaxvalue=$point->x();
	    }
	    if ($point->y()<$yminvalue){
		$yminvalue=$point->y();
	    } 
	    if ($point->y()>$ymaxvalue){
		$ymaxvalue=$point->y();
	    }
	}
	$polygon_xmin{$polygon}=$xminvalue;
	$polygon_xmax{$polygon}=$xmaxvalue;
	$polygon_ymin{$polygon}=$yminvalue;
	$polygon_ymax{$polygon}=$ymaxvalue;
    }
        
    #################################################
    # Loop over each line of pixels in tessellation #
    #################################################


    my $open_tag ="  <light_pattern tag=\"$current_tag\" resolution=\"$pixel_size\">\n";
    my $close_tag="  <light_pattern/>\n";
    push (@tags,'1');
    $current_tag++;

    $$stack_ref .= $open_tag;

    my $y=$bbox_starty;
    while($y<$bbox_endy){
	# Scan y lines
	my $p1 = new Point(-1000000,$y);
	my $p2 = new Point(1000000,$y);
	my $line = new Line ($p1,$p2,2);

	my %store_intersects; # Keeps number of layers to add or subtract as
                              # you scan across lines

        # Find all polygons that have a point that crosses x line
	# Check hashes
	foreach my $polygon (@data){
	    # If polygon crosses y line get left and right values and add to a hash 
	    if ($polygon_ymin{$polygon} < $y and $polygon_ymax{$polygon} > $y){
		# Check intersects
		my @intersects=$polygon->intersect($line);

		if (@intersects == 2){
		    my ($point1,$point2)=@intersects;

		    my $x1=$point1->x();
		    my $x2=$point2->x();

		    if ($x1>$x2){
			$store_intersects{$x1} -= 1;
			$store_intersects{$x2} += 1;
		    } else {
			$store_intersects{$x1} += 1;
			$store_intersects{$x2} -= 1;
		    }
		} else {
		    warn "Did not get two intersects for polygon! In render_light_pattern. Ignoring\nIntersects @intersects\n";
		    $polygon->write();
		    $line->write();
		    $line->translate(0,+$hp);
		    $line->write();

		    my @intersects=$polygon->intersect($line);

		    if (@intersects == 2){
			my ($point1,$point2)=@intersects;

			my $x1=$point1->x();
			my $x2=$point2->x();
			
			if ($x1>$x2){
			    $store_intersects{$x1} -= 1;
			    $store_intersects{$x2} += 1;
			} else {
			    $store_intersects{$x1} += 1;
			    $store_intersects{$x2} -= 1;
			}
		    } else {
			warn "Oh no failed again to get two intersects! Will be problems with light rendering. Sorry!\nIntersects @intersects\n";
		    }
		    $line->translate(0,-$hp);
		}
	    }
	}

	################################
	# Now render line to XML stack #
	################################
	my $oldthickness = 0;
	my $xstart=-600;

	foreach my $element (sort {$a <=> $b} keys %store_intersects){

	    my $newthickness=$oldthickness+$store_intersects{$element};
	    my $x=$element;
	    my $grey_scale=1-(0.05*$oldthickness);

	    my $x1=int($xstart);
	    my $y1=$y-$hp;
	    my $x2=int($x);
	    my $y2=$y-$hp;
	    my $x3=int($x);
	    my $y3=$y+$hp;
	    my $x4=int($xstart);
	    my $y4=$y+$hp;

	    if ($oldthickness>0){
		my $rectangle = "    <polygon tag=\"$current_tag\" layers=\"$oldthickness\" x1=\"$x1\" y1=\"$y1\" x2=\"$x2\" y2=\"$y2\" x3=\"$x3\" y3=\"$y3\" x4=\"$x4\" y4=\"$y4\"/>\n";
		$current_tag++;
		push (@tags,'1');
		$$stack_ref .= $rectangle;
	    }

	    # Reset variables
	    $xstart=$x;
	    $oldthickness=$newthickness;
	}
	$y=$y+$pixel_size;

    }
    $$stack_ref .= $close_tag;

    return ($current_tag,$stack_ref,\@tags);
}

# Internal method for render_postscript
sub _add_to_hash {
    my %hash=%{shift @_};
    my $x1=shift @_;
    my $x2=shift @_;
}

##########################
# opaque_postscript_body #
##########################
=head1 opaque_postscript_body

Naive attempt to get folded pattern from opaque paper.

Prints output object to FILEHANDLE in postscript format

    $output ->opaque_postscript_body(\*FILEHANDLE,$parameters);

=cut

sub opaque_postscript_body {
    my ($self,$FILEHANDLE,$parameters) = @_;

#    $self->postscript_header($FILEHANDLE);

    # Gonna make big assumption that this object is a tiling object!
    foreach my $tiling (@{$self->{'list'}}){
	print STDERR  "In Output::opaque_postscript $tiling\n";

	# Make a hash with points from all lines that contains number of points
	# in vertex and also another hash with total sum of creases, which should
	# be +2 or -2 to satisfy kawasaki's theorem.

	# Make list of point objects ordered by x coordinate - Borrowed
	# from Perl 4 Camel book page 249.
	my (%data,@data,%owner,%owner_crease);
	foreach my $polygonset ($tiling->each()){
	    foreach my $polygon ($polygonset->each()){
		foreach my $line ($polygon->eachline()){
		    my ($p1,$p2)=$line->points();

		    #if ($owner_crease{$p1} or $owner_crease{$p2}){
		#	warn "Already looked at this point already $p1 or $p2\n";
		#    }

		    my $crease=$line->crease();
		    if (! $crease){
			die "No crease parity for $line\n";
		    }

		    $owner_crease{$p1}=$owner_crease{$p1}+$crease; # Get parity of crease for each line
		    $owner_crease{$p2}=$owner_crease{$p2}+$crease; # Get parity of crease for each line

		    ###############################
		    # List of all points in $data #
		    ###############################
		    if (! $data{$p1}){
			push(@data,$p1);
			$data{$p1}=1;
		    }

		    if (! $data{$p2}){
			push(@data,$p2);
			$data{$p2}=1;
		    }

		    ###################
		    # Do I need this? #
		    ###################
		    $owner{$p1}=$polygon;
		    $owner{$p2}=$polygon;
		}
	    }
	}

	###############################################
	# Sort all points by x coordinate to speed up #
	###############################################
	my (@datakeys);
	foreach my $point (@data){
	    push (@datakeys,$point->x());
	}
	my @sortdata=@data[sort {$datakeys[$a] <=> $datakeys[$b]} $[..$#data];

	my (%number_of_lines,%crease_parity);

# Maybe I shouldn't shift elements! #

	while(@sortdata){       # Not all vertices assigned
	    # Shift first element of list into vertex object
	    my $point=shift @sortdata;

	    if (! $point){die "No point!\n";}
	    
	    my @pointlist;
	    push(@pointlist,$point);

	    ######################################################    
	    # Traverse list checking if next point is close with #
	    # (x1-x2)^2+(y1-y2)^2 < distance^2                   #
	    ######################################################

	    my $nf;                          # Element of array @sortdata
	    for ($nf=0; $nf<@sortdata; $nf++){
		my $test_point=$sortdata[$nf];
		my $xdist=$point->x()-$test_point->x();
		my $ydist=$point->y()-$test_point->y();
		if ($xdist<-1){ # Short circuit speed up
		    last;
		}
		if ($ydist*$ydist<1 and $xdist*$xdist<1){
		    push(@pointlist,splice @sortdata,$nf,1);
		    $nf--;
		}
	    }
	    
	    #print STDERR @pointlist,"\n";

	    ########################################
	    # OK now we can count number of points #
	    # and tot up crease parities           #
	    ########################################
	    my $number_of_lines=@pointlist;

	    if (! $number_of_lines){
		die "No number of lines for $point!!\n";
	    }

	    foreach my $foopoint (@pointlist){ 
		$number_of_lines{$foopoint}=$number_of_lines;
	    }

	    if (! $number_of_lines{$point}){
		die "No number of lines for $point!!!!\n";
	    }

	    # Get overall parity of vertex
	    my $crease_total;
	    foreach my $point (@pointlist){
		if ($owner_crease{$point}){
		    $crease_total = $crease_total+$owner_crease{$point};
		} else {
		    die "No crease for point $point\n";;
		}
	    }

	    if (! $crease_total and ! $number_of_lines==2){
		die "No value for crease total! for @pointlist\n";
	    }

	    foreach my $point (@pointlist){
		$crease_parity{$point}=$crease_total;
	    }

	    #print STDERR "There are $number_of_lines{$point} parity:$crease_total in this vertex containing $point\n";
	}

	# OK now have the hashes we wanted.
	# Now check all lines
	foreach my $set ($tiling->each()){
	    foreach my $polygon ($set->each()){
		foreach my $line ($polygon->eachline()){
		    # Test parity of line
		    if ($line->crease() == -1){

			# Now check if a +2,+2 line
			my ($p1,$p2)=$line->points();
			if (! $number_of_lines{$p1}){
			    $p1->write();
			    die "No number of lines, Hmm! for p1 : $p1\n";
			}
			if (! $number_of_lines{$p2}){
			    $p2->write();
			    die "No number of lines, Hmm! for p2 : $p2\n";
			}

			my $parity1=$crease_parity{$p1};
			my $parity2=$crease_parity{$p2};

			#print STDERR "For $line parity [$parity1] [$parity2]\n";

			if (! $parity1 and not $number_of_lines{$p1} == 2){
			    
			    $p1->write();
			    $line->write();
			    warn "No parity for $p1 in $line , number lines:$number_of_lines{$p1}\n";
			}

			if (! $parity2 and not $number_of_lines{$p2} == 2){
			    $p2->write();
			    $line->write();
			    $owner{$p2}->write();
			    warn "No parity for $p2 in $line , number lines:$number_of_lines{$p2}  $owner_crease{$p2}  $owner{$p2}\n";
			}

			#########################
			# Print all -2,-2 lines #
			#########################

			# This is probably oversimplified there will probably be times when
                        # these lines do intersect and need to be trimmed!
			if ($parity1==-2 and $parity2==-2){
			    #$line->label('-2,-2');
			    $line->postscript($FILEHANDLE);
			}

			###################################
			# Swap -2,+2 lines to +2,-2 lines #
			###################################
			if ($parity1==-2 and $parity2==2){
			    $line->swap();
			    ($p1,$p2)=$line->points();
			    $parity1=$crease_parity{$p1}; # Reset parity parameters
			    $parity2=$crease_parity{$p2};
			} 

			if ($parity1==2 and $parity2==-2){
			    # Find closest intersection point, possibly with -2,-2 line # 
			    # Get intersections with all polygons
			    my @all_intersect;
			    foreach my $testset ($tiling->each()){
				foreach my $test_polygon ($testset->each()){
				    if ($polygon == $test_polygon){ 
					next;
				    }

				    my @intersect;
				    foreach my $testline ($test_polygon->eachline()){
					# Only look at -1 lines #
					if ($testline->crease() == 1){
					    #die "Ignore these lines\!!n";
					} else {
					    my $intersect=$testline->intersect($line);

					    if ($intersect == 0){} elsif ($intersect == -1){} else {
						push (@intersect,$intersect);
					    }
					}
				    }
				    push (@all_intersect,@intersect);
				}
			    }

			    ########################################################
			    # Loop over array and find furthest intersection point #
			    ########################################################
			    my $number_intersect=@all_intersect;

			    my $max_distance=0;
			    my $closest_point;

			    if ($number_intersect){
				foreach my $intersect_point (@all_intersect){
				    #print STDERR "I point: $intersect_point\n";
				    my $new_distance=$intersect_point->distance($p2);
				    if ($new_distance>$max_distance){
					$max_distance=$new_distance;
					$closest_point=$intersect_point;
				    }
				}
			    } else {
				# do nothing, line should still get printed below!
			    }

			    if (! $closest_point){
				$line->postscript($FILEHANDLE);
			    } else {
				my $clipped_line=new Line($p2,$closest_point,-1);
				$clipped_line->postscript($FILEHANDLE);
			    }
			}


			##########################################
			# Print out certain points for debugging #
			##########################################
			my $debug=0;
			if ($debug){
			    if ($parity1 == 2){
				$p1->postscript($FILEHANDLE);
			    }			   
			    if ($parity2 == 2){
				$p2->postscript($FILEHANDLE);
			    }
			}
 
		    } else {
			# Don't print line, wrong crease assignment
		    }
		}
	    }
	}
    }
#    $self->postscript_footer($FILEHANDLE);
}

#########
# write #
#########
=head1 write

Debugging method: Write tiling to STDERR

$tiling->write();

=cut
sub write {
  my $tiling = shift @_;      # Get object

  print STDERR "Writing $tiling\n";
  if (! $tiling){
    die "In tilng::write not passed a tiling!\n";
  }

  foreach my $set ($tiling->each()){
    $set->write();
  }
}

##########
# remove #
##########
=head1 remove

Will remove a polygonset object from tiling.  Will return 1 if removed
and 0 if not.

$tiling->remove($polygonset);

=cut
sub remove {
  my($tiling,$polygonset)=@_;
  my $return=0;
  my @array;

  # Hmm got to be careful modifying arrays while looping over them.
  foreach my $tmp_polygonset ($tiling->each()){
    if ($polygonset eq $tmp_polygonset){
      #print "Yep this polygonset is in the tiling $tiling, removing\n";
      $return=1;
    } else {
      push (@array,$tmp_polygonset);
    }
  }
  # Replace point list!
  #print "Replacing @array\n";
  $tiling->{'sets'}=[];

  foreach my $element (@array){
    $tiling->add($element);
  }
  return $return;
}


###################################################################################

                                  # CODE IN PROGRESS #

# These are new tiling generation routines using the patch object.
# Eventual plan is to remove these patches to a seperate file as a precursor
# to user driven patch generation.

###################################################################################
########################
# make_8_4pi_4_8_4pi_4 # WEAVE?
########################
sub make_test {
  my $self = shift @_;

  $self->description('8.4*pi/4.8.4*pi/4 weave? patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>261.3,  'y1'=>0,'alternate1'=>'false',
			'x2'=>0,'y2'=>261.3,  'alternate2'=>'false');

  my $polygon = Adult::new_ngon('8','1');
  $polygon->rotate(22.5);

  $patch->add($polygon);

  # Star polygons
  my @foo;
  my $star_set = PolygonSet::new_star(4,45,4,8,'-1:-1:-1');
  $star_set->translate(-130.7,-130.7);
  foreach my $p ($star_set->each()){
    $p->write(); 
    $p->colour(2);
    unshift(@foo,$p);
  }
  $patch->add(@foo);
  warn "@foo";

  #$set1->multiple_rectangle_grid($star_set,130.7,130.7,261.3,261.3,$tiling->bbox());
  #$set2->rectangle_grid($polygon,0,0,261.3,261.3,$tiling->bbox());

  $self->expand_patch($patch);
}

sub make_test2 {
  my $self=shift @_;

  $self->description('3.3.4.12 species');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>236.6,'y1'=>409.8,'alternate1'=>'false',
			'x2'=>473.2,'y2'=>0,  'alternate2'=>'false');

#  $patch->make_species(3,3,4,12);
  $patch->make_species(3,3,6,6);
  #$patch->make_species(3,6,3,6);
  my $primarypatch = new PrimaryPatch();
  foreach my $p ($patch->each()){
    $primarypatch->add($p->copy());
  }
  $self->patch($primarypatch);
  

  $self->add($patch);

  $self->vertexify();

  my $i=0;
  my $pi = atan2(1,1)*4;

  my $theta_plus_thi=90;
  #my @thi=(150,30,135,75);
    my @thi=(30,150,60,120);
 # my @thi=(30,120,30,120);

  # Calculate alphas

  my @alpha=(0.2);
  foreach my $thi (@thi){
    $i++;

#$psi_1=$psi_1*$pi/180;

    $alpha[$i]=sin($thi[0]*$pi/180)*$alpha[0]/sin($thi[$i%4]*$pi/180);
    warn "$i alpha $alpha[$i] $thi";
  }



  $i=0;
  foreach my $p ($patch->each()){
    $p->scale($alpha[$i]);

    my $theta=$theta_plus_thi - $thi[$i];

    warn "Theta $theta";
    $p->rotate($theta);
    $i++;
  }
}




##########################
# make_ 3_4_4_6__3_4_6_4 #
##########################
sub make_3_4_4_6__3_4_6_4 {
  my $self = shift @_;

  $self->description('3.4.4.6;3.4.6.4 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>236.6,'y1'=>409.8,'alternate1'=>'false',
			'x2'=>473.2,'y2'=>0,  'alternate2'=>'false');

  #########################
  # 3.4.6 dodecagon patch #
  #########################

  my $p1=Adult::new_ngon('6','1');
  my $p2=Adult::new_ngon('4','2');
  $p2->translate(0,136.6);
  my $p3  = $p2->copy();
  $p3->rotate(60,0,0);
  my $p4  = $p2->copy();
  $p4->rotate(120,0,0);
  my $p5  = $p2->copy();
  $p5->rotate(180,0,0);
  my $p6  = $p2->copy();
  $p6->rotate(240,0,0);
  my $p7  = $p2->copy();
  $p7->rotate(300,0,0);
  my $p8  = Adult::new_ngon('3','1');
  $p8->translate(0,157.74);
  $p8->rotate(30,0,0);
  my $p9=$p8->copy();
  $p9->rotate(60,0,0);
  my $p10=$p8->copy();
  $p10->rotate(120,0,0);
  my $p11=$p8->copy();
  $p11->rotate(180,0,0);
  my $p12=$p8->copy();
  $p12->rotate(240,0,0);
  my $p13=$p8->copy();
  $p13->rotate(300,0,0);

  $patch->add($p1,$p2,$p3,$p4,$p5,$p6,$p7,$p8,$p9,$p10,$p11,$p12,$p13);
  $patch->rotate(30,0,0);

  my $square1   = Adult::new_ngon('4','1');
  $square1->translate(0,236.6);
  $square1->rotate(-30,0,0);

  my $hexagon1  = Adult::new_ngon('6','2');
  $hexagon1->translate(0,273.2);

  my $square2=$square1->copy();
  $square2->rotate(60,0,0);
  my $square3=$square1->copy();
  $square3->rotate(120,0,0);

  my $hexagon2=$hexagon1->copy();
  $hexagon2->rotate(60,0,0);

  $patch->add($square1,$hexagon1,$square2,$square3,$hexagon2);

  $self->expand_patch($patch);
  return $self;
}




###########################
# make_3_3_4_3_4__3_4_6_4 #
###########################
sub make_3_3_4_3_4__3_4_6_4 {
  my $self = shift @_;

  $self->description('3.3.4.3.4;3.4.6.4 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>186.2,    'y1'=>323.2,'alternate1'=>'false',
			'x2'=>373.2,'y2'=>0,    'alternate2'=>'true');

  #########################
  # 3.4.6 dodecagon patch #
  #########################

  my $p1=Adult::new_ngon('6','1');
  my $p2=Adult::new_ngon('4','2');
  $p2->translate(0,136.6);
  my $p3  = $p2->copy();
  $p3->rotate(60,0,0);
  my $p4  = $p2->copy();
  $p4->rotate(120,0,0);
  my $p5  = $p2->copy();
  $p5->rotate(180,0,0);
  my $p6  = $p2->copy();
  $p6->rotate(240,0,0);
  my $p7  = $p2->copy();
  $p7->rotate(300,0,0);
  my $p8  = Adult::new_ngon('3','1');
  $p8->translate(0,157.74);
  $p8->rotate(30,0,0);
  my $p9=$p8->copy();
  $p9->rotate(60,0,0);
  my $p10=$p8->copy();
  $p10->rotate(120,0,0);
  my $p11=$p8->copy();
  $p11->rotate(180,0,0);
  my $p12=$p8->copy();
  $p12->rotate(240,0,0);
  my $p13=$p8->copy();
  $p13->rotate(300,0,0);

  $patch->add($p1,$p2,$p3,$p4,$p5,$p6,$p7,$p8,$p9,$p10,$p11,$p12,$p13);

  #######################
  my $triangle1  = Adult::new_ngon('3','3b'); # left top CORRECT 
  $triangle1->rotate(60);
  $triangle1->translate(0,215.5);
  my $triangle2  = Adult::new_ngon('3','3a'); # top CORRECT
  $triangle2->rotate(-120); # To get crease pattern correct!
  $triangle2->translate(186.6,107.7);

  $patch->add($triangle1,$triangle2);

  $self->expand_patch($patch);

  return $self;
}

###########################
# make_3_3_3_4_4__3_4_6_4 #
###########################
sub make_3_3_3_4_4__3_4_6_4 {
  my $self = shift @_;

  $self->description('3.3.3.4.4;3.4.6.4 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>186.2,    'y1'=>323.2,'alternate1'=>'false',
			'x2'=>373.2,'y2'=>0,    'alternate2'=>'true');

  #########################
  # 3.4.6 dodecagon patch #
  #########################

  my $p1=Adult::new_ngon('6','1');
  my $p2=Adult::new_ngon('4','2');
  $p2->translate(0,136.6);
  my $p3  = $p2->copy();
  $p3->rotate(60,0,0);
  my $p4  = $p2->copy();
  $p4->rotate(120,0,0);
  my $p5  = $p2->copy();
  $p5->rotate(180,0,0);
  my $p6  = $p2->copy();
  $p6->rotate(240,0,0);
  my $p7  = $p2->copy();
  $p7->rotate(300,0,0);
  my $p8  = Adult::new_ngon('3','1');
  $p8->translate(0,157.74);
  $p8->rotate(30,0,0);
  my $p9=$p8->copy();
  $p9->rotate(60,0,0);
  my $p10=$p8->copy();
  $p10->rotate(120,0,0);
  my $p11=$p8->copy();
  $p11->rotate(180,0,0);
  my $p12=$p8->copy();
  $p12->rotate(240,0,0);
  my $p13=$p8->copy();
  $p13->rotate(300,0,0);

  $patch->add($p1,$p2,$p3,$p4,$p5,$p6,$p7,$p8,$p9,$p10,$p11,$p12,$p13);
  $patch->rotate(30,0,0);

  #######################
  my $triangle1  = Adult::new_ngon('3','3b'); # left top CORRECT 
  $triangle1->rotate(60);
  $triangle1->translate(0,215.5);
  my $triangle2  = Adult::new_ngon('3','3a'); # top CORRECT
  $triangle2->rotate(-120); # To get crease pattern correct!
  $triangle2->translate(186.6,107.7);

  $patch->add($triangle1,$triangle2);

  $self->expand_patch($patch);

  return $self;
}


######################################
# make_3_3_3_4_4__3_3_4_3_4__3_4_6_4 #
######################################
sub make_3_3_3_4_4__3_3_4_3_4__3_4_6_4 {
  my $self = shift @_;

  $self->description('3.3.3.4.4;3.3.4.3.4;3.4.6.4 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>373.2,  'y1'=>0,'alternate1'=>'true',
			'x2'=>0,'y2'=>373.2,  'alternate2'=>'true');

  #########################
  # 3.4.6 dodecagon patch #
  #########################

  my $p1=Adult::new_ngon('6','1');
  my $p2=Adult::new_ngon('4','2');
  $p2->translate(0,136.6);
  my $p3  = $p2->copy();
  $p3->rotate(60,0,0);
  my $p4  = $p2->copy();
  $p4->rotate(120,0,0);
  my $p5  = $p2->copy();
  $p5->rotate(180,0,0);
  my $p6  = $p2->copy();
  $p6->rotate(240,0,0);
  my $p7  = $p2->copy();
  $p7->rotate(300,0,0);
  my $p8  = Adult::new_ngon('3','1');
  $p8->translate(0,157.74);
  $p8->rotate(30,0,0);
  my $p9=$p8->copy();
  $p9->rotate(60,0,0);
  my $p10=$p8->copy();
  $p10->rotate(120,0,0);
  my $p11=$p8->copy();
  $p11->rotate(180,0,0);
  my $p12=$p8->copy();
  $p12->rotate(240,0,0);
  my $p13=$p8->copy();
  $p13->rotate(300,0,0);

  $patch->add($p1,$p2,$p3,$p4,$p5,$p6,$p7,$p8,$p9,$p10,$p11,$p12,$p13);
  $patch->translate(186.6,186.6);

  #######################

  my $square    = Adult::new_ngon('4','1');
  my $triangle1 = Adult::new_ngon('3','2');
  $triangle1->translate(0,78.9);
  $triangle1->rotate(60);

  my $triangle2=$triangle1->copy();
  $triangle2->rotate(90,0,0);
  my $triangle3=$triangle1->copy();
  $triangle3->rotate(180,0,0);
  my $triangle4=$triangle1->copy();
  $triangle4->rotate(270,0,0);

  $patch->add($triangle1,$square,$triangle2,$triangle3,$triangle4);

  $self->expand_patch($patch);
}

########################
# make_8_4pi_4_8_4pi_4 #
########################
sub make_8_4pi_4_8_4pi_4 {
  my $self = shift @_;

  $self->description('8.4*pi/4.8.4*pi/4 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>261.3,  'y1'=>0,'alternate1'=>'false',
			'x2'=>0,'y2'=>261.3,  'alternate2'=>'false');


  # Star polygons
  my $star_set = PolygonSet::new_star(4,45,4,8,'-1:-1:-1');
  $star_set->translate(130.7,130.7);
  foreach my $p ($star_set->each()){
    $patch->add($p);
  }

  my $polygon = Adult::new_ngon('8','1');
  $polygon->rotate(22.5);

  $patch->add($polygon);

  #$set1->multiple_rectangle_grid($star_set,130.7,130.7,261.3,261.3,$tiling->bbox());
  #$set2->rectangle_grid($polygon,0,0,261.3,261.3,$tiling->bbox());

  $self->expand_patch($patch);
}

####################
# make_4_6_4pi_6_6 #
####################
sub make_4_6_4pi_6_6 {
  my $self = shift @_;

  $self->description('4.6.4*pi/6.6 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>386.4,  'y1'=>0,'alternate1'=>'false',
			'x2'=>0,'y2'=>386.4,  'alternate2'=>'false');

  # Star polygons
  my $star_set1 = PolygonSet::new_star(4,30,6,6,'-1:-1:-1');
  my $star_set2 = PolygonSet::new_star(4,30,6,6,'-1:-1:-1');

  foreach my $p ($star_set1->each()){
      $patch->add($p);
  }

  $star_set2->translate(193.2,193.2);
  foreach my $p ($star_set2->each()){
      $patch->add($p);
  }

  # ngons
  my $polygon3 = Adult::new_ngon('4','2');
  my $polygon4 = Adult::new_ngon('4','2');
  my $polygon5 = Adult::new_ngon('6','1');
  my $polygon6 = Adult::new_ngon('6','1');
  my $polygon7 = Adult::new_ngon('6','1');
  my $polygon8 = Adult::new_ngon('6','1');

  $polygon3->rotate(45);
  $polygon3->translate(193.2,0);
  $polygon4->rotate(45);
  $polygon4->translate(0,193.2);
  $polygon5->rotate(-15);
  $polygon5->translate(96.6,96.6);
  $polygon6->rotate(15);
  $polygon6->translate(289.8,96.6);
  $polygon7->rotate(15);
  $polygon7->translate(96.6,289.8);
  $polygon8->rotate(-15);
  $polygon8->translate(289.8,289.8);

  $patch->add($polygon3,$polygon4,$polygon5,$polygon6,$polygon7,$polygon8);
  
  $self->expand_patch($patch);

  return $self;
}


##############################
# make_3_3_3_4_4__4_4_4_4__1 #
##############################
sub make_3_3_3_4_4__4_4_4_4__1 {
  my $self = shift @_;

  $self->description('3.3.3.4.4;4.4.4.4 v1 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>200,  'y1'=>0,'alternate1'=>'true',
			'x2'=>50,'y2'=>286.6,    'alternate2'=>'true');


  my $polygon1 = Adult::new_ngon('4','2');
  my $polygon7 = Adult::new_ngon('4','3b');
  $polygon7->translate(0,100);
  my $polygon9  = Adult::new_ngon('4','3a');
  $polygon9->translate(100,0);
  my $polygon10 = Adult::new_ngon('4','1');
  $polygon10->translate(100,100);

  my $polygon3 = Adult::new_ngon('3','2');
  $polygon3->rotate(60);
  $polygon3->translate(0,178.9);
  my $polygon4 = Adult::new_ngon('3','1');
  $polygon4->translate(50,207.7);

  my $polygon5 = Adult::new_ngon('3','2');
  $polygon5->rotate(60);
  $polygon5->translate(100,178.9);
  my $polygon6 = Adult::new_ngon('3','1');
  $polygon6->translate(150,207.7);

  $patch->add($polygon1,$polygon7,$polygon9,$polygon10,$polygon3,$polygon4,$polygon5,$polygon6);

  $self->expand_patch($patch); # problem is somewhere here!

  $self->reassign_creases(10000,0);
  return $self;
}


##############################
# make_3_3_3_4_4__4_4_4_4__2 #
##############################
sub make_3_3_3_4_4__4_4_4_4__2 {
  my $self = shift @_;

  $self->description('3.3.3.4.4;4.4.4.4 v2 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>200,  'y1'=>0, 'alternate1'=>'false',
			'x2'=>50,'y2'=>386.6,'alternate2'=>'true');


  my $polygon1 = Adult::new_ngon('4','2');
  my $polygon7 = Adult::new_ngon('4','3b');
  $polygon7->translate(0,100);
  my $polygon9  = Adult::new_ngon('4','3a');
  $polygon9->translate(100,0);
  my $polygon10 = Adult::new_ngon('4','1');
  $polygon10->translate(100,100);
  my $polygon11 = Adult::new_ngon('4','2');
  $polygon11->translate(0,200);
  my $polygon12 = Adult::new_ngon('4','3a');
  $polygon12->translate(100,200);




  my $polygon3 = Adult::new_ngon('3','1');
  $polygon3->rotate(60);
  $polygon3->translate(0,278.9);
  my $polygon4 = Adult::new_ngon('3','2');
  $polygon4->translate(50,307.7);

  my $polygon5 = Adult::new_ngon('3','1');
  $polygon5->rotate(60);
  $polygon5->translate(100,278.9);
  my $polygon6 = Adult::new_ngon('3','2');
  $polygon6->translate(150,307.7);

  $patch->add($polygon1,$polygon7,$polygon9,$polygon10,$polygon3,$polygon4,$polygon5,$polygon6,$polygon11,$polygon12);

  $self->expand_patch($patch); # problem is somewhere here!

  return $self;
}



##################
# make_squarex_1 #
##################
sub make_squarex_1 {
  my $self = shift @_;

  $self->description('Squarex tiling patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>200,  'y1'=>0,'alternate1'=>'true',
			'x2'=>0,'y2'=>200,    'alternate2'=>'true');

  # Star set
  my $star_set = PolygonSet::new_star(4,90,4,4,'1:1:1');

  foreach my $p ($star_set->each()){
      $p->rotate(45,0,0);
      $patch->add($p);
  }

  $self->expand_patch($patch); # problem is somewhere here!
  return $self;
}

##################
# make_squarex_2 #
##################
sub make_squarex_2 {
  my $self = shift @_;

  $self->description('Squarex 2 tiling patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>200,  'y1'=>100,'alternate1'=>'true',
			'x2'=>0,'y2'=>200,    'alternate2'=>'false');

  # Star set
  my $star_set = PolygonSet::new_star(4,90,4,4,'1:-1:1','alternate');

  foreach my $p ($star_set->each()){
      $p->rotate(45,0,0);
      $patch->add($p);
  }

  $self->expand_patch($patch);
  return $self;
}


##################
# make_squarex_3 #
##################
sub make_squarex_3 {
  my $self = shift @_;

  $self->description('Squarex 3 tiling patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>300,  'y1'=>100,'alternate1'=>'false',
			'x2'=>100,'y2'=>200,  'alternate2'=>'true');

  # Star set
  my $star_set = PolygonSet::new_star(4,90,4,4,'1:1:1');

  foreach my $p ($star_set->each()){
      $p->rotate(45,0,0);
      $patch->add($p);
  }

  my $square = Adult::new_ngon('4','3b');
  $square->translate(150,50);
  $patch->add($square);

  $self->expand_patch($patch); # problem is somewhere here!
  return $self;
}


##################
# make_squarex_4 #
##################
sub make_squarex_4 {
  my $self = shift @_;

  warn "THe colouring for this geometry is wrong.";

  $self->description('Squarex 4 tiling patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>200,  'y1'=>200,'alternate1'=>'false',
			'x2'=>400,'y2'=>0,  'alternate2'=>'false');

  # Star set
  my $star_set = PolygonSet::new_star(4,90,4,4,'1:-1:1','alternate');

  foreach my $p ($star_set->each()){
    $p->reverse();
    $p->rotate(45,0,0);
    $patch->add($p);
  }

  my $square1 = Adult::new_ngon('4','1');
  $square1->translate(150,50);
  my $square2 = Adult::new_ngon('4','2');
  $square2->translate(250,50);
  my $square3 = Adult::new_ngon('4','2');
  $square3->translate(150,-50);
  my $square4 = Adult::new_ngon('4','1');
  $square4->translate(250,-50);

  $patch->add($square1,$square2,$square3,$square4);

  $self->expand_patch($patch); # problem is somewhere here!
  return $self;
}



##################
# make_butterfly #
##################
# Tessellation from
# http://library.thinkquest.org/16661/of.non-regular.polygons/pentagons.hands-on.2.m.html
# Suggested by Melisande

sub make_butterfly {
    my $self = shift @_;

    $self->description('Butterfly patch');
    print STDERR "Using ",$self->description()," geometry\n";


    # Guessed!
    my $patch = new Patch('x1'=>-50,  'y1'=>121,'alternate1'=>'false',
			  'x2'=>221,'y2'=>-50,  'alternate2'=>'false');


    my @triangle=("-50 0 0 50 1",
		  "0 50 50 0 1",
		  "50 0 -50 0 1");
    
    my $triangle1 = Adult::new_from_old_data('1',@triangle);
    my $triangle2=$triangle1->copy();

    $triangle2->rotate(180);    
    
    $triangle1->translate(0,35.36);
    $triangle2->translate(0,-35.36);

    my $triangle3=$triangle1->copy();
    my $triangle4=$triangle2->copy();

    my @rectangle1=("-50 -35.36 -50 35.36 1",
		    "-50 35.36 50 35.36 1",
		    "50 35.36 50 -35.36 1",
		    "50 -35.36 -50 -35.36 1",
		    );

    my $rectangle1 = Adult::new_from_old_data('2',@rectangle1);

    my $rectangle2 = $rectangle1->copy();
    $rectangle2->rotate(45);
    $rectangle2->translate(110.5,-25);

    $triangle3->rotate(45,0,0);
    $triangle4->rotate(45,0,0);

    $triangle3->translate(110.5,-25);
    $triangle4->translate(110.5,-25);

    $patch->add($rectangle1,$triangle1,$triangle2,$rectangle2,$triangle3,$triangle4);

    $self->expand_patch($patch);

    $self->reassign_creases(10000,10000);
    return $self;
}


###############
# make_squave #
###############

sub make_squave {
    my $self = shift @_;

    $self->description('Squave patch');
    print STDERR "Using ",$self->description()," geometry\n";


    my $patch = new Patch('x1'=>170.7,'y1'=>0,'alternate1'=>'false',
			  'x2'=>0,    'y2'=>170.7,  'alternate2'=>'false');


    my $square1 = Adult::new_ngon('4','3b');
    $square1->translate(0,85.35);
    my $square2 = Adult::new_ngon('4','3a');
    $square2->scale(0.7071);
    $square2->translate(85.35,0);



    my $decomposed_set1 = new PolygonSet(0,1,4,4,'colour1');
    my $decomposed_set2 = new PolygonSet(0,1,4,4,'colour1');
    $decomposed_set1->decompose($square1);
    $decomposed_set2->decompose($square2);


    my @rectangle1=("-50 -35.36 -50 35.36 1",
		    "-50 35.36 50 35.36 1",
		    "50 35.36 50 -35.36 1",
		    "50 -35.36 -50 -35.36 1",
		    );

    my $rectangle1 = Adult::new_from_old_data('2',@rectangle1);

    my $rectangle2 = $rectangle1->copy();
    $rectangle2->colour(1);
    $rectangle2->rotate(90);
    $rectangle2->translate(85.36,85.36);

    my @set=$decomposed_set1->each();

    $patch->add($rectangle1,$set[1],$set[2],$set[3],$set[0],$rectangle2,$decomposed_set2->each());

    $self->expand_patch($patch);
    return $self;

}

#################
# make_windmill #
#################
sub make_windmill {
  my $self = shift @_;

  $self->description('Windmill patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>141.4,  'y1'=>282.8,'alternate1'=>'false',
			'x2'=>282.8,'y2'=>-141.4,    'alternate2'=>'false');

  my @triangle=("-70.71 0 0 70.71 1",
		"0 70.71 70.71 0 1",
		"70.71 0 -70.71 0 1");

  my $triangle1 = Adult::new_from_old_data('1',@triangle);
  $triangle1->translate(70.71,70.71);
  $triangle1->rotate(90);

  my @octagon=("-141.42 70.71 -70.71 141.42 -1",
	       "-70.71 141.42 70.71 141.42 -1",
	       "70.71 141.42 141.42 70.71 -1",
	       "141.42 70.71 141.42 -70.71 -1",
	       "141.42 -70.71 70.71 -141.42 -1",
	       "70.71 -141.42 -70.71 -141.42 -1",
	       "-70.71 -141.42 -141.42 -70.71 -1",
	       "-141.42 -70.71 -141.42 70.71 -1");

  my $octagon=Adult::new_from_old_data('2',@octagon);
  $octagon->translate(212.13,70.71);

  my $triangle2=$triangle1->copy();
  $triangle2->rotate(90,0,0);
  my $triangle3=$triangle1->copy();
  $triangle3->rotate(180,0,0);
  my $triangle4=$triangle1->copy();
  $triangle4->rotate(270,0,0);

  my $square = Adult::new_ngon('4','2');
  $square->rotate(45);
  $patch->add($square,$triangle1,$triangle2,$triangle3,$triangle4,$octagon);

  $self->expand_patch($patch);
  return $self;
}

###################
# make_kites_test #
###################
sub make_kites_test {
  my $self = shift @_;

  $self->description('Kites patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>0,  'y1'=>115.5,'alternate1'=>'false',
			'x2'=>100,'y2'=>0,    'alternate2'=>'false');

  my @kite=("0 -57.74 50 28.87 1",
	    "50 28.87 0 57.74 1",
	    "0 57.74 -50 28.87 1",
	    "-50 28.87 0 -57.74 1");

  my $colour=1;
  my $polygon1 = Adult::new_from_old_data($colour,@kite);
  my $polygon2 = $polygon1->copy();
  $polygon2->translate(50,-28.87);
  $polygon2->reverse();
  $polygon2->rotate(180);

  $patch->add($polygon1,$polygon2);

  $self->expand_patch($patch);
  return $self;
}
################
# make_3_12_12 #
################
sub make_3_12_12 {
  my $self = shift @_;

  $self->description('3.12.12 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>0,    'y1'=>646.4,'alternate1'=>'true',
			'x2'=>373.2,'y2'=>0,    'alternate2'=>'true');

  my $dodecagon1 = Adult::new_ngon('12','1');
  my $dodecagon2 = Adult::new_ngon('12','2');
  $dodecagon2->translate(186.6,323.2);
  my $triangle1  = Adult::new_ngon('3','3b'); # left top CORRECT 
  $triangle1->rotate(60);
  $triangle1->translate(0,215.5);
  my $triangle2  = Adult::new_ngon('3','3a'); # top CORRECT
  $triangle2->rotate(-120); # To get crease pattern correct!
  $triangle2->translate(186.6,107.7);
  my $triangle3  = Adult::new_ngon('3','3a'); # bottom left
  $triangle3->translate(0,430.9);
  my $triangle4  = Adult::new_ngon('3','3a'); # bottom CORRECT!
  $triangle4->rotate(60);
  $triangle4->rotate(240);
  $triangle4->translate(186.6,538.7);

  $patch->add($dodecagon1,$triangle1,$triangle2,$triangle3,$triangle4,$dodecagon2);

  $self->expand_patch($patch);

  $self->reassign_creases(0,0);

  return $self;
}

##########################
# make_3_4_3_12__3_12_12 #
##########################
sub make_3_4_3_12__3_12_12 {
  my $self = shift @_;

  $self->description('3.4.3.12; 3.12.12 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>0,    'y1'=>373.2,'alternate1'=>'true',
			'x2'=>373.2,'y2'=>0,    'alternate2'=>'true');

  my $dodecagon = Adult::new_ngon('12','3a');
  $dodecagon->translate(186.6,186.6);

  my $square    = Adult::new_ngon('4','1');
  my $triangle1 = Adult::new_ngon('3','2');
  $triangle1->translate(0,78.9);
  $triangle1->rotate(60);

  my $triangle2=$triangle1->copy();
  $triangle2->rotate(90,0,0);
  my $triangle3=$triangle1->copy();
  $triangle3->rotate(180,0,0);
  my $triangle4=$triangle1->copy();
  $triangle4->rotate(270,0,0);

  $patch->add($dodecagon,$triangle1,$square,$triangle2,$triangle3,$triangle4);

  $self->expand_patch($patch);
  return $self;
}
##################
# make_3_3_3_4_4 #
##################
sub make_3_3_3_4_4 {
  my $self = shift @_;

  $self->description('3.3.3.4.4 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>100,'y1'=>0,   'alternate1'=>'false',
			'x2'=>50,'y2'=>186.6,'alternate2'=>'true');

  my $polygon1 = Adult::new_ngon('4','3a');
  my $polygon2 = Adult::new_ngon('3','2');
  $polygon2->rotate(60);
  $polygon2->translate(0,78.9);
  my $polygon3 = Adult::new_ngon('3','1');
  $polygon3->translate(50,107.7);

  $patch->add($polygon1,$polygon2,$polygon3);

  $self->expand_patch($patch);

  $self->reassign_creases(0,10000);

  return $self;
}
##############################
# make_decomposed_4_8_8_test #
##############################
sub make_decomposed_4_8_8_test {
  my $self = shift @_;

  $self->description('Decomposed 4.8d.8d patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>0,'y1'=>241.4,'alternate1'=>'true',
			'x2'=>241.4,'y2'=>0,'alternate2'=>'true');

  my $octagon = Adult::new_ngon('8','3a');
  my $square  = Adult::new_ngon('4','3b');
  $square->translate(120.7,120.7);
  $square->rotate('45');

  my $decomposed_set = new PolygonSet(0,1,8,4,'colour1');
  $decomposed_set->decompose($octagon);
  my @octagon=$decomposed_set->each();
  $patch->add(@octagon,$square);
      
  $self->expand_patch($patch);
  return $self;

}


#################################
# make_decomposed_4_8_8__2_test #
#################################
sub make_decomposed_4_8_8__2_test {
  my $self = shift @_;

  $self->description('Decomposed 4d.8d.8d');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>0,'y1'=>241.4,'alternate1'=>'true',
			'x2'=>241.4,'y2'=>0,'alternate2'=>'true');

  my $octagon = Adult::new_ngon('8','3a');
  my $square  = Adult::new_ngon('4','3b');
  $square->translate(120.7,120.7);
  $square->rotate('45');

  my $decomposed_set1 = new PolygonSet(0,1,8,4,'colour1');
  $decomposed_set1->decompose($octagon);
  my @octagon=$decomposed_set1->each();
  my $decomposed_set2 = new PolygonSet(0,1,8,4,'colour1');
  $decomposed_set2->decompose($square);
  my @square=$decomposed_set2->each();
  $patch->add(@square,@octagon);
      
  $self->expand_patch($patch);
  return $self;
}


###############
# make_4_6_12 #
###############
sub make_4_6_12 {
  my $self = shift @_;

  $self->description('4.6.12 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>236.6,'y1'=>409.8,'alternate1'=>'false',
			'x2'=>473.2,'y2'=>0,  'alternate2'=>'false');


  my $dodecagon = Adult::new_ngon('12','1');
  my $square1   = Adult::new_ngon('4','2');
  $square1->translate(0,236.6);
  $square1->rotate(-30,0,0);

  my $hexagon1  = Adult::new_ngon('6','3b');
  $hexagon1->translate(0,273.2);

  my $square2=$square1->copy();
  $square2->rotate(60,0,0);
  my $square3=$square1->copy();
  $square3->rotate(120,0,0);

  my $hexagon2=$hexagon1->copy();
  $hexagon2->rotate(60,0,0);

  $patch->add($dodecagon,$square1,$hexagon1,$square2,$square3,$hexagon2);

  $self->expand_patch($patch);
  return $self;
}


##############################
# make_3_3_3_3_3_3__3_3_4_12 #
##############################
sub make_3_3_3_3_3_3__3_3_4_12 {
  my $self = shift @_;

  $self->description('3.3.3.3.3.3;3.3.4.12 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>236.6,'y1'=>409.8,'alternate1'=>'false',
			'x2'=>473.2,'y2'=>0,  'alternate2'=>'false');


  my $dodecagon = Adult::new_ngon('12','1');
  my $square1   = Adult::new_ngon('4','2');
  $square1->translate(0,236.6);
  $square1->rotate(-30,0,0);

  my $hexagon1  = Adult::new_ngon('6','3b');

  my $decomposed_set = new Patch();
  $decomposed_set->decompose($hexagon1);
  $decomposed_set->translate(0,273.2);

  my $square2=$square1->copy();
  $square2->rotate(60,0,0);
  my $square3=$square1->copy();
  $square3->rotate(120,0,0);

  my $decomposed_set2=$decomposed_set->copy();
  $decomposed_set2->rotate(60,0,0);

  $patch->add($dodecagon,$square1,$decomposed_set->each(),$square2,$square3,$decomposed_set2->each());

  $self->expand_patch($patch);
  return $self;
}


################
# make_3_4_6_4 #
################

sub make_3_4_6_4 {
  my $self = shift @_;

  $self->description('3.4.6.4 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>0,'y1'=>273.2,  'alternate1'=>'false',
			'x2'=>236.6,'y2'=>136.6,'alternate2'=>'false');

  my $polygon1  = Adult::new_ngon('6','1');
  my $polygon2  = Adult::new_ngon('4','2');
  $polygon2->translate(0,136.6);
  my $polygon3  = $polygon2->copy();
  $polygon3->rotate(60,0,0);
  my $polygon4  = $polygon2->copy();
  $polygon4->rotate(120,0,0);
  my $polygon5  = Adult::new_ngon('3','1');
  $polygon5->translate(0,157.74);
  $polygon5->rotate(30,0,0);
  my $polygon6=$polygon5->copy();
  $polygon6->rotate(60,0,0);

  $patch->add($polygon1,$polygon2,$polygon3,$polygon4,$polygon5,$polygon6);

#  $self->expand_patch($patch);
  $self->expand_patch_special($patch);

  return $self;
}


################
# make_3_6_3_6 #
################

sub make_3_6_3_6 {
  my $self = shift @_;

  $self->description('3.6.3.6 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>100,'y1'=>173.2,'alternate1'=>'false',
			'x2'=>200,'y2'=>0,    'alternate2'=>'false');

  my $polygon1 = Adult::new_ngon('6','1');
  my $polygon2 = Adult::new_ngon('3','2');
  $polygon2->translate(100,57.7);
  my $polygon3 = Adult::new_ngon('3','2');
  $polygon3->translate(100,-57.7);
  $polygon3->rotate(60);

  $patch->add($polygon1,$polygon2,$polygon3);

  $self->expand_patch($patch);
  return $self;
}

#########################
# make_3_6_3_6__3_3_6_6 #
#########################

sub make_3_6_3_6__3_3_6_6 {
  my $self = shift @_;

  $self->description('3.6.3.6;3.3.6.6 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>0,'y1'=>173.2,'alternate1'=>'true',
			'x2'=>200,'y2'=>0,  'alternate2'=>'false');

  my $polygon1 = Adult::new_ngon('6','1');
  my $polygon2 = Adult::new_ngon('3','2');
  $polygon2->translate(100,57.7);
  my $polygon3 = Adult::new_ngon('3','2');
  $polygon3->translate(100,-57.7);
  $polygon3->rotate(60);

  $patch->add($polygon1,$polygon2,$polygon3);

  $self->expand_patch($patch);
  return $self;
}

#############################
# make_3_3_3_3_3_3__3_3_6_6 #
#############################

sub make_3_3_3_3_3_3__3_3_6_6 {
  my $self = shift @_;

  $self->description('3.3.3.3.3.3;3.3.6.6 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>150,'y1'=>259.8,'alternate1'=>'false',
			'x2'=>300,'y2'=>0,  'alternate2'=>'false');

  my $polygon1 = Adult::new_ngon('6','1');

#  $patch->make_species(3,3,3,3,3,3);

  my $t1 = Adult::new_ngon('3','2');
  $t1->translate(0,57.6);
  my $t2 = $t1->copy();
  $t2->rotate(60,0,0);
  my $t3 = $t1->copy();
  $t3->rotate(120,0,0);
  my $t4 = $t1->copy();
  $t4->rotate(180,0,0);
  my $t5 = $t1->copy();
  $t5->rotate(240,0,0);
  my $t6 = $t1->copy();
  $t6->rotate(300,0,0);
  $t2->reverse();
  $t4->reverse();
  $t6->reverse();


  $patch->add($t1,$t2,$t3,$t4,$t5,$t6);

  my $p1 = Adult::new_ngon('6','1');
  $p1->translate(0,173.2);
  my $p2 = $p1->copy();
  $p2->rotate(60,0,0);
  $p2->reverse();

  $patch->add($p1,$p2);
  $self->expand_patch_special($patch);
  return $self;
}

##############
# make_4_8_8 #
##############

sub make_4_8_8 {
  my $self = shift @_;

  $self->description('4.8.8 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>0,    'y1'=>241.4,'alternate1'=>'true',
			'x2'=>241.4,'y2'=>0,    'alternate2'=>'true');

  my $polygon1 = Adult::new_ngon('8','3a');
  my $polygon2 = Adult::new_ngon('4','3b');
  $polygon2->rotate(45);
  $polygon2->translate(120.7,120.7);
  $patch->add($polygon1,$polygon2);

  $self->expand_patch($patch);

  return $self;
}

####################
# make_3_3_3_3_3_3 #
####################

sub make_3_3_3_3_3_3 {
  my $self = shift @_;

  $self->description('3.3.3.3.3.3 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>50, 'y1'=>86.6,'alternate1'=>'false',
			'x2'=>100,'y2'=>0,   'alternate2'=>'false');

  my $polygon1 = Adult::new_ngon('3','1');
  my $polygon2 = Adult::new_ngon('3','2');
  $polygon2->rotate(60);
  $polygon2->translate(50,-29);
  $patch->add($polygon1,$polygon2);

  $self->expand_patch($patch);
  return $self;
}

##############
# make_6_6_6 #
##############

sub make_6_6_6 {
  my $self = shift @_;

  $self->description('6.6.6 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>0,  'y1'=>173.2,'alternate1'=>'false',
			'x2'=>300,'y2'=>0, 'alternate2'=>'false');

# A bit ugly but 
  $patch->{'x1'}=0;
  $patch->{'y1'}=173.2;
  $patch->{'alternate1'}='false';	
  $patch->{'x2'}=300;
  $patch->{'y2'}=0;
  $patch->{'alternate2'}='false';

  my $polygon1 = Adult::new_ngon('6','3b');
  my $polygon2 = Adult::new_ngon('6','3b');
  $polygon2->translate(150,86.6);
  $patch->add($polygon1,$polygon2);

  $self->expand_patch($patch);
  return $self;
}

######################
# make_6_6_6_crystal #
######################

sub make_6_6_6_crystal {
  my $self = shift @_;

  $self->description('6.6.6 crystal patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $polygon = Adult::new_ngon('6','3a');
  my  $patch = crystal Patch($polygon,0.3);

  # A bit ugly but crystal method requires this as its currently coded :(
  $patch->{'x1'}=0;
  $patch->{'y1'}=173.2;
  $patch->{'alternate1'}='false';	
  $patch->{'x2'}=150;
  $patch->{'y2'}=86.6;
  $patch->{'alternate2'}='false';
  $self->expand_patch($patch);
  return $self;
}

########################
# make_3_6_3_6_crystal #
########################

sub make_3_6_3_6_crystal {
  my $self = shift @_;

  $self->description('3.6.3.6 crystal patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $polygon = Adult::new_ngon('6','1');
  my  $patch = crystal Patch($polygon,0.3);

  # A bit ugly but crystal method requires this as its currently coded :(
  $patch->{'x1'}=100;
  $patch->{'y1'}=173.2;
  $patch->{'alternate1'}='false';	
  $patch->{'x2'}=200;
  $patch->{'y2'}=0;
  $patch->{'alternate2'}='false';

  my $polygon2 = Adult::new_ngon('3','2');
  $polygon2->translate(100,57.7);
  my $polygon3 = Adult::new_ngon('3','2');
  $polygon3->translate(100,-57.7);
  $polygon3->rotate(60);

  $patch->add($polygon2,$polygon3);

  $self->expand_patch($patch);
  return $self;
}



################
# make_4_4_4_4 #
################

sub make_4_4_4_4 {
  my $self = shift @_;

  $self->description('4.4.4.4 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>0,  'y1'=>200,'alternate1'=>'false',
			'x2'=>100,'y2'=>0,  'alternate2'=>'true');

  my $polygon1 = Adult::new_ngon('4','1');
  my $polygon2 = Adult::new_ngon('4','2');
  $polygon2->translate(0,100);
  $patch->add($polygon1,$polygon2);

  $self->expand_patch($patch);

  # This will allow this tiling to be deformed!
  return $self;
}


##################
# make_3_3_4_3_4 #
##################
sub make_3_3_4_3_4 {
  my $self = shift @_;

  $self->description('3.3.4.3.4 patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>-50,  'y1'=>186.6,'alternate1'=>'false',
			'x2'=>186.6,'y2'=>50,  'alternate2'=>'false');

  $patch->make_species(3,3,4,3,4);

  my ($p1,$p2,$p3,$p4,$p5)=$patch->each();

  $p1->colour('2');
  $p2->colour('3a');
  $p3->colour('1');
  $p4->colour('2');
  $p5->colour('1');

  my $p = Adult::new_ngon('3','2');
  $p->translate(128.9,-50);
  $p->rotate(90);
  $patch->add($p);

  $self->expand_patch($patch);

  $self->reassign_creases(0,0);
  return $self;
}

########################
# make_9_9_6_4pi9_test #
########################
sub make_9_9_6_4pi9_test {
    my $self = shift @_;

    $self->description('9.9.6*4pi/9 patch');
    print STDERR "Using ",$self->description()," geometry\n";

    my $patch = new Patch('x1'=>0,'y1'=>475.8,'alternate1'=>'false',
			  'x2'=>412.2,'y2'=>238.0,'alternate2'=>'false');

#412.2,238.0    
    # Star sets
    my $star_set1 = PolygonSet::new_star(6,80,9,9,'1:-1:-1','alternate');
    my @star_set=$star_set1->each();

    # nonagon sets
    my $nonagon1  = Adult::new_ngon('9','2');
    $nonagon1->rotate(-10);
    $nonagon1->translate(274.8,0);

    my $nonagon2  = Adult::new_ngon('9','1');
    $nonagon2->rotate(10);
    $nonagon2->translate(137.4,238.0);

    $patch->add(@star_set,$nonagon1,$nonagon2);
    $self->expand_patch($patch);
    return $self;
}
###########################
# make_decomposed_4_4_4_4_test #
###########################
sub make_decomposed_4_4_4_4_test {
  my $self = shift @_;

  $self->description('4.4.4.4 Decomposed patch');
  print STDERR "Using ",$self->description()," geometry\n";

  my $patch = new Patch('x1'=>0,'y1'=>100,'alternate1'=>'true',
			'x2'=>100,'y2'=>0,'alternate2'=>'true');

  my $set = new PolygonSet(0,1,4,4,'colour1');
  my $polygon = Adult::new_ngon('4','3a');
  my $decomposed_set = new PolygonSet(0,1,4,4,'colour1');
  $decomposed_set->decompose($polygon);

  my @set=$decomposed_set->each();
  $patch->add(@set);

  $self->expand_patch($patch);
  return $self;
}


######################################
# make_decomposed_9_9_6_4pi9__1_test #
######################################
sub make_decomposed_9_9_6_4pi9__1_test {
    my $self = shift @_;

    $self->description('9.9.6*4pi/9 partial decomposed patch');
    print STDERR "Using ",$self->description()," geometry\n";

    my $patch = new Patch('x1'=>0,'y1'=>475.8,'alternate1'=>'false',
			  'x2'=>412.2,'y2'=>238.0,'alternate2'=>'false');

#412.2,238.0    
    # Star sets
    my $star_set1 = PolygonSet::new_star(6,80,9,9,'1:-1:-1','alternate');
    my @star_set=$star_set1->each();

    # nonagon sets
    my $nonagon1  = Adult::new_ngon('9','2');
    $nonagon1->rotate(-10);
    $nonagon1->translate(274.8,0);

    my $decomposed_set = new PolygonSet(0,1,9,9,'colour1');
    $decomposed_set->decompose($nonagon1);
    my @nonagon=$decomposed_set->each();

    my $nonagon2  = Adult::new_ngon('9','1');
    $nonagon2->rotate(10);
    $nonagon2->translate(137.4,238.0);


    $patch->add(@star_set,@nonagon,$nonagon2);
    $self->expand_patch($patch);
    return $self;
}


##############################
# make_3_4_4_6__3_6_3_6_test #
##############################
sub make_3_4_4_6__3_6_3_6__1_test {
    my $self = shift @_;

    $self->description('3.4.4.6;3.6.3.6 version 1 patch');
    print STDERR "Using ",$self->description()," geometry\n";

    my $patch = new Patch('x1'=>0,'y1'=>273.2,'alternate1'=>'false',
			  'x2'=>200,'y2'=>0,'alternate2'=>'false');

    my $hexagon  =Adult::new_ngon('6','1');
    my $square1  =Adult::new_ngon('4','2');
    $square1->translate(0,136.6);
    my $square2  =Adult::new_ngon('4','1');
    $square2->translate(100,136.6);
    my $triangle1=Adult::new_ngon('3','2');
    $triangle1->rotate(60);
    $triangle1->translate(100,-57.7);
    my $triangle2=Adult::new_ngon('3','2');
    $triangle2->rotate(0);
    $triangle2->translate(100,57.7);

#    $patch->add($hexagon,$square1,$square2,$triangle1,$triangle2);
    $patch->add($hexagon,$triangle1,$triangle2,$square1,$square2);
    $self->expand_patch($patch);
    return $self;
}

#################################
# make_3_4_4_6__3_6_3_6__2_test #
#################################
sub make_3_4_4_6__3_6_3_6__2_test {
    my $self = shift @_;

    $self->description('3.4.4.6;3.6.3.6 version 2 patch');
    print STDERR "Using ",$self->description()," geometry\n";

    my $patch = new Patch('x1'=>100,'y1'=>273.2,'alternate1'=>'true',
			  'x2'=>200,'y2'=>0,'alternate2'=>'false');

    my $hexagon  =Adult::new_ngon('6','1');
    my $square1  =Adult::new_ngon('4','2');
    $square1->translate(0,136.6);
    my $square2  =Adult::new_ngon('4','1');
    $square2->translate(100,136.6);
    my $triangle1=Adult::new_ngon('3','2');
    $triangle1->rotate(60);
    $triangle1->translate(100,-57.7);
    my $triangle2=Adult::new_ngon('3','2');
    $triangle2->rotate(0);
    $triangle2->translate(100,57.7);

    $patch->add($hexagon,$square1,$square2,$triangle1,$triangle2);
    $self->expand_patch($patch);
    return $self;
}


######################################
# make_decomposed_9_9_6_4pi9__2_test #
######################################
sub make_decomposed_9_9_6_4pi9__2_test {
    my $self = shift @_;

    $self->description('9.9.6*4pi/9 partial decomposed patch');
    print STDERR "Using ",$self->description()," geometry\n";

    my $patch = new Patch('x1'=>0,'y1'=>475.8,'alternate1'=>'false',
			  'x2'=>412.2,'y2'=>238.0,'alternate2'=>'false');

#412.2,238.0    
    # Star sets
    my $star_set1 = PolygonSet::new_star(6,80,9,9,'1:-1:-1','alternate');
    my @star_set=$star_set1->each();

    # nonagon sets
    my $nonagon1  = Adult::new_ngon('9','2');
    $nonagon1->rotate(-10);
    $nonagon1->translate(274.8,0);

    my $decomposed_set = new PolygonSet(0,1,9,9,'colour1');
    $decomposed_set->decompose($nonagon1);
    my @nonagon1=$decomposed_set->each();

    my $nonagon2  = Adult::new_ngon('9','1');
    $nonagon2->rotate(10);
    $nonagon2->translate(137.4,238.0);

    my $decomposed_set2 = new PolygonSet(0,1,9,9,'colour1');
    $decomposed_set->decompose($nonagon2);
    my @nonagon2=$decomposed_set->each();


    $patch->add(@star_set,@nonagon1,@nonagon2);
    $self->expand_patch($patch);
    return $self;
}


         #########################
         # ROTATIONAL GEOMETRIES #
         #########################


###########################################################
# Internal method to calculate lvalue for make_log_spiral #
###########################################################

sub _calc_psi {
  my ($psi,$rho,$m)= @_;

  if (! $m){
    die "No axial param passed\n";
  }

  my $value=(2*$rho**$m)*(-1+$rho**2)*sin($psi) -2*$rho*(-1+$rho**(2*$m))*sin($m*$psi) -$rho**2*sin((1+$m)*$psi) +$rho**(2*$m)*sin((1+$m)*$psi) + sin($psi-($m*$psi)) -$rho**(2+2*$m)*sin($psi-($m*$psi));
  return $value;
}

###################
# make_log_spiral #
###################

sub make_log_spiral {
    my $self = shift @_;
    my %args=@_;

    my $rho = $args{'spiral_rho'};
    my $m   = $args{'spiral_m'};

    if (! $rho){$rho=0.9;}
    if (! $m){$m=10;}

    if (! $m or ! $rho){
	die "No m or rho in make_log_spiral\n";
    }
    $self->description("Log spiral");
    print STDERR "Using ",$self->description()," geometry\n";

    my $scale=300;
    my $pi=3.1415927;

    ###########################################################
    # Calculate a value of $psi that will work with m and rho #
    ###########################################################
    my $i;
    my ($new_psi,$value);
    my $step=0.1;
    my $guess=2*$pi/$m;

    my $psi;
    while ($step>0.0000001){
	$i++;
	#print STDERR "Round $i Guess is $guess Scoring",&_calc_psi($guess,$rho,$m)," Step $step\n";
	
	my $upper=&_calc_psi($guess+$step,$rho,$m);
	my $middle=&_calc_psi($guess,$rho,$m);
	my $lower=&_calc_psi($guess-$step,$rho,$m);
	
	if ($middle**2>$upper**2){
	    # New guess was better
	    #      print "New upper guess was better\n";
	    $guess=$guess+$step;
	    next;
	} elsif ($middle**2>$lower**2){
	    # New guess was better
	    #      print "New lower guess was better\n";
	    $guess=$guess-$step;
	    next;
	} else {
	    #      print "Last guess was best yet, lower step parameter\n";
	    # Last guess was best lower step
	    $step=$step/2;
	}
    }
    $psi=$guess;
    
    # Use polar coordinates for ease for first polygon
    my ($r,$theta);
    
    my $set = new Patch();   
    $self->add($set);
    my $primarypatch = new PrimaryPatch();
    $self->patch($primarypatch);

    
    #############################
    # Calculate centre for tile #
    #############################
    my $x = $scale*0.25*sqrt(((1+$rho**2+2*$rho*cos($psi))*(1+$rho**(2*$m)+2*($rho**$m)*cos($m*$psi)))/((1+$rho**2-2*$rho*cos($psi))*(1+$rho**(2*$m)-2*($rho**$m)*cos($m*$psi))))*(($rho**$m)*sin(0)-$rho**(1+$m)*sin($psi)-sin($m*$psi)+$rho*sin($psi+$psi*$m));
    my $y = $scale*0.25*sqrt(((1+$rho**2+2*$rho*cos($psi))*(1+$rho**(2*$m)+2*($rho**$m)*cos($m*$psi)))/((1+$rho**2-2*$rho*cos($psi))*(1+$rho**(2*$m)-2*($rho**$m)*cos($m*$psi))))*(-($rho**$m)*cos(0)+$rho**(1+$m)*cos($psi)+cos($m*$psi)-$rho*cos($psi+$psi*$m));
    
    #print STDERR "Centre for first tiling is $x,$y\n";
    my $polygon = new Adult($x,$y,'1');
    
    # Point1
    $r=$scale;
    $theta=0;
    my $point1 = new Point($r*cos($theta),$r*sin($theta),$polygon);
    
    # Point2
    $r=$scale*$rho;
    $theta=$psi; # Angle in radians
    my $point2 = new Point($r*cos($theta),$r*sin($theta),$polygon);
    
    # Point3
    $r=$scale*$rho**$m;
    $theta=$psi*$m;
    my $point3 = new Point($r*cos($theta),$r*sin($theta),$polygon);
    
    # Point4
    $r=$scale*$rho**($m+1);
    $theta=$psi*($m+1); # Angle in radians
    my $point4 = new Point($r*cos($theta),$r*sin($theta),$polygon);
    
    my $line1= new Line($point1,$point2->copy(),'1');
    my $line2= new Line($point2,$point4->copy(),'1');
    my $line3= new Line($point4,$point3->copy(),'-1');
    my $line4= new Line($point3,$point1->copy(),'-1');
    
    $polygon->add($line1,$line2,$line3,$line4);
    #$polygon->write();
    
    $set->add($polygon);
    
    # Now make copy and rotations of tile
    $i=1;
    while ($rho**$i>0.005){
	my $new_polygon = $polygon->copy();
	$new_polygon->scale($rho**$i,0,0); # This will be a bit slow
	$new_polygon->rotate(($i*$psi)*180/$pi,0,0);
	$set->add($new_polygon);

	# Make primary patch for log spiral out of first two tiles.
	if ($i==2){
	    my @polygons=$set->each();
	    foreach my $p (@polygons){
		$primarypatch->add($p->copy());
	    }
	}	
	$i++;
    }
    #print STDERR "Made $i tiles\n";

    $self->centre();
    return $self;
}


###############
# make_spiral #
###############
sub make_spiral {
  my $self = shift @_;
  my %args = @_;
  
  $self->description('Voderbergs spiral');
  print STDERR "Using ",$self->description()," geometry\n";

  my $m   = $args{'spiral_m'};
  my $shift = $args{'spiral_shift'};
  my $depth = $args{'spiral_depth'};

  #######################
  # Make first triangle #
  #######################
  my $set1 = new Patch();
  my $triangle1 = new Adult(0,0,'1');
  my $point1 = new Point(0,0,$triangle1);
  my $point2 = new Point(100,0,$triangle1);
  my $point3 = $point2->copy();
  $point3->rotate(360/$m);

  my $line1=new Line($point1,$point2,'1');
  my $line2=new Line($point2,$point3,'1');
  my $line3=new Line($point3,$point1,'1');

  $triangle1->add($line1,$line2,$line3);
  $triangle1->rotate(0.05,0,0);
  $triangle1->find_centre2();
  $triangle1->rotate(-0.05,0,0);

  ########################
  # Make second triangle #
  ########################
  my $set2 = new Patch();
  my $triangle2 = new Adult(0,0,'2');
  my $point4=$point2->copy($triangle2);
  my $point5=$point3->copy($triangle2);
  my $point6=new Point(($point3->x()+100),$point3->y(),$triangle2);

  my $line4=new Line($point4,$point5,'-1');
  my $line5=new Line($point5,$point6,'-1');
  my $line6=new Line($point6,$point4,'-1');

  $triangle2->add($line4,$line5,$line6);
  $triangle2->find_centre2();

  # Create primary patch

    
  #my $patch=new Patch();


  #####################
  # Make translations #
  #####################
  my $x_translate=$point3->x();
  my $y_translate=$point3->y();

  my $i=0;
  while ($i<$depth){
    my $j=0;
    while ($j<$i){
      print "I $i J $j\n";
      # Copy original tile and translate j to x
      my $polygon1=$triangle1->copy();
      $polygon1->translate(($i-1)*100-$j*(100-$x_translate),$j*$y_translate);
      $set1->add($polygon1);
      my $polygon2=$triangle2->copy();
      $polygon2->translate(($i-1)*100-$j*(100-$x_translate),$j*$y_translate);
      $set2->add($polygon2);

      if ($i == 1 and $j==0){
	warn "Got here";
	my $primarypatch = new PrimaryPatch();
	$primarypatch->add($polygon1->copy(),$polygon2->copy());
	$self->patch($primarypatch);
      }

      $j++;
    }
    $i++;
  }

  #############################
  # Make rotation of segments #
  #############################
  $i=0;
  my $rev=1; # This value alternates between 1 and -1, when value is -1
             # reverse colours!
  while($i<$m/2){
    my $seta = new Patch();
    $seta=$set1->copy();
    $seta->rotate($i*360/$m,0,0);
    my $setb = new Patch();
    $setb=$set2->copy();
    $setb->rotate($i*360/$m,0,0);

    # Change colours of sets
    if ($rev == -1){
	$seta->reverse();
	$setb->reverse();
    }

    $self->add($seta,$setb);
    $i++;
    $rev=$rev*(-1);
  }

  $rev=-1;
  while($i<$m){
    my $seta = new Patch();
    $seta=$set1->copy();
    $seta->rotate($i*360/$m,0,0);
    my $setb = new Patch();
    $setb=$set2->copy();
    $setb->rotate($i*360/$m,0,0);

    # Add shift
    $seta->translate($shift*100,0);
    $setb->translate($shift*100,0);

    # Change colours of sets
    if ($rev == -1){
	$seta->reverse();
	$setb->reverse();
    }

    $self->add($seta,$setb);
    $i++;
    $rev=$rev*(-1);
  }
  return $self;
}

###############
# make_kepler #
###############

sub make_kepler {
    my $self = shift @_;

    $self->description('kepler patch');
    print STDERR "Using ",$self->description()," geometry\n";

    my $star_set = Patch::new_star(5,36,'1:1:1');
    
    my $primarypatch = new PrimaryPatch();
    foreach my $p ($star_set->each()){
	$primarypatch->add($p->copy());
    }
    $self->patch($primarypatch);
    
    my $patch1=new Patch();

    my $p1 = Adult::new_ngon('5','2');
    $p1->translate(80.9,111.35);
    $p1->rotate(36);
    my $p2=Adult::new_ngon('5','1');
    $p2->translate(0,222.7);
    my $p3=Adult::new_ngon('5','2');
    $p3->rotate(36);
    $p3->translate(0,360.34);

    my $p4=Adult::new_ngon('10','3b');
    $p4->translate(211.8,291.52);

    my $p5=Adult::new_ngon('5','1');
    $p5->translate(80.9,471.69);

    my $p6=Adult::new_ngon('5','1');
    $p6->translate(-80.9,471.69);

    my $p7=Adult::new_ngon('5','2');
    $p7->rotate(36);
    $p7->translate(211.8,514.22);

    my $p8=Adult::new_ngon('5','2');
    $p8->rotate(36);
    $p8->translate(423.6,360.44,);

    my $p9=Adult::new_ngon('5','1');
    $p9->translate(342.7,471.7);

    my $star_set2=$star_set->copy();
    $star_set2->rotate(36,0,0);
    $star_set2->translate(0,582.96);
    $star_set2->reverse();

    $patch1->add($p1,$p2,$p3,$p4,$p5,$p6,$p7,$p8,$p9,$star_set2->each());

    #$patch1->write();
    my $patch2=$patch1->copy();
    $patch2->rotate(72,0,0);

    my $patch3=$patch1->copy();
    $patch3->rotate(144,0,0);

    my $patch4=$patch1->copy();
    $patch4->rotate(216,0,0);

    my $patch5=$patch1->copy();
    $patch5->rotate(288,0,0);

    $self->add($star_set,$patch1,$patch2,$patch3,$patch4,$patch5);
    
    $self->scale(0.3,0,0);

    $self->centre();
    return $self;
}


#################
# make_rosette1 #
#################

sub make_rosette1 {
    my $self = shift @_;
    
    my %args=@_;
    
    my $n=$args{'rot_n'};
    if (! $n){
	$n=6;
    }

    $self->description("Rosette, type I $n-fold");
    print STDERR "Using ",$self->description()," geometry\n";
    
    # Find alpha
    my $pi = atan2(1,1)*4;
    my $alpha=($pi/2)-($pi/$n);
    # Convert to degrees
    $alpha=$alpha*180/$pi;
    
    my $star_set = Patch::new_star($n,$alpha,'1:1:1');

    # Make star set the Primary patch!    
    my $primarypatch = new PrimaryPatch();
    foreach my $polygon ($star_set->each()){
	$primarypatch->add($polygon->copy());
    }
    $self->patch($primarypatch);

    # Now make  new kite shape
    my @polygons=$star_set->each();
    my @line1=$polygons[$n-2]->eachline();
    my $kite1=$line1[2]->copy();
    
    my @line2=$polygons[$n-1]->eachline();
    my $kite2=$line2[0]->copy();

    my ($foopoint)=$kite2->points();
    # Now find where one of these lines crosses midline.
    my $point1=new Point(0,0);
    my $point2=$foopoint->copy();
    my $testline=new Line($point1,$point2,1);
    $testline->scale(100,0,0);
    
    ($point1,$point2)=$kite1->points();
    my $fooline=$kite1->copy();
    $fooline->rotate(-90,$point1->x(),$point1->y()); # This line should be orthogonal to
    # kite 1
    $fooline->scale(50,$point1->x(),$point1->y());
    
    my $p4=$testline->intersect($fooline);
    
    if (! $p4){
	warn "Oh no got no intersect in make_rosette1";
	$testline->write();
	$fooline->write();
    }
    
    my ($p1)=$kite1->points();
    my ($p2,$p3)=$kite2->points();
    
    my $kite3=new Line($p3->copy(),$p4->copy(),1);
    my $kite4=new Line($p4->copy(),$p1->copy(),1);
    
    # Calculate inflation factor r
    my $r=$kite3->length()/$kite1->length();
    
    # Find origin
    my $kite_polygon=new Adult(($p2->x()+$p4->x())/2,($p2->y()+$p4->y())/2,1);
    $kite_polygon->add($kite1,$kite2,$kite3,$kite4);
    
    $self->add($star_set);

    # Now make rings of kites
    my $depth=6;
    my $j=0;
    while($j<$depth){
	my $set = new Patch();
	my $i=0;
	while($i<$n){
	    my $p=$kite_polygon->copy();
	    $p->rotate(360*$i/$n,0,0);
	    
	    # If j is odd rotate by a bit extra and reverse creases
	    my $value=(-1)**$j;
	    if ($value<0){
		$p->rotate(180/$n,0,0);
	    } else {
		$p->reverse();
	    }
	    
	    $p->scale($r**$j,0,0);
	    
	    $set->add($p);
	    
	    $i++;
	}
	$self->add($set);
	$j++;
    }
    
    $self->centre();
    return $self;
}

#################
# make_rosette2 #
#################

sub make_rosette2 {
    my $self = shift @_;

    my %args=@_;
    
    my $n=$args{'rot_n'};
    if (! $n){
	$n=6;
    }

    $self->description("Rosette, type II $n-fold");
    print STDERR "Using ",$self->description()," geometry\n";


    # Find alpha
    my $pi = atan2(1,1)*4;
    my $alpha=($pi/2)-(2*$pi/$n);
    # Convert to degrees
    $alpha=$alpha*180/$pi;
    
    my $star_set = Patch::new_star($n,$alpha,'1:1:1');
    
    # Make star set the Primary patch!    
    my $primarypatch = new PrimaryPatch();
    foreach my $polygon ($star_set->each()){
	$primarypatch->add($polygon->copy());
    }
    $self->patch($primarypatch);
    


    #########################
    # Now make square shape #
    #########################
    
    my @polygons=$star_set->each();
    my @line1=$polygons[$n-2]->eachline();
    my $l1=$line1[2]->copy();
    $l1->reverse();
    
    my @line2=$polygons[$n-1]->eachline();
    my $l2=$line2[0]->copy();
    $l2->reverse();
    
    my ($foopoint)=$l2->points();
    
    # Now find where one of these lines crosses midline.
    my $point1=new Point(0,0);
    my $point2=$foopoint->copy();
    my $testline=new Line($point1,$point2,1);
    $testline->scale(100,0,0);
    
    ($point1,$point2)=$l1->points();
    my $fooline=$l1->copy();
    $fooline->rotate(-90,$point1->x(),$point1->y()); # This line should be orthogonal to
    # kite 1
    $fooline->scale(50,$point1->x(),$point1->y());
    
    my $p4=$testline->intersect($fooline);
    
    if (! $p4){
	warn "Oh no got no intersect in make_rosette1";
	$testline->write();
	$fooline->write();
    }
    
    my ($p1)=$l1->points();
    my ($p2,$p3)=$l2->points();

    my $l3=new Line($p3->copy(),$p4->copy(),-1);
    my $l4=new Line($p4->copy(),$p1->copy(),-1);
    
    # Find origin
    my $square_polygon=new Adult(($p2->x()+$p4->x())/2,($p2->y()+$p4->y())/2,2);
    $square_polygon->add($l1,$l2,$l3,$l4);
    
    # Make ring of squares
    my $square_set = new Patch();
    my $i=0;
    while($i<$n){
	my $p=$square_polygon->copy();
	$p->rotate(360*$i/$n,0,0);
	$square_set->add($p);
	$i++;
    }

    $self->add($star_set);

    #################
    # Now make kite #
    #################
    
    my @squares=$square_set->each();
    my @kite_line1=$squares[1]->eachline();
    my $kite1=$kite_line1[2]->copy();
    $kite1->reverse();

    my @kite_line2=$squares[2]->eachline();
    my $kite2=$kite_line2[3]->copy();
    $kite2->reverse();
    
    ($foopoint)=$kite1->points();
    
    # Now find where one of these lines crosses midline.
    $point1=new Point(0,0);
    $point2=$foopoint->copy();
    
    $testline=new Line($point1,$point2,1);
    $testline->scale(100,0,0);
    
    ($point1,$point2)=$kite1->points();
    
    $fooline=$kite1->copy();
    $fooline->rotate(-90,$point2->x(),$point2->y()); # This line should be orthogonal to
    # kite 1
    $fooline->scale(50,$point2->x(),$point2->y());
    
    $p4=$testline->intersect($fooline);
    
    ($p2,$p1)=$kite1->points();
    ($p3,$p2)=$kite2->points();
    
    my $kite3=new Line($p3->copy(),$p4->copy(),1);
    my $kite4=new Line($p4->copy(),$p1->copy(),1);
    
    # Find origin
    my $kite_polygon=new Adult(($p2->x()+$p4->x())/2,($p2->y()+$p4->y())/2,1);
    $kite1->swap();
    $kite2->swap();
    $kite_polygon->add($kite1,$kite2,$kite3,$kite4);
    
    my $kite_set = new Patch();
    $i=0;
    while($i<$n){
	my $p=$kite_polygon->copy();
	$p->rotate(360*$i/$n,0,0);
	$kite_set->add($p);
	$i++;
    }
    
    # Calculate inflation factor r
    my $r=$kite3->length()/$kite1->length();
    
    # Now make rings of kites
    my $depth=3;
    my $j=0;
    while($j<$depth){
	my $set1 = $square_set->copy();
	my $set2 = $kite_set->copy();
	
	my $scale=$r**$j;
	
	# Inflate sets
	$set1->scale($scale,0,0);
	$set2->scale($scale,0,0);
	$self->add($set1,$set2);
	$j++;
    }      
    $self->centre();
    return $self;
}

###################
# make_tower_test #
###################

sub make_tower_test {
    my $self = shift @_;
    my %args=@_;
    
    my $array=$args{'tower_array'};
    my $n    =$args{'rot_n'};
    
    if (! $n){$n=5}
    
    if (! $array){
	$array="0.5,0.5,0.5";
    }
    
    $self->description("$n-fold Tower");
    print STDERR "Using ",$self->description()," geometry\n";
    
    my $patch = new Patch();
    my @array  = split(/,/,($array));
    my $crystal_set = new Patch();
    my $polygon = Adult::new_ngon($n,'1');
    $crystal_set = crystal Patch($polygon,@array);
    
    foreach my $p ($crystal_set->each()){
	$patch->add($p);
    }

    # Now need to write expand_patch type functionality.
    my $primarypatch = new PrimaryPatch();
    foreach my $polygon ($patch->each()){
	$primarypatch->add($polygon->copy());
    }

    $self->add($patch);
    $self->patch($primarypatch);
    $self->centre();

    return $self;
}

##############
# make_twist #
##############

sub make_twist {
    my $self = shift @_;
    my %args=@_;
    my $n=$args{'rot_n'};
    
    if (! $n){
	die "In Tiling::make_twist. No n parameter given";
    }

    $self->description("Simple $n-gon twist");
    print STDERR "Using ",$self->description()," geometry\n";

    my $patch = new Patch();

    my $polygon;
    if ($args{'iso_area'}){
	warn "Using iso area";
	$polygon = Adult::new_ngon($n,'3a');
    } else {
	$polygon = Adult::new_ngon($n,'1');
    }
    my $decomposed_set = new PolygonSet(0,1,4,4,'colour1');
    $decomposed_set->decompose($polygon);

    foreach my $p ($decomposed_set->each()){
	$patch->add($p);
    }

    # Now need to write expand_patch type functionality.
    my $primarypatch = new PrimaryPatch();
    foreach my $polygon ($patch->each()){
	$primarypatch->add($polygon->copy());
    }

    $self->add($patch);
    $self->patch($primarypatch);
    $self->centre();

    return $self;
}


################
# expand_patch #
################
=head1 expand_patch

Method takes a patch and does translations

$tiling->expand_patch($patch);

=cut

sub expand_patch {
  my ($self,$patch) = @_;

  # Set up primary patch for tiling. This is used to calculate parameters
  # of the object.

  my $primarypatch = new PrimaryPatch('x1'=>$patch->{'x1'},'y1'=>$patch->{'y1'},'alternate1'=>$patch->{'alternate1'},
				      'x2'=>$patch->{'x2'},'y2'=>$patch->{'y2'},'alternate2'=>$patch->{'alternate2'});


  foreach my $polygon ($patch->each()){
      $primarypatch->add($polygon->copy());
  }

  $self->patch($primarypatch);

  my $i=0;
  my $i_max=$self->x_expand();
  my $j_max=$self->y_expand();

#$i_max=1;
#$j_max=1;

  my $phase=1;
  while($i<$i_max){
      my $j=0;
      while($j<$j_max){
	  my $sum=0;
	  if ($primarypatch->{'alternate1'}eq'true'){$sum=$sum+$i;}
	  if ($primarypatch->{'alternate2'}eq'true'){$sum=$sum+$j;}
	  $phase=(-1)**$sum;

	  my $new_patch=$primarypatch->copy();
	  if ($phase<0){$new_patch->reverse();}
	  $new_patch->translate($i*$primarypatch->{'x1'},$i*$primarypatch->{'y1'});
	  $new_patch->translate($j*$primarypatch->{'x2'},$j*$primarypatch->{'y2'});

	  $self->add($new_patch);###
	  $j++;
      }
      $i++;
  }
  $self->centre(); # Problem in here with squarex geometry

  return $self;
}


##########
# centre #
##########

=head1 centre

    Method Centre the tiling about origin. Takes centres of every tile
    and finds average.  Uses this to translate patch.

    $self->centre();

=cut
sub centre {
    my $self= shift @_;

    my $n=0;
    my $x_total=0;
    my $y_total=0;
    foreach my $patch ($self->each()){
	foreach my $polygon ($patch->each()){
	    my ($x,$y)=$polygon->centre()->xy();
	    if ($x eq "NaNQ"){
		$polygon->write();
		die "in Tiling::centre polygon centre is not defined for $polygon";
	    }

	    $x_total=$x_total+$x;
	    $y_total=$y_total+$y;
	    $n++;
	}
    }

    if ($x_total eq "NaNQ"){
	die "in Tiling::centre $x_total $y_total $n";
    }

    if ($n){
	$self->translate(-$x_total/$n,-$y_total/$n);
    } else {
	die "In Tiling::centre. Trying to centre without any tiles defined!";
    }
}



########################
# expand_patch_special #
########################
=head1 expand_patch_special

Method takes a patch and does translations
But will attempt to do all closed back creases. Will
only work with two colourable tilings.

$tiling->expand_patch_special($patch);

=cut

sub expand_patch_special {
  my ($self,$patch) = @_;

  # Set up primary patch for tiling. This is used to calculate parameters
  # of the object.

  my $primarypatch = new PrimaryPatch('x1'=>$patch->{'x1'},'y1'=>$patch->{'y1'},'alternate1'=>$patch->{'alternate1'},
				      'x2'=>$patch->{'x2'},'y2'=>$patch->{'y2'},'alternate2'=>$patch->{'alternate2'});


  foreach my $polygon ($patch->each()){
    my $copy=$polygon->copy();
    $primarypatch->add($copy);
  }

  $self->patch($primarypatch);

  my $i=0;
  my $i_max=$self->x_expand();
  my $j_max=$self->y_expand();

  my $phase=1;
  while($i<$i_max){
      my $j=0;
      while($j<$j_max){
	  my $sum=0;
	  if ($primarypatch->{'alternate1'}eq'true'){$sum=$sum+$i;}
	  if ($primarypatch->{'alternate2'}eq'true'){$sum=$sum+$j;}
	  $phase=(-1)**$sum;

	  my $new_patch=$primarypatch->copy();
	  if ($phase<0){$new_patch->reverse();}
	  $new_patch->translate($i*$primarypatch->{'x1'},$i*$primarypatch->{'y1'});
	  $new_patch->translate($j*$primarypatch->{'x2'},$j*$primarypatch->{'y2'});

	  $self->add($new_patch);###
	  $j++;
      }
      $i++;
  }
  $self->centre();

  # Vertexify
  $self->vertexify();

  my $theta_plus_psi=90;
  my $pi = atan2(1,1)*4;

  my $alpha_over_sin_psi=0.4; # Is this right?
  my $pleat_width=25;
  #my $pleat_width=17; # make_3_3_3_3_3_3__3_3_4_12

  foreach my $set ($self->each()){
    foreach my $p ($set->each()){
      my ($line1,$line2)=$p->eachline();
      if (! $line2){
	next;
      }
      my $angle=$line1->angle($line2);
      my $max_angle=(180-$angle)/2;
      warn "Angle for polygon is $angle max angle is $max_angle";
      
      # Find out max pleat angle and based on colour assign it.
      my $psi;
      if ($p->colour() == 1){
	$psi=$max_angle;
      } elsif($p->colour() == 2){
	$psi=180-$max_angle;
      } else {
	warn "Oh no not two colourable!";
	next;
      }

      # Calculate alpha
      #my $alpha=$alpha_over_sin_psi*sin ($psi*$pi/180);

      my $alpha=$pleat_width/(sin ($psi*$pi/180) * $line1->length());

      warn "alpha $alpha psi $psi";
      
      $p->scale($alpha);
      my $theta=$theta_plus_psi - $psi;

      warn "Theta $theta";
      $p->rotate($theta);
    }
  }
  return $self;
}




########### 
# stretch #
###########

=head1 stretch

Experimental method to try and stretch a tiling along the x
axis.  Argument gives ratio to scale x axis by. 2 would stretch
tiling. 0.5 would squeeze it.

    $tiling->stretch(0.5);

=cut

sub stretch {
    my ($self,$scale) = @_;
    
    foreach my $set ($self->each()){
	foreach my $polygon ($set->each()){
	    # Scale centre
	    if (defined $polygon->centre()){
		my $centre=$polygon->centre();
		my $y=$centre->y();
		$centre->y($scale*$y);
	    }

	    foreach my $line ($polygon->eachline()){
		# Scale each line
		my ($p1,$p2)=$line->points();
		my $p1x=$p1->x();
		my $p2x=$p2->x();
		$p1->x($scale*$p1x);
		$p2->x($scale*$p2x);
	    }
	}
    }
}

1;  # says use was ok
__END__
