=head1 NAME

View

=head1 DESCRIPTION

The View object is the class object that specific view types inherit from.
For example there is a View::canvas and View::postscript object.  These
objects read in an XML render stack from the model object and render
the data in the relevant format.

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for View
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package View;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Postscript;
use PDF;
use SVG;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

#######
# new #
#######

=head1 new

Create new view object.

    $view = new View($model,handle);

The handle is a filehandle for Postscript and a canvas object for Canvas

=cut

sub new {
    my ($class,$model,$handle)=@_;
    my $self = {
	"model"  => $model,
	"handle" => $handle,
	"show"   => '',        # Whether to render patch, tiling or creases
    };
    bless $self, $class;

    $self->light("0");        # Says whether light pattern is in render XML
    $self->fill("0");         # Says whether Adults and Babies should be colour filled
    $self->show_centres("0"); # Says whether centres of tiles are viewed

    return $self;
}

##########
# handle #
##########

=head1 handle

Returns output handle for View object

    $view->handle();

=cut

sub handle {
    my ($self) = @_;
    my $handle=$self->{'handle'};
    return $handle;
}

##########
# render #
##########

=head1 render

Prints model XML stack data to FILEHANDLE or filename in prescribed format

    $view->render();

=cut

sub render {
    my ($self) = @_;

    my $HANDLE=$self->handle;

    $self->header($HANDLE);

    my $light_state=$self->light();
    if ($light_state){
	$self->body($HANDLE, light => $light_state);
    } else {
	$self->body($HANDLE, show => $self->show());
    }

    $self->footer($HANDLE);
}


##########
# update #
##########

=head1 update

Will update the view when notified by the Model. This method is
called from within the Model object

    $view->update();

=cut

sub update {
    my ($self)=@_;

    $self->render();
}

########
# show #
########

=head1 show

Get/Set method tells view whether to render patch, tiling or crease pattern.

    $state=$view->show('patch');
    $state=$view->show('tiling');
    $state=$view->show('creases');

Returns current value


=cut

sub show {
    my ($self,$state)=@_;

    if (defined $state){
	$self->{'show'}=$state;
    } else {
	$state=$self->{'show'};
    }
    if (! $state){
	die "In View::show state is not defined.  Bad!";
    }

    return $state;
}


################
# show_centres #
################

=head1 show_centres

Get/Set method tells view whether centres of tiles are to be rendered or not

To switch centres on

    $state=$view->show_centres(1);

or to switch centres off

    $state=$view->show_centres(0);

=cut

sub show_centres {
    my ($self,$state)=@_;

    if (defined $state and $state == 0){
	$self->{'centres'}=$state;
    } elsif ($state == 1){
	$self->{'centres'}=$state;
    } else {
	$state=$self->{'centres'};
    }
    return $state;
}


########
# fill #
########

=head1 fill

Get/Set method tells view whether to fill Adults and Babies

To fill polygons

    $state=$view->fill(1);

or to switch fill off

    $state=$view->fill(0);

=cut

sub fill {
    my ($self,$state)=@_;

    if (defined $state and  $state == 0){
	$self->{'fill'}=$state;
    } elsif ($state == 1){
	$self->{'fill'}=$state;
    } else {
	$state=$self->{'fill'};
    }
    return $state;
}

#########
# light #
#########

=head1 light

Get/Set method tells view whether to show light pattern or not

To show light pattern

    $view->light($resolution);

or to switch off

    $view->fill(0);

=cut

sub light {
    my ($self,$state)=@_;

    if (! defined $state){
	$state=$self->{'light'};
	return $state;
    }
    #warn "Setting light state to $state";

    if (defined $state){
	$self->{'light'}=$state;
    } else {
	die "In View::light argument must be a number not [$state]";
    }

    return $state;
}





1;  # says use was ok

__END__

